#!/usr/bin/python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
Scanner driver administration utility.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"

import sys

import locale
locale.setlocale(locale.LC_ALL, '')
expryes = locale.nl_langinfo(locale.YESEXPR)
exprno = locale.nl_langinfo(locale.NOEXPR)

import re
regexyes = re.compile(locale.nl_langinfo(locale.YESEXPR))
regexno = re.compile(locale.nl_langinfo(locale.NOEXPR))

import logging
# Log Levels
LOGLEVELS = {
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warning': logging.WARNING,
    'error': logging.ERROR,
    'critical': logging.CRITICAL
}

def __findchr(regex, defchr):
  r = ""
  for c in xrange(ord('a'), ord('z')+1):
    if regex.match(chr(c)):
      r = r + chr(c)
  if len(r) == 0:
    return defchr
  return r

KEY_YES = __findchr(regexyes, 'y')
KEY_NO = __findchr(regexno, 'n')

import os
from subprocess import *
from gettext import dgettext 
import exceptions
import time
import socket

TEXT_DOMAIN="okimfpdrv"
def _I18N(text):
  return dgettext(TEXT_DOMAIN, text)

TITLE=_I18N("Network Scanner Utility")
MSG_OP_ADDDEV=_I18N("Add Network Scanner")
MSG_OP_DELDEV=_I18N("Delete Network Scanner")
MSG_INPUT_OPE_NO=_I18N("Please select option")
MSG_SEARCH_YN=_I18N("Searching for network scanner . . .") 
MSG_INPUT_BADR=_I18N("Broadcast address used in search")
MSG_SEARCHING=_I18N("Searching...")
MSG_NOTFOUND=_I18N("Not found")
MSG_MODELDATA_NOTFOUND=_I18N("Model Data Not found.")
MSG_INPUT_DEV_NO=_I18N("Please select scanner")
MSG_INPUT_MODEL_NO=_I18N("Please select the model of the scanner")
MSG_INPUT_IPADDR=_I18N("Please enter scanner IP Address")
MSG_INPUT_MAC=_I18N("Please enter scanner MAC Address")
MSG_INPUT_NAME=_I18N("Please enter scanner name")
MSG_INPUT_PORT=_I18N("Please enter scanner port #")
MSG_INPUT_PCNAME=_I18N("Please enter PC Domain Name")
MSG_INPUT_PCIPADDR=_I18N("Please enter PC IP Address")
MSG_INPUT_PCPORT=_I18N("Please enter PC port #")
MSG_CONFIRM_ADDDEV=_I18N("Scanner added . . .")
MSG_CONFIRM_DELDEV=_I18N("Scanner deleted . . .")
MSG_CONFIRM=_I18N("Proceed?")
MSG_PROG_ADD_SUCCESS=_I18N("* Added a network scanner on this PC.")
MSG_PROG_ADD_MFP_SUCCESS=_I18N("* Added this PC on a MFP.")
MSG_PROG_DEL_SUCCESS=_I18N("* Deleted the designated network scanner from this PC.")
MSG_PROG_DEL_MFP_SUCCESS=_I18N("* Deleted this PC information from the MFP.")
MSG_INV_VAL=_I18N("Invalid input.")
MSG_OUT_OF_RANGE=_I18N("Out of range.")
MSG_STR_TOO_LONG=_I18N("Too long. less than or equal to %d characters.")
MSG_INPUT_DELDEV_NO=MSG_INPUT_DEV_NO
MSG_NO_REGDEV=_I18N("Registered scanner not exist")
MSG_CONFFILE_NOEXT=_I18N("Configuration file not exist")
MSG_PERM_DENIED=_I18N("Permission denied")
MSG_ENT_EXIST=_I18N("IP Address or Scanner name already registered")
#MSG_SUCCESS=_I18N("Success.")
MSG_FAIL=_I18N("Failed.")
#MSG_LEAVE=_I18N("Bye")
#MSG_QUIT=_I18N("Quit")
MSG_INSTALL_ERROR=_I18N("Installation error")
MSG_NAME_IP=_I18N("IP Address")
MSG_NAME_MAC=_I18N("MAC Address")
MSG_NAME_MODEL=_I18N("Model name")
MSG_NAME_SCANNERNAME=_I18N("Scanner name")
MSG_NAME_SCANNERPORT=_I18N("Scanner port #")
MSG_NAME_PCNAME=_I18N("PC Domain Name")
MSG_NAME_PCIP=_I18N("PC IP Address")
MSG_NAME_PCPORT=_I18N("PC port #")
MSG_ADDABOOK_FAIL=_I18N("Could not add your PC to the MFP.")
MSG_DELABOOK_FAIL=_I18N("Could not delete your PC from the MFP.")
MSG_RETRY=_I18N("Try again?")

LH_IPADDR=MSG_NAME_IP
LH_MAC=MSG_NAME_MAC
LH_MODEL=MSG_NAME_MODEL
LH_NAME=MSG_NAME_SCANNERNAME


DEF_BROADCAST_ADDR="255.255.255.255"
DEF_SCANNER_PORT=9967
DEF_PC_PORT=9968


YES=1
NO=0
QUIT=-1

PROMPT1=": "
PROMPT2=" (%c/%c) " % (KEY_YES[0], KEY_NO[0])
PROMPT2Y=" ([%c]/%c) " % (KEY_YES[0], KEY_NO[0])
PROMPT2N=" (%c/[%c]) " % (KEY_YES[0], KEY_NO[0])

CMD_SCAND_INITSCR="/usr/libexec/okimfpdrv/scand.sh"
CMD_BASEDIR="/usr/libexec/okimfpdrv/"
CMD_LS_NAME="lsscndrv"
CMD_ADD_NAME="addlandev"
CMD_DEL_NAME="dellandev"
CMD_ADDABOOK_NAME="addabook"
CMD_DELABOOK_NAME="delabook"
b=False
for p in [CMD_BASEDIR, "./"]:
  CMD_LS=p + CMD_LS_NAME
  CMD_ADD=p + CMD_ADD_NAME
  CMD_DEL=p + CMD_DEL_NAME
  CMD_ADDABOOK=p + CMD_ADDABOOK_NAME
  CMD_DELABOOK=p + CMD_DELABOOK_NAME
  try:
    os.stat(CMD_LS)
    os.stat(CMD_ADD)
    os.stat(CMD_DEL)
    os.stat(CMD_ADDABOOK)
    os.stat(CMD_DELABOOK)
    b=True
    break
  except:
    pass
if b == False:
  print MSG_INSTALL_ERROR
  sys.exit()



def unicode_dispsize(unistr):
  import unicodedata
  dictlen = {'Na':1, 'H':1, 'W':2, 'F':2, 'N':1, 'A':1}
  l = 0
  for uc in unistr.decode('utf8'):
    try:
      lc = unicodedata.east_asian_width(uc)
      l = l + dictlen[lc]
      logging.debug("lc=%s, size=%d" % (lc, dictlen[lc]))
    except KeyError,e:
      l = l + 1
    except Exception,e:
      logging.debug(e)
      logging.debug("unistr=" + unistr + ", uc=" + uc)
  logging.debug("unicode_dispsize('%s') => %d" % (unistr, l))
  return l


class Quit (exceptions.Exception):
  pass


class GetListByCmd:
  def __init__(self, proccmd, exp):
    self.proccmd = proccmd
    self.regex = re.compile(exp)
    self.list = []

  def collectList(self):
    self.list = []
    p=Popen(self.proccmd, shell=False, bufsize=1024, stdin=PIPE, stdout=PIPE, stderr=PIPE, close_fds=True);
    while True:
      line = p.stdout.readline()
      if line == '':
        break
      line = line.strip()
      if len(line) == 0:
        continue
      m = self.regex.search(line, 0)
      if m != None:
        g = m.groups()
        self.list.append(g)
    sts = p.wait()
    if sts != 0:
      return -sts
    self.list.sort()
    return len(self.list)

  def get(self, index):
      return self.list[index]  

  # fmt : ((index, name, len),...)
  def printlist(self, fmt):
    n = 0
    buf = ""
    for f in fmt:
      if len(buf) > 0:
        buf = buf + "  "
      if f[2] == None:
        fs = "%s"
      else:
        l = f[2]
        lt = unicode_dispsize(f[1])
        if l <= lt:
          l = lt
      buf = buf + f[1]+("".ljust(l - lt, ' '))
    print "     " + buf
    for d in self.list:
      n = n + 1
      buf = ""
      for f in fmt:
        if len(buf) > 0:
          buf = buf + "  "
        if f[2] == None:
          buf = buf + d[f[0]]
        else:
          l = f[2]
          lt = unicode_dispsize(f[1])
          if l <= lt:
             l = lt
          buf = buf + d[f[0]]+("".ljust(l - unicode_dispsize(d[f[0]]), ' '))
      print ("%2d)  " % n) + buf   

  def selectList(self, fmt, msg):
        print
        self.printlist(fmt)
        print
        selno = input_int(msg, 1, len(self.list), None)
        return self.list[selno - 1]


def execCmd(cmd):
  p=Popen(cmd, shell=False, bufsize=1024, stdin=PIPE, stdout=PIPE, stderr=PIPE, close_fds=True)
  err=0
  while True:
    line = p.stderr.readline()
    if line == '':
      break
  sts = p.wait()
  #print "Ststus: " + str(sts) #DBG
  return sts



def input_yn(msg, defval):
  global regexyes
  global regexno
  while True:
    m = msg
    if defval == YES:
      m = m + PROMPT2Y
    elif defval == NO:
      m = m + PROMPT2N
    else:
      m = m + PROMPT2
    yn = raw_input(m)
    yn = yn.strip()
    if len(yn) > 0:
      if not regexyes.match(yn) == None:
        return YES
      elif not regexno.match(yn) == None:
        return NO
      elif yn == 'q':
        raise Quit()
    else:
      if defval == YES or defval == NO:
          return defval


def input_str(msg, defval=None, maxlen=0):
  m = msg
  if defval != None:
    m = m + "[%s]"%defval
  m = m + PROMPT1
  s = ""
  while True:
    s = raw_input(m)
    s = s.strip()
    l = len(s)
    if l > 0:
      if maxlen > 0:
          if l <= maxlen:
            break
          else:
            print MSG_STR_TOO_LONG % (maxlen)
      else:
        break
    elif l == 0:
      if defval != None:
          return defval
  return s


def input_int(msg, range_min, range_max, defval):
  m = msg
  if defval != None:
    m = m + "[%d]"%defval
  m = m + PROMPT1
  while True:
    s = raw_input(m)
    s = s.strip()
    if len(s) > 0:
      if s == "q":
         raise Quit()
      try:
        v = int(s)
        if range_min != None and range_min > v:
          print MSG_OUT_OF_RANGE
          continue
        if range_max != None and range_max < v:
          print MSG_OUT_OF_RANGE
          continue
        return v        
      except ValueError:
        print MSG_INV_VAL
    else:
      if defval != None:
        return defval


def check_ipv4str(s):
  v = s.split('.')
  if len(v) != 4:
    return False
  try:
    for i in v:
      vv = int(i)
      if vv < 0 or 255 < vv:
        return False
    return True
  except ValueError:
    return False


def check_macstr(s):
  v = s.split(':')
  if len(v) != 6:
    return False
  try:
    for i in v:
      vv = int("0x"+i, 16)
      if vv < 0 or 255 < vv:
        return False
    return True
  except ValueError:
    return False


def input_ipaddr(msg, defval, retry):
  m = msg
  if defval != None:
    m = m + "[" + defval + "]"
  m = m + PROMPT1
  while retry:
    s = raw_input(m)
    s = s.strip()
    if s == "q":
      raise Quit()
    if len(s) == 0:
      if defval == None:
        continue
      return defval
    if check_ipv4str(s):
      return s
    else:
      print MSG_INV_VAL
  return None


def input_mac(msg, defval, retry):
  m = msg
  if defval != None:
    m = m + "[" + defval + "]"
  m = m + PROMPT1
  while retry:
    s = raw_input(m)
    s = s.strip()
    if s == "q":
      raise Quit()
    if len(s) == 0:
      if defval == None:
        continue
      return defval
    if check_macstr(s):
      return s
    else:
      print MSG_INV_VAL
  return None


def select_device(selmsg):
  # input broadcast address
  addr = input_ipaddr(MSG_INPUT_BADR, DEF_BROADCAST_ADDR, True)
  if addr == None:
    return None
  
  # discover device
  print MSG_SEARCHING
  g = GetListByCmd([CMD_LS, "-d", addr], "([^ ].+[^ ])[ ]+([0-9\.]+)[ ]+([0-9a-fA-F\:]+)") 
  rc = g.collectList()
  if rc == 0:
    print MSG_NOTFOUND
    return None
  elif rc < 0:
    rc = -rc
    if rc == 252: # config file not exist
      print MSG_CONFFILE_NOEXT;
    elif rc == 253: # permission denied
      print MSG_PERM_DENIED;
    elif rc == 254: # unknown error
      print MSG_FAIL
    elif rc == 255: # parameter error(usage)
      print MSG_FAIL
    else:
      print MSG_FAIL
    return None
  # select device
  listfmt = [
    [1, LH_IPADDR, 15],
    [0, LH_MODEL, 15],
    [2, LH_MAC, 20]
  ]
  return g.selectList(listfmt, selmsg)


# Retrieve Device Info.
# @param[in] uniaddr MFP IP Address
# @return (ip, mac, model)
def retrieve_device(uniaddr):
  # discover device
  g = GetListByCmd([CMD_LS, "-d", uniaddr], "([^ ].+[^ ])[ ]+([0-9\.]+)[ ]+([0-9a-fA-F\:]+)") 
  rc = g.collectList()
  if rc == 0:
    # Not found
    return None
  elif rc < 0:
    return None
  
  d = g.get(0)
  return (d[1], d[2], d[0])


def select_model(selmsg):
  # collect model list
  g = GetListByCmd([CMD_LS, "-m"], "(.+)[ ]+([0-9a-fA-F]{4}:[0-9a-fA-F]{4})") 
  rc = g.collectList()
  if rc == 0:
    print MSG_MODELDATA_NOTFOUND
    return None
  elif rc < 0:
    rc = -rc
    if rc == 252: # config file not exist
      print MSG_CONFFILE_NOEXT;
    elif rc == 253: # permission denied
      print MSG_PERM_DENIED;
    elif rc == 254: # unknown error
      print MSG_FAIL
    elif rc == 255: # parameter error(usage)
      print MSG_FAIL
    else:
      print MSG_FAIL
    return None
  # select model
  listfmt = [
    [0, LH_MODEL, 15]
  ]
  result = g.selectList(listfmt, selmsg)
  return result[0]



def select_regdev(selmsg):
  # collect registered device
  g = GetListByCmd([CMD_LS, "-r"], "([0-9\.]+)[ ]+[\"]([^ ].*[^ ])[\"][ ]+[\"](.+)[\"][ ]+([0-9a-fA-F\:]+)[ ]+([0-9]+)[ ]+([^ ]+)[ ]+([0-9\.]+)[ ]+([0-9]+)") 
  rc = g.collectList()
  if rc == 0:
    print MSG_NO_REGDEV
    return None
  elif rc < 0:
    rc = -rc
    if rc == 252: # config file not exist
      print MSG_CONFFILE_NOEXT;
    elif rc == 253: # permission denied
      print MSG_PERM_DENIED;
    elif rc == 254: # unknown error
      print MSG_FAIL
    elif rc == 255: # parameter error(usage)
      print MSG_FAIL
    else:
      print MSG_FAIL
    return None
  # select device
  listfmt = [
    [0, LH_IPADDR, 15],
    [1, LH_MODEL, 15],
    [2, LH_NAME, None]
  ]
  return g.selectList(listfmt, selmsg)


def print_op_list():
  print " 1)  "+MSG_OP_ADDDEV
  print " 2)  "+MSG_OP_DELDEV


def select_op():
  print
  print_op_list()
  print
  while True:
    selno = input_int(MSG_INPUT_OPE_NO, None, None, None)
    if selno!=None and 1<=selno and selno<=2:
      return selno


def input_regster_device():
  ip = ""
  model = ""
  name = ""
  port = DEF_SCANNER_PORT
  pc_port = DEF_PC_PORT

  yn = input_yn(MSG_SEARCH_YN+PROMPT1, YES)
  if yn == YES:
    d = select_device(MSG_INPUT_DEV_NO)
    if d == None:
      return None
    else:
      ip = d[1]
      model = d[0]
      mac = d[2]
  elif yn == NO:
    ip = input_ipaddr(MSG_INPUT_IPADDR, None, True)
    if ip == None:
      return None
    rd = retrieve_device(ip)
    if rd != None:
      model = rd[2]
      mac = rd[1]
    else:
      mac = input_mac(MSG_INPUT_MAC, None, True)
      if mac == None:
        return None
      model = select_model(MSG_INPUT_MODEL_NO)
      if model == None:
        return None
  else:
    return None

  name = input_str(MSG_INPUT_NAME, None, 31)
  port = input_int(MSG_INPUT_PORT, 0, 65535, DEF_SCANNER_PORT)
  
  pc_name = input_str(MSG_INPUT_PCNAME, socket.gethostname(), 16)
  pc_ip = None
  try:
    wkaddr = socket.gethostbyaddr(pc_name)
    pc_ip = wkaddr[2][0]
  except:
      pass
  pc_ip = input_ipaddr(MSG_INPUT_PCIPADDR, pc_ip, True)
  pc_port = input_int(MSG_INPUT_PCPORT, 0, 65535, DEF_PC_PORT)

  return (ip, mac, model, name, port, pc_name, pc_ip, pc_port)

def addscanner():
  dev = input_regster_device()
  if dev != None:
    # confirm
    print
    print MSG_CONFIRM_ADDDEV
    print 
    print "%s : %s" % (MSG_NAME_IP, dev[0])
    print "%s : %s" % (MSG_NAME_MAC, dev[1])
    print "%s : %s" % (MSG_NAME_MODEL, dev[2])
    print "%s : %s" % (MSG_NAME_SCANNERNAME, dev[3])
    print "%s : %d" % (MSG_NAME_SCANNERPORT, dev[4])
    print "%s : %s" % (MSG_NAME_PCNAME, dev[5])
    print "%s : %s" % (MSG_NAME_PCIP, dev[6])
    print "%s : %d" % (MSG_NAME_PCPORT, dev[7])
    print
    yn = input_yn(MSG_CONFIRM, NO)
    if yn == YES:
      print
      rc = execCmd([CMD_ADD, "-name", dev[3], "-model", dev[2], "-mac", dev[1], "-ip", dev[0], "-port", str(dev[4]), "-pc-name", dev[5], "-pc-ip", dev[6], "-pc-port", str(dev[7])])
      if rc == 0:
        os.system(CMD_SCAND_INITSCR + " reload")
      else:
        if rc == 250: # unknown model
          print MSG_FAIL
        elif rc == 251: # already registered
          print MSG_ENT_EXIST;
        elif rc == 252: # config file not exist
          print MSG_CONFFILE_NOEXT;
        elif rc == 253: # permission denied
          print MSG_PERM_DENIED;
        elif rc == 254: # unknown error
          print MSG_FAIL
        elif rc == 255: # parameter error(usage)
          print MSG_FAIL
        else:
          print MSG_FAIL
        return
      print MSG_PROG_ADD_SUCCESS
    
      while True:
        rc = execCmd([CMD_ADDABOOK, "-ip", dev[0], "-pc-name", dev[5], "-pc-ip", dev[6], "-pc-port", str(dev[7])])
        if rc == 0:
          print MSG_PROG_ADD_MFP_SUCCESS
          #print MSG_SUCCESS
          break
        else:
          print MSG_ADDABOOK_FAIL+" (code=%d)" % rc
          yn = input_yn(MSG_RETRY, YES)
          if yn == NO:
              break
        
      return
  print
  #print MSG_LEAVE


def delscanner():
  dev = select_regdev(MSG_INPUT_DELDEV_NO)
  if dev != None:
    # confirm
    print
    print MSG_CONFIRM_DELDEV
    print 
    print "%s : %s" % (MSG_NAME_IP, dev[0])
    print "%s : %s" % (MSG_NAME_MAC, dev[3])
    print "%s : %s" % (MSG_NAME_MODEL, dev[1])
    print "%s : %s" % (MSG_NAME_SCANNERNAME, dev[2])
    print "%s : %s" % (MSG_NAME_SCANNERPORT, dev[4])
    print "%s : %s" % (MSG_NAME_PCNAME, dev[5])
    print "%s : %s" % (MSG_NAME_PCIP, dev[6])
    print "%s : %s" % (MSG_NAME_PCPORT, dev[7])
    print
    yn = input_yn(MSG_CONFIRM, NO)
    if yn == YES:
      print
      rc = execCmd([CMD_DEL, "-ip", dev[0]])
      if rc == 0:
        os.system(CMD_SCAND_INITSCR + " reload")
      else:
        if rc == 251: # IP not exist (リスト選択のため、基本的には発生しないので、単にエラーとする)
          print MSG_FAIL
        elif rc == 252: # config file not exist
          print MSG_CONFFILE_NOEXT;
        elif rc == 253: # permission denied
          print MSG_PERM_DENIED;
        elif rc == 254: # unknown error
          print MSG_FAIL
        elif rc == 255: # parameter error(usage)
          print MSG_FAIL
        else:
          print MSG_FAIL
        return
      print MSG_PROG_DEL_SUCCESS
    
      while True:
        rc = execCmd([CMD_DELABOOK, "-ip", dev[0], "-pc-name", dev[5]])
        if rc == 0:
          # 0:Success
          print MSG_PROG_DEL_MFP_SUCCESS
          #print MSG_SUCCESS
          break
        if rc == 251:
          # 251: Not Found
          print MSG_PROG_DEL_MFP_SUCCESS
          #print MSG_SUCCESS
          break
        else:
          print MSG_DELABOOK_FAIL+" (code=%d)" % rc
          yn = input_yn(MSG_RETRY, YES)
          if yn == NO:
              break
        
      return
  print
  #print MSG_LEAVE



def main():
  print TITLE
  try:
    opno = select_op()
    if opno == 1:
      addscanner()
    elif opno == 2:
      delscanner()
    else:
      #print MSG_LEAVE
      pass
    #time.sleep(2)
  except KeyboardInterrupt, e:
    print ""
  except Quit, e:
    #print MSG_QUIT
    print ""

if __name__ == "__main__":
  logging.basicConfig(format="%(asctime)s %(message)s")
  if len(sys.argv) > 1:
    rl = logging.getLogger()
    rl.setLevel(LOGLEVELS[sys.argv[1]])
  
  main()

