#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
This program is configure execution application after scanning.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"
__icon__ = "/usr/libexec/okimfputl/images/exeicon48x48.png"

# Directories
DRV_BASEDIR="/usr/libexec/okimfpdrv/"
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(DRV_BASEDIR)
sys.path.append(UTL_BASEDIR)
sys.path.append("../driver/.libs/")#DEBUG
sys.path.append(".")

import locale
locale.setlocale(locale.LC_ALL, '')

import os
import os.path
import pygtk
pygtk.require('2.0')
import gtk
import gtk.gdk
import gobject

import time
import exceptions
import logging

from guicom import *
from defs import *
from mfpcfgfile import *




class AppFile(ItemFile):
    def __init__(self):
        ItemFile.__init__(self)
        
    def _proc_get_text(self, text):
        if not text == None and len(text) > 0:
            return text.split()[0]
        return text
        
    def _proc_set_text(self, text):
        if not text == None and len(text) > 0:
            return text + " " + EXEC_OPT_FILE
        return text


class AppConfigDialog(CommonPalletWindow):
    
    def onOk(self):
        # check configuration title
        do_check = True
        do_warn = True
        do_warn_cne = False
        
        name = self._name.get_text()
        cmd = self._appfile.get_path()
        
        if name == None:
            error_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_EMPTY_APPNAME)
            return False
        if cmd == None:
            error_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_EMPTY_APPCMD)
            return False
        
        name = name.strip()
        cmd = cmd.strip()
        
        if len(name) == 0:
            error_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_EMPTY_APPNAME)
            return False
        if len(cmd) == 0:
            error_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_EMPTY_APPCMD)
            return False
        
        app = cmd.split()[0]
        
        if not self._mode_append:
            # edit mode
            if self._in_name == name:
                # not changed
                do_check = False
        else:
            # append mode
            do_warn = False
        
        try:
            os.stat(app) #check command exists
        except Exception, e:
            do_warn_cne = True
        
        if do_check:
            #print "check '%s'" % name
            if self._config.has_section(name):
                # already exists
                error_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_APPNAME_EXISTED, name)
                return False

        if do_warn_cne:
            rc = confirm_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_CONFIRM_APP_NOTEXIST % (app))
            if not rc == gtk.RESPONSE_OK:
                return False

        if do_warn:
            rc = confirm_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_CONFIRM_MOD_APP)
            if not rc == gtk.RESPONSE_OK:
                return False
        return True

    def onCancel(self):
       return True
        
    def __init__(self, parent, config, title=None, command=None, append=False):
        CommonPalletWindow.__init__(self, TITLE_APPCONF_DLG, parent)
        # Set the help document URI for this window
        self.setHelpUri("file://"+UTLDOC_DIR+"/appconfig.html")
        
        self._config = config
        self._in_name = title
        self._in_cmdstr = command
        self._mode_append = append
        
        self._name = gtk.Entry()
        if not title == None:
            self._name.set_text(title)
        self.config_name = ItemHBox(LABEL_APPNAME, self._name, True, padding=6)
        self.vbox.pack_start(self.config_name,False, padding=4)
        
        self._appfile = AppFile()
        self._appfile.set_path(command)
        self.command = ItemHBox(LABEL_COMMAND_STRING, self._appfile, True, padding=6)
        self.vbox.pack_start(self.command,False, padding=4)
        
        self.add_button(BUTTON_OK, gtk.RESPONSE_OK)
        self.add_button(BUTTON_CANCEL, gtk.RESPONSE_CANCEL)
        
        self.resize(500, 60)
        self.show_all()
    
    def get_title(self):
        return self._name.get_text()
    
    def get_command(self):
        return self._appfile.get_path()











class ItemApplication(gtk.HBox):
    
    """ItemApplication"""
    
    def __init__(self):
        gtk.HBox.__init__(self)
        self.config = ApplicationConfig()
        self.config.load()
        la_app_name = self.config.sections_ordered()
        self.cb = build_ComboBox(la_app_name)
        self.pack_start(self.cb, True)
        self.btn1 = gtk.Button(BUTTON_ADD)
        self.btn2 = gtk.Button(BUTTON_EDIT)
        self.btn3 = gtk.Button(BUTTON_DELETE)
        self.pack_end(self.btn1, False)
        self.pack_end(self.btn2, False)
        self.pack_end(self.btn3, False)
        self.cb.connect('changed', self._changed)
        self.btn1.connect('clicked', self._add)
        self.btn2.connect('clicked', self._edit)
        self.btn3.connect('clicked', self._del)
        if len(la_app_name) == 0:
            self.btn2.set_sensitive(False)
            self.btn3.set_sensitive(False)
        self.__supress = False
    
        
    def get_active(self):
        if self.cb.get_active():
            return self.cb.get_active_text()
        
        model = self.cb.get_model()
        ite = model.get_iter_root()
        if ite != None:
            return model.get_value(ite, 0)
        return None
    
    def set_active(self, title):
        model = self.cb.get_model()
        ite = model.get_iter_root()
        while ite != None:
            text = model.get_value(ite, 0)
            #print text
            if text == title:
                self.cb.set_active_iter(ite)
                return True
            ite = model.iter_next(ite)
        return False
        
    def remove_active(self):
        actidx = self.cb.get_active()
        actite = self.cb.get_active_iter()
        model = self.cb.get_model()
        nite = model.iter_next(actite)
        if nite != None:
            self.cb.remove_text(actidx)
            self.cb.set_active(actidx)
        else:
            self.cb.remove_text(actidx)
            nidx = actidx - 1
            if nidx >= 0:
                self.cb.set_active(nidx)
           
    def check_values(self):
        if self.__supress:
            return
        active = self.cb.get_active_text()
        if active == None:
            #configuration deleted?
            return
        cfg = self.config.gets(active)
        if cfg == None:
            logging.error("active=%s" % (active))
            return
        cmd = cfg[OPT_COMMAND]
        if cmd != None:
            cmd = cmd.strip()
            if len(cmd) > 0:
                app = cmd.split()[0]
                try:
                    os.stat(app)
                except Exception,e:
                    warning_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_NOTEXIST_APPCMD, active)
                return
        #illegal configuration
        warning_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_INV_APPCFG, active)
        
    def _changed(self, combobox, *args):
        w = retrieve_window(self)
        if w == None:
            return
        if w.is_shown():
            self.check_values()
        
    def _add(self, btn, *args):
        dlg = AppConfigDialog(retrieve_window(self), self.config, title=S_UNTITLED, append=True)
        rc = dlg.do_modal()
        if rc == gtk.RESPONSE_OK:
            title = dlg.get_title()
            command = dlg.get_command()
            logging.debug("ADD: title=%s, command=%s" % (title, command))
            self.config.add(title, {OPT_COMMAND:command})
            self.config.save()
            try:
                self.cb.append_text(title)
            except:
                pass
            
            try:
                self.__supress = True
                self.set_active(title)
            finally:
                self.__supress = False
            
            self.btn2.set_sensitive(True)
            self.btn3.set_sensitive(True)
        dlg.destroy()
        
    def _del(self, btn, *args):
        active = self.cb.get_active_text()
        rc = confirm_dialog(retrieve_window(self), TITLE_APPCONF_DLG, MSG_CONFIRM_DEL_APP, active)
        if rc == gtk.RESPONSE_OK:
            self.config.delete(active) #delete selected config
            self.config.save()
            self.remove_active()
            if self.cb.get_active_text() == None:
                self.btn2.set_sensitive(False)
                self.btn3.set_sensitive(False)
         
    def _edit(self, btn, *args):
        active = self.cb.get_active_text()
        copts = self.config.gets(active)
        dlg = AppConfigDialog(retrieve_window(self), self.config, title=active, command=copts[OPT_COMMAND])
        rc = dlg.do_modal()
        if rc == gtk.RESPONSE_OK:
            try:
                self.__supress = True
                aidx = self.cb.get_active()
                self.cb.remove_text(aidx)
                title = dlg.get_title()
                command = dlg.get_command()
                if active != title:
                    #change configuration title(change section name)
                    self.config.rename(active, title)
                self.config.add(title, {OPT_COMMAND:command})
                self.config.save()
                self.cb.insert_text(aidx, title)
                self.cb.set_active(aidx)
            finally:
                self.__supress = False
        dlg.destroy()







if __name__ == "__main__":
    logging.basicConfig(format=LOGFORMAT)
    if len(sys.argv) > 1:
        rl = logging.getLogger()
        rl.setLevel(LOGLEVELS[sys.argv[1]])
    
    dlg = CommonPalletWindow(title=TITLE_APPCONF_DLG, buttons=(BUTTON_CLOSE, gtk.RESPONSE_CLOSE))
    dlg.vbox.pack_start(ItemApplication())
    dlg.set_icon_from_file(__icon__)
    dlg.do_modal()
    dlg.destroy()



