#!/usr/bin/python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
Definitions
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"

# Directories
DRV_BASEDIR="/usr/libexec/okimfpdrv/"
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
import os
import os.path
import re
from subprocess import *
from gettext import dgettext 
import exceptions
import time
import socket
import locale
import logging




TEXT_DOMAIN="okimfputl"
def _I18N(text):
  return dgettext(TEXT_DOMAIN, text)




def get_lang_dir(basedir, deflang='en'):
    """basedirのサブディレクトリに、現在のロケールの言語に対応するディレクトリが存在していれば、
    そのディレクトリの絶対パスとサブディレクトリ名（言語を表す文字列）を返す。
    現在のロケールのサブディレクトリが無い場合、deflangの指定があれば、指定されたサブディレクトリを検索する。
    それも存在していなければbasedirとNoneを返す。"""
    langdir = None
    abd = os.path.abspath(os.path.expandvars(os.path.expanduser(basedir)))
    bd = abd + os.path.sep
    (lang, enc) = locale.getdefaultlocale()
    if not lang == None:
        langdir = bd + lang
        if os.path.isdir(langdir):
            return (langdir, lang)
        
        lang = lang.split('_')[0]
        langdir = bd + lang
        if os.path.isdir(langdir):
            return (langdir, lang)
        
    if deflang != None and len(deflang)>0:
        lang = deflang
        langdir = bd + lang
        if os.path.isdir(langdir):
            return (langdir, lang)
    
    return (abd, None)




lang_dir = lambda d : get_lang_dir(d)[0]




def check_command(cmdname):
    """
    check a command specified at the parameter 'cmdname' exists.
    If a command exists, this function return a real pass.
    """
    DIRS=["/usr/bin", "/usr/sbin", "/bin", "/sbin", "/usr/local/bin", "/usr/local/sbin"]
    for d in DIRS:
        try:
            p = os.path.realpath(d + os.sep + cmdname)
            s = os.stat(p)
            if os.path.isfile(p):
                return p
        except:
            pass
    return None




# Driver Document Directory
DRVDOC_BASEDIR = DRV_BASEDIR+"doc/"
DRVDOC_DIR = lang_dir(DRVDOC_BASEDIR)

# Utility Document Directory
UTLDOC_BASEDIR = UTL_BASEDIR+"doc/"
UTLDOC_DIR = lang_dir(UTLDOC_BASEDIR)


# User login home directory
USER_HOMEDIR=os.path.expanduser("~")

# User special dirs dict.
USER_DIRS = {}
try:
    import glib
    names = {
        glib.USER_DIRECTORY_DESKTOP : 'USER_DIRECTORY_DESKTOP',
        glib.USER_DIRECTORY_DOCUMENTS : 'USER_DIRECTORY_DOCUMENTS',
        glib.USER_DIRECTORY_DOWNLOAD : 'USER_DIRECTORY_DOWNLOAD',
        glib.USER_DIRECTORY_MUSIC : 'USER_DIRECTORY_MUSIC',
        glib.USER_DIRECTORY_PICTURES : 'USER_DIRECTORY_PICTURES',
        glib.USER_DIRECTORY_PUBLIC_SHARE : 'USER_DIRECTORY_PUBLIC_SHARE',
        glib.USER_DIRECTORY_TEMPLATES : 'USER_DIRECTORY_TEMPLATES',
        glib.USER_DIRECTORY_VIDEOS : 'USER_DIRECTORY_VIDEOS'
    }
    for id in names.keys():
        try:
            d = glib.get_user_special_dir(id)
            if not d == None:
                USER_DIRS.update({names[id]:d})
        except:
            pass
except:
    pass


def get_user_special_dir(name):
    """Get user special directory."""
    global USER_DIRS
    global USER_HOMEDIR
    try:
        return USER_DIRS[name]
    except KeyError, e:
        return USER_HOMEDIR
    



# icon images
ICON_TOFOLDER_IMG_PATH=UTL_BASEDIR + "images/folder.png"
ICON_TOAPP_IMG_PATH=UTL_BASEDIR + "images/app.png"
#ICON_START_IMG_PATH=UTL_BASEDIR + "images/start.png"
#ICON_STOP_IMG_PATH=UTL_BASEDIR + "images/stop.png"
ICON_STATUSICON_IMG_PATH=UTL_BASEDIR + "images/trayicon.png"
ICON_EXEICON48x48_IMG_PATH=UTL_BASEDIR + "images/exeicon48x48.png"


# Icon configuration values
BUILTIN_ICON_TOFOLDER="(builtin-icon-folder)"
BUILTIN_ICON_TOAPP="(builtin-icon-app)"


# Parameter
EXEC_OPT_DIR="$DIR"
EXEC_OPT_FILE="$FILE"


# File extensions
EXT_PNM="pnm"
EXT_PPM="ppm"   #PNM:color
EXT_PGM="pgm"   #PNM:gray
EXT_PBM="pbm"   #PNM:black&white
EXT_PNG="png"
EXT_JPG="jpeg"
EXT_TIF="tiff"
EXT_PDF="pdf"


# Simple words
# Please do not use it directly
S_NONE=_I18N("None")
S_SCAN=_I18N("Scan")
S_ADD=_I18N("Add")
S_DELETE=_I18N("Delete")
S_MODIFY=_I18N("Modify")
S_EDIT=_I18N("Edit")
S_OK=_I18N("OK")
S_CANCEL=_I18N("Cancel")
S_CLOSE=_I18N("Close")
S_QUIT=_I18N("Quit")
S_APPLY=_I18N("Apply")
S_UNTITLED=_I18N("untitled")    #名称未設定

# ScanToID
S_SCAN_TO_APP=_I18N("Scan To Application")
S_SCAN_TO_FOLDER=_I18N("Scan To Folder")


# Messages
TITLE_SCAN=S_SCAN
TITLE_SCAN_TOOL=_I18N("Scan Tool")
TITLE_SCAN_CONFIG=_I18N("Scan Configuration")
TITLE_TRAYICON_APPLET=_I18N("MFP Scanner applet")
TITLE_PUSH_ASSISTANT=_I18N("PushScan Assistant")
TITLE_PUSH_CONFIG=_I18N("PushScan Configuration")
TITLE_UTL_CONFIG=_I18N("Configurations")
TITLE_APPCONF_DLG=_I18N("Application Configuration")
TITLE_SELAPP_FCDLG=_I18N("Select Application")
TITLE_DIRCHOOSER_DLG=_I18N("Select Folder")
TITLE_FILECHOOSER_DLG=_I18N("Select File")

# Button Label
BUTTON_DDD="..."
BUTTON_OK=S_OK
BUTTON_APPLY=S_APPLY
BUTTON_CANCEL=S_CANCEL
BUTTON_CLOSE=S_CLOSE
BUTTON_ADD=S_ADD
BUTTON_DELETE=S_DELETE
BUTTON_EDIT=S_EDIT
BUTTON_START_SCAN=_I18N("Start scan")
BUTTON_CANCEL_SCAN=_I18N("Cancel scan")

# Values(string)
VALUE_NONE=S_NONE   #pushconfig

# Widget Label
LABEL_APPNAME=_I18N("Application Name") #appconfig
LABEL_COMMAND_STRING=_I18N("Command")  #appconfig
LABEL_PUSH_CONFIG=_I18N("Scan Configuration to use") #pushconfig [Frame Label]
LABEL_PUSH_SCAN_STATUS=_I18N("Scan Status") #pushconfig [Frame Label]
LABEL_SHOW_DIALOG=_I18N("Show Scan Window")   #pushconfig
LABEL_SCAN_TO_APP=S_SCAN_TO_APP  #pushconfig
LABEL_SCAN_TO_FOLDER=S_SCAN_TO_FOLDER    #pushconfig
LABEL_DEVICE=_I18N("Device") #scantool
LABEL_SCANSETTING=_I18N("Scan Setting")    #scantool [Frame Label]
LABEL_OUTPUTSETTING=_I18N("Output Setting")    #scantool [Frame Label]
LABEL_APPSETTING=_I18N("Application Setting")    #scantool [Frame Label]
LABEL_ICONSETTING=_I18N("Icon Setting")    #scantool [Frame Label]
LABEL_NOTIFY_METHOD=_I18N("Notification")   #genconfig [Frame Label]
LABEL_NOTIFY_PYNOTIFY=_I18N("Use Desktop Notification Function") #genconfig
LABEL_NOTIFY_DIALOG=_I18N("Use Message Dialog") #genconfig
LABEL_HELP_VIEWER=_I18N("HELP Viewer")   #genconfig [Frame Label]
LABEL_USE_WEBBROWSER=_I18N("Use the specific Web Browser")   #genconfig
LABEL_WEBBROWSER=_I18N("Web Browser")   #genconfig
LABEL_GENERAL=_I18N("General")      #mfpatray [Config GUI TAB Label]
LABEL_PUSHSCAN=_I18N("PushScan")    #mfpatray [Config GUI TAB Label]

# Message format
FORMAT_PROGRESS_BEGIN=_I18N("Page %(pagenumber)d (Wait for scan to begin...)") #scantool %{'pagenumber':page}
FORMAT_PROGRESS_CANCELED=_I18N("Page %(pagenumber)d (Canceled.)") #scantool %{'pagenumber':page}
FORMAT_PROGRESS_FINISHED=_I18N("Success. (Scanned page %(pagenumber)d)") #scantool %{'pagenumber':page}
FORMAT_PROGRESS_FAIL=_I18N("Fail. (%(detail)s)") #scantool %{'detail':detail-string}
FORMAT_PROGRESS=_I18N("Page %(pagenumber)d (Scanning... %(percentage)s%%)") #scantool %{'pagenumber':page, 'percentage':pregress}
FORMAT_PROGRESS_PAGENUMBER='pagenumber'
FORMAT_PROGRESS_PERCENTAGE='percentage'
FORMAT_PROGRESS_DETAIL='detail'
FORMAT_PROGRESS_CNVFMT=_I18N("Convert image format into '%s'.") #scantool %(image file ext.)
FORMAT_PROGRESS_RUNAPP=_I18N("Start application '%s'.") #scantool %(application preset name)
# Exception message format 
FORMAT_EXCEPTMSG_CMDEXEC_NG=_I18N("Command execution error. command='%s'")  #%(command-string)


#status icon menu
MENUITEM_START_ACCEPT=_I18N("Start _accept PushScan")
MENUITEM_STOP_ACCEPT=_I18N("Stop _accept PushScan")
MENUITEM_NO_EXEC=_I18N("No scanning")
MENUITEM_CANCEL_SCAN=_I18N("_Cancel Scan")
MENUITEM_RUN_SCANTOOL=_I18N("Scan _Tool")
MENUITEM_RUN_PUSHCONF=_I18N("Configure _PushScan")#removed
MENUITEM_RUN_UTLCONF=_I18N("C_onfigurations")
MENUITEM_DRVDOC=_I18N("_Driver Manual")
MENUITEM_HELP=_I18N("_Help")
MENUITEM_QUIT_TRAYICON=_I18N("_Quit")
#scan tool menu bar
MENUITEM_FILE=_I18N("_File")
MENUITEM_FILE_QUIT=MENUITEM_QUIT_TRAYICON
MENUITEM_SCAN=_I18N("_Scan")
MENUITEM_SCAN_START=_I18N("_Start Scan")
MENUITEM_EDIT=_I18N("_Edit")
MENUITEM_EDIT_ADD=_I18N("_Add")
MENUITEM_EDIT_MODIFY=_I18N("_Modify")
MENUITEM_EDIT_DELETE=_I18N("_Delete")


# Notification Messages
MSG_PUSH_ACPT_STA=_I18N("Started PushScan acceptance.")
MSG_ALREADY_REGUSR=_I18N("Acceptance has been already started by %s.")      #%(user-name)
MSG_PUSHSCAN_ACCEPT=_I18N("PushScan(%(id)s) accepted. (%(device)s)")      #%{'id':scantoid, 'device':device-name}
MSG_PUSHSCAN_ACCEPT_ID='id'
MSG_PUSHSCAN_ACCEPT_DEVICE='device'
MSG_PUSHSCAN_CANCELD=_I18N("PushScan canceled. (%s)")   #%(device-name)
MSG_START_SCAN=_I18N("Choose a device to use, and please click 'Start scan' button.")

# Confirmation Message
MSG_CONFIRM_DEL_SCANPRESET=_I18N("Do you want to delete selected scan configuration?")
MSG_CONFIRM_DEL_APP=_I18N("This change influences all scan configurations. Do you want to delete selected Application configuration?")
MSG_CONFIRM_MOD_APP=_I18N("This change influences all scan configurations. Do you want to modify selected Application configuration?")
MSG_CONFIRM_APP_NOTEXIST=_I18N("'%s' not found. Do you want to continue?") #%(command-path)
MSG_CONFIRM_SAVE_CHG=_I18N("Configurations are changed. Do you save changes?")

# Warning/Error Messages
MSG_SCAN_TOOL_QUIT_WRN=_I18N("Please close all 'Scan' windows before quit.")
MSG_QUIT_WRN=_I18N("Please close all tool windows before quit.")
MSG_PRESET_NOT_SELECT=_I18N("Please select scan configuration.")
MSG_ERROR=_I18N("Error.")
MSG_NOT_WORK_DAEMON=_I18N("Scan Service not working.")
MSG_DAEMON_ACCESS_NG=_I18N("Scan Service Error.")
MSG_NO_DEVICES=_I18N("No devices found.")
MSG_SCAN_START_NG=_I18N("Could not start scan.")
MSG_SCAN_ERROR=_I18N("Could not scan.")
MSG_SCANPRESETNAME_EXISTED=_I18N("Specified Scan configuration name has already existed.")
MSG_APPNAME_EXISTED=_I18N("Specified Application Name has already existed.")
MSG_EMPTY_APPNAME=_I18N("Please enter Application Name.")
MSG_EMPTY_APPCMD=_I18N("Please enter Command.")
MSG_NOTEXIST_APPCMD=_I18N("The command appointed by chosen application setting does not exist. Please check this application setting.")
MSG_INV_APPCFG=_I18N("Invalid application setting. Please check this application setting.")
MSG_SCANCFG_REGMAX=_I18N("You can add to up to 8 Scan configuration.")
MSG_SCANCFG_MOD_WRN=_I18N("This configuration is used by PushScan. Can not change the Scan configuration title.")
MSG_SCANCFG_DEL_WRN=_I18N("This configuration is used by PushScan. Can not delete this Scan configuration.")
MSG_PUSHSCAN_INVCFG=_I18N("Invalid PushScan configurtion. (%s)")   #%(scantoid)
MSG_PUSHSCAN_NOTCFG=_I18N("PushScan is not configured. (%s)")   #%(scantoid)
MSG_SCCNF_NOT_EXIST=_I18N("There are no scan settings designated by the PushScan Configuration. (%s)")  #%(preset-name)
MSG_OUTDIR_NOTEXISTS=_I18N("Output directory not exists.")
MSG_FILECONV_NG=_I18N("File format conversion error.")
MSG_APP_EXEC_NG=_I18N("Application execution error.")
MSG_WRN_OFILEFMT=_I18N("Can not save the scan image in '%(ext)s' format. If you needs save image in '%(ext)s' format, please install following package.")  #%{'ext':file-extension}
MSG_WRN_OFILEFMT_EXT='ext'

# Tips
TIPS_ACCEPT_PUSHSCAN=_I18N("Accepting PushScan")
TIPS_NOT_ACCEPT_PUSHSCAN=_I18N("Not accepting PushScan")

# Scan parameter name/value
SPN_PRESETNAME=_I18N("Title")
SPV_DEF_PRESETNAME=S_UNTITLED
SPN_SCANSRC=_I18N("Scan source")
SPV_AUTO=_I18N("Auto")
SPV_FLATBED=_I18N("Flatbed")
SPV_ADF=_I18N("Automatic Document Feeder (ADF)")
SPN_DUPSCAN=_I18N("Duplex Scanning")
SPV_DUPSCAN_NONE=S_NONE
SPV_DUPSCAN_RLB=_I18N("Right and Left Binding")
SPV_DUPSCAN_TB=_I18N("Top Binding")
SPN_SCANMODE=_I18N("Color mode")
SPV_SCANMODE_COLOR=_I18N("Color")
SPV_SCANMODE_GRAY=_I18N("Gray")
SPV_SCANMODE_BW=_I18N("Black and White")
SPN_PAPERSIZE=_I18N("Paper size")
SPN_SCANRES=_I18N("Scan Resolution")
SPN_OUTFMT=_I18N("Output Format")
SPN_OUTDIR=_I18N("Output Directory")
SPV_DEF_OUTDIR=get_user_special_dir('USER_DIRECTORY_PICTURES')
SPN_OUTFILE=_I18N("Output File Name (not include ext.)")
SPV_DEF_OUTFILE=S_UNTITLED
SPN_APPLICATION=_I18N("Application")
SPV_CK_EXECAPP=_I18N("Execute Application")
SPN_ICONIMG=_I18N("Icon Image")
SPV_ICONIMG_TOFOLDER=_I18N("Built-in ToFolder icon")
SPV_ICONIMG_TOAPP=_I18N("Built-in ToApplication icon")
SPV_ICONIMG_CUSTOM=_I18N("Custom icon")
SPN_CUSTOM_ICONIMG=_I18N("Custom Icon File")

# Pagesize Name
PAPERSIZENAME_A0=_I18N("A0")
PAPERSIZENAME_A1=_I18N("A1")
PAPERSIZENAME_A2=_I18N("A2")
PAPERSIZENAME_A3=_I18N("A3")
PAPERSIZENAME_A4=_I18N("A4")
PAPERSIZENAME_A5=_I18N("A5")
PAPERSIZENAME_A6=_I18N("A6")
PAPERSIZENAME_A7=_I18N("A7")
PAPERSIZENAME_A8=_I18N("A8")
PAPERSIZENAME_A9=_I18N("A9")
PAPERSIZENAME_A10=_I18N("A10")
PAPERSIZENAME_A11=_I18N("A11")
PAPERSIZENAME_A12=_I18N("A12")
PAPERSIZENAME_B0=_I18N("B0")
PAPERSIZENAME_B1=_I18N("B1")
PAPERSIZENAME_B2=_I18N("B2")
PAPERSIZENAME_B3=_I18N("B3")
PAPERSIZENAME_B4=_I18N("B4")
PAPERSIZENAME_B5=_I18N("B5")
PAPERSIZENAME_B6=_I18N("B6")
PAPERSIZENAME_B7=_I18N("B7")
PAPERSIZENAME_B8=_I18N("B8")
PAPERSIZENAME_B9=_I18N("B9")
PAPERSIZENAME_B10=_I18N("B10")
PAPERSIZENAME_B11=_I18N("B11")
PAPERSIZENAME_B12=_I18N("B12")
PAPERSIZENAME_LETTER=_I18N("Letter")
PAPERSIZENAME_LEGAL=_I18N("Legal 14")
PAPERSIZENAME_LEGAL135=_I18N("Legal 13.5")
PAPERSIZENAME_FLSA=_I18N("Legal 13")
PAPERSIZENAME_FLSE=PAPERSIZENAME_FLSA
PAPERSIZENAME_EXECUTIVE=_I18N("Executive")

# Unit
UNIT_MM=0
UNIT_INCH=1
UNIT_POINT=2
UNIT_LOCAL={
    UNIT_MM:_I18N("mm"),
    UNIT_INCH:_I18N("inch"),
    UNIT_POINT:_I18N("pt")
}


# Log Levels
LOGLEVELS = {
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warning': logging.WARNING,
    'error': logging.ERROR,
    'critical': logging.CRITICAL
}
# Log format
LOGFORMAT = "%(asctime)s [%(process)d:%(thread)d] %(levelname)s %(message)s [%(filename)s:%(funcName)s:%(lineno)d]"


# Scale
MM_INCH=25.4        # 1 inch = 25.4 mm
POINT_INCH=72       # 1 inch = 72 point


# Scan resolution
MIN_SCAN_RES=50
MAX_SCAN_RES=1200
DEF_SCAN_RES=75
STP_SCAN_RES=1

# Maximum Number of Scan Configuration Preset
MAX_SCAN_CONFIG_PRESET=8

# Auto register retry interval time(second)
RETRY_INTERVAL_TIME=15

# Scan thread callback id/arg
CB_ERROR_IND=-1
CB_ERRORMSG_IND=-2
CB_CANCELED_IND=-3
CB_INFO_IND=-4
CB_OUTPUT_FILE=-5
CB_ARG_MSG='message'
CB_ARG_FILE='file-list'
