#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
manipulate utility configuration files.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"
__icon__ = "/usr/libexec/okimfputl/images/exeicon48x48.png"

# Directories
DRV_BASEDIR="/usr/libexec/okimfpdrv/"
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(DRV_BASEDIR)
sys.path.append(UTL_BASEDIR)
sys.path.append("../driver/.libs/")#DEBUG
sys.path.append(".")

import locale
locale.setlocale(locale.LC_ALL, '')

import pygtk
pygtk.require('2.0')
import gtk
import gtk.gdk
import gobject
import logging
import webbrowser

from guicom import *
from defs import *
from mfpcfgfile import *




class GeneralConfigPanel(AbstractConfigPanel):
    def __init__(self, padding=8):
        AbstractConfigPanel.__init__(self, LABEL_GENERAL, padding, "file://"+UTLDOC_DIR+"/genconfig.html")
        
        self._genconf = GeneralConfig()
        self._genconf.load()
        gendict = self._genconf.gets(SEC_GENERAL)
        
        
        
        
        f = VBoxFrame(label=LABEL_NOTIFY_METHOD)
        self.pack_start(f, False, padding=0)
        
        self.notify_rb1 = gtk.RadioButton(label=LABEL_NOTIFY_PYNOTIFY)
        self.notify_rb2 = gtk.RadioButton(group=self.notify_rb1, label=LABEL_NOTIFY_DIALOG)
        f.pack_start(self.notify_rb1, False)
        f.pack_start(self.notify_rb2, False)
        
        
        
        
        f = VBoxFrame(label=LABEL_HELP_VIEWER)
        self.pack_start(f, False, padding=0)
        
        self.use_webbrowser = gtk.CheckButton(LABEL_USE_WEBBROWSER)
        f.pack_start(self.use_webbrowser, False, padding=0)
        browsers = []
        for k in webbrowser._browsers.keys():
            logging.debug(k)
            if k in ("w3m", "www-browser"):
                continue#text browsers
            if k in ("x-www-browser"):
                continue
            browsers.append(k)
        self.webbrowsercb = build_ComboBox(browsers)
        self.webbrowser = ItemHBox(LABEL_WEBBROWSER, self.webbrowsercb)
        f.pack_start(self.webbrowser, False, padding=0)
        
        self.update()
        
        self.notify_rb1.connect('toggled', self.__notify_rb_cb)
        self.notify_rb2.connect('toggled', self.__notify_rb_cb)
        self.use_webbrowser.connect('toggled', self._use_webbrowser_chg_cb)
        self.webbrowsercb.connect('changed', self.__webbrowser_chg_cb)
    
    def __notify_rb_cb(self, radiobutton, *args):
        logging.debug("radiobutton=%s" % (radiobutton))
        self.set_changed(True)
    
    def _use_webbrowser_chg_cb(self, togglebutton, *args):
        logging.debug("togglebutton=%s" % (togglebutton))
        self.set_changed(True)
        if self.use_webbrowser.get_active():
            self.webbrowser.set_sensitive(True)
        else:
            self.webbrowser.set_sensitive(False)
    
    def __webbrowser_chg_cb(self, combobox, *args):
        logging.debug("combobox=%s" % (combobox))
        self.set_changed(True)
    
    def save(self):
        if has_notification_function():
            if self.notify_rb1.get_active():
                notify_method = V_USE_PYNOTIFY
                set_use_notification(True)
            elif self.notify_rb2.get_active():
                notify_method = V_USE_DIALOG
                set_use_notification(False)
            else:
                notify_method = None
                set_use_notification(True)
            self._genconf.set(SEC_GENERAL, OPT_NOTIFICATION, notify_method)
        
        if self.use_webbrowser.get_active():
            browser = self.webbrowsercb.get_active_text()
        else:
            browser = None
        self._genconf.set(SEC_GENERAL, OPT_HELP_VIEWER, browser)
        set_help_viewer(browser)
        
        self._genconf.save()
        logging.info("saved.")
    
    def update(self):
        gendict = self._genconf.gets(SEC_GENERAL)
        
        if has_notification_function():
            if gendict is not None and gendict[OPT_NOTIFICATION]:
                if gendict[OPT_NOTIFICATION] == V_USE_PYNOTIFY:
                    self.notify_rb1.set_active(True)
                else:
                    self.notify_rb2.set_active(True)
            else:
                self.notify_rb1.set_active(True)
        else:
            self.notify_rb2.set_active(True)
            self.notify_rb1.set_sensitive(False)
            self.notify_rb2.set_sensitive(False)
        
        
        if gendict is not None and gendict[OPT_HELP_VIEWER]:
            self.use_webbrowser.set_active(True)
            set_active_ComboBox(self.webbrowsercb, gendict[OPT_HELP_VIEWER])
        else:
            self.use_webbrowser.set_active(False)
            self.webbrowser.set_sensitive(False)




def configure():
    try:
        genconf = GeneralConfig()
        genconf.load()
        gendict = genconf.gets(SEC_GENERAL)
        if gendict is not None:
            set_help_viewer(gendict[OPT_HELP_VIEWER])
            if has_notification_function():
                if gendict[OPT_NOTIFICATION] == V_USE_PYNOTIFY:
                    set_use_notification(True)
                else:
                    set_use_notification(False)
    except Exception,e:
        logging.error(str(e))




if __name__ == "__main__":
    logging.basicConfig(format=LOGFORMAT)
    if len(sys.argv) > 1:
        rl = logging.getLogger()
        rl.setLevel(LOGLEVELS[sys.argv[1]])
    
    dlg = UtilConfigDialog()
    dlg.gtkmain()



