#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
Common GUI functions.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"


# Directories
DRV_BASEDIR="/usr/libexec/okimfpdrv/"
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(DRV_BASEDIR)
sys.path.append(UTL_BASEDIR)
sys.path.append("../driver/.libs/")#DEBUG
sys.path.append(".")

import os
import signal

import logging
import pygtk
pygtk.require('2.0')
import gtk
import gtk.gdk
import gobject
gtk.gdk.threads_init()
__has_pynotify__ = False
__use_pynotify__ = False
try:
    import pynotify
    __has_pynotify__ = True
    __use_pynotify__ = True
except:
    pass
import threading
import webbrowser

from defs import *
import pyscand




# msgview-treeview style
# see: http://library.gnome.org/devel/pygtk/stable/class-gtkrcstyle.html 
gtk.rc_parse_string('''
style "msgview-treeview-style"
{
  GtkTreeView::grid-line-pattern = '\1\1'
  GtkTreeView::grid-line-width = 0
  GtkTreeView::vertical-separator = 0
  GtkTreeView::horizontal-separator = 0
  GtkTreeView::allow-rules = 0
  GtkTreeView::even-row-color = 'white'
  GtkTreeView::odd-row-color = 'gray'
}
widget "*.msgview-treeview" style "msgview-treeview-style"
''')





def disable_pynotify():
    global __has_pynotify__
    global __use_pynotify__
    __has_pynotify__ = False
    __use_pynotify__ = False




def retrieve_window(w):
    while (True):
        if isinstance(w, gtk.Window):
            return w
        w = w.get_parent()
        if w is None:
            break
    return None



S_DESKTOP_KDE="kde"
S_DESKTOP_GNOME="gnome"
S_DESKTOP_UNITY="unity"
def get_desktop_environ():
    """デスクトップ環境を示す文字列を返します。不明な場合はNoneを返します。"""
    if os.environ.get('KDE_SESSION_UID') is not None:
        return S_DESKTOP_KDE
    
    if os.environ.get('GNOME_DESKTOP_SESSION_ID') is not None:
        #this-is-deprecated
        v = os.environ.get('DESKTOP_SESSION')
        if v is not None:
            rx = re.compile(".*ubuntu.*")
            if rx.match(v):
                v = os.environ.get('XDG_CURRENT_DESKTOP')
                if v is not None and v == 'Unity':
                    return S_DESKTOP_UNITY
        
        return S_DESKTOP_GNOME
    else:
        v = os.environ.get('DESKTOP_SESSION')
        if v is not None and v == "gnome":
            # for ubuntu 10.04
            return S_DESKTOP_GNOME
    
    return None




def gtk_threads_call(mt, func, args=()):
    if mt:
        try:
            gtk.gdk.threads_enter()
            rc = apply(func, args)
        finally:
            gtk.gdk.threads_leave()
    else:
        rc = apply(func, args)
    return rc




def has_notification_function():
    global __has_pynotify__
    return __has_pynotify__

def can_use_notification():
    """True if can use Notification UI."""
    global __has_pynotify__
    if __has_pynotify__:
        info = pynotify.get_server_info()
        if info is not None:
            return True
    return False

def set_use_notification(use=True):
    global __use_pynotify__
    __use_pynotify__ = use

def get_use_notification():
    global __use_pynotify__
    return __use_pynotify__

def use_notification():
    global __use_pynotify__
    if can_use_notification():
        if get_use_notification():
            return True
    return False




def _execcmd(cmd, succeedcodes=[0]):
    logging.debug("Popen('%s')" % cmd)
    p = Popen(cmd, shell=True, stderr=PIPE, close_fds=True)
    sts = p.wait()
    err = p.stderr.read()
    logging.info("exist status: %d, return code: %d" % (sts, p.returncode))
    logging.info("succeedcodes: " + str(succeedcodes))
    logging.info(err)
    for code in succeedcodes:
        if sts == code:
            logging.info("success")
            return
    # not success
    exmsg = ""
    try:
        if len(err) > 0:
             exmsg = err
    finally:
        pass
    if len(exmsg) == 0:
        exmsg = FORMAT_EXCEPTMSG_CMDEXEC_NG % (cmd)
    raise Exception(exmsg)




__help_viewer__=None
def set_help_viewer(webbrowsername=None):
    """
        webbrowsername: if None using gnome-open. if not None, using specified webbrowser.
    """
    global __help_viewer__
    __help_viewer__ = webbrowsername
    
def open_help(uri):
    """
        uri: help document uri.
    """
    global __help_viewer__
    if __help_viewer__ is not None:
        try:
            browser = webbrowser.get(__help_viewer__)
            logging.debug("webbrowser open(%s)" % (uri))
            browser.open(uri)
            return True
        except Exception,e:
            logging.warn("webbrowser open(%s) error. %s" % (uri, e))
            raise e
    else:
        cmdpath = check_command("gnome-open")
        if cmdpath is not None:
            try:
                logging.debug("gnome-open (%s)" % (uri))
                cmd = "%s %s &" % (cmdpath, uri)
                check_call(cmd, shell=True)
                return True
            except Exception,e:
                logging.warn("gnome-open (%s) error. %s" % (uri, e))
                raise e
        else:
            try:
                browser = webbrowser.get()
                logging.debug("default webbrowser open(%s)" % (uri))
                browser.open(uri)
                return True
            except Exception,e:
                logging.warn("default webbrowser open(%s) error. %s" % (uri, e))
                raise e
    return False




def concatstatus(msg, status):
    if status is not 0:
        if msg is not None:
            s = "%s (%s)" % (msg, pyscand.strstatus(status))
        else:
            s = pyscand.strstatus(status)
        return s
    else:
        return msg




def msg_comp(msg=None, detail=None, status=0):
    """return ( concatinated, message, secondary-message)"""
    s = msg
    ds = concatstatus(detail, status)
    if s is not None:
        ss = s
        if ds is not None:
            ss = s + " - " + ds
    else:
        s = ds
        ss = s
        ds = None
    return (ss, s, ds)




markup_set_color = lambda s, color : '<span foreground="' + color + '">'+s+'</span>'




def _dialog(parent, type, buttons, title, msg, secondary=None, icon=None, keep_above=False, timeout_sec=0):
    logging.debug("parent=%s, type=%s, title=%s, msg=%s, secondary=%s, icon=%s, keep_above=%s, timeout_sec=%s"
                  % (parent, type, title, msg, secondary, icon, keep_above, timeout_sec))
    try:
        if parent is not None:
            parent.set_sensitive(False)
            
        dlg = gtk.MessageDialog(parent=parent, type=type, buttons=buttons)
        dlg.set_title(title)
        if icon is not None:
            dlg.set_icon_from_file(icon)
        if parent is not None:
            dlg.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        else:
            dlg.set_position(gtk.WIN_POS_CENTER)
            if keep_above:
                dlg.set_keep_above(True)
        dlg.set_markup("" + msg + "")
        if secondary is not None:
            dlg.format_secondary_markup(("<i>%s</i>"%(secondary)))
        
        dlg.timeout = False
        if timeout_sec > 0:
            def timeout_cb(dlg):
                dlg.timeout = True
                dlg.response(gtk.RESPONSE_CLOSE)
            gobject.timeout_add_seconds(timeout_sec, timeout_cb, dlg)
        
        rc = dlg.run()
        
        if dlg.timeout:
            rc = 0
        
        dlg.destroy()
        return rc
    finally:
        if parent is not None:
            parent.set_sensitive(True)


def error_dialog(parent, title, msg, secondary=None, icon=None, keep_above=False, timeout_sec=0, mt=False):
    return gtk_threads_call(mt, _dialog, 
        (parent, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, title, msg, secondary, icon, keep_above, timeout_sec))


def warning_dialog(parent, title, msg, secondary=None, icon=None, keep_above=False, timeout_sec=0, mt=False):
    return gtk_threads_call(mt, _dialog,
        (parent, gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE, title, msg, secondary, icon, keep_above, timeout_sec))


def notify_dialog(parent, title, msg, secondary=None, icon=None, keep_above=False, timeout_sec=0, mt=False):
    return gtk_threads_call(mt, _dialog,
        (parent, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, title, msg, secondary, icon, keep_above, timeout_sec))


def confirm_dialog(parent, title, msg, secondary=None, icon=None, keep_above=False, timeout_sec=0, mt=False):
    return gtk_threads_call(mt, _dialog,
        (parent, gtk.MESSAGE_INFO, gtk.BUTTONS_OK_CANCEL, title, msg, secondary, icon, keep_above, timeout_sec))




def build_ComboBox(itemlist, actitem=None):
    combobox = gtk.combo_box_new_text()
    if itemlist is not None:
        set_ComboBox(combobox, itemlist, actitem)
    return combobox

def set_ComboBox(combobox, itemlist, actitem=None):
    actidx = 0
    idx = 0
    for i in itemlist:
        combobox.append_text(i)
        if actitem != None and actitem == i:
            actidx = idx
        idx = idx + 1
    combobox.set_active(actidx)

def set_active_ComboBox(combobox, actitem):
    model = combobox.get_model()
    for idx in xrange(len(model)):
        if model[idx][0] == actitem:
            combobox.set_active(idx)
            return True
    return False




class WindowList:
    def __init__(self):
        self.__lock__ = threading.Lock()
        self.__windows__ = []
    
    def count(self, parent=None):
        if parent is None:
            return len(self.__windows__)
        c = 0
        for w in self.__windows__:
            if w.get_transient_for() == parent:
                c += 1
        return c
    
    def print_(self):
        for w in self.__windows__:
            print w
    
    def debug(self):
        for w in self.__windows__:
            logging.debug(str(w))
    
    def present_all(self):
        n = 0
        for w in self.__windows__:
            try:
                w.present()
                n += 1
            except:
                pass
        return n
    
    def append(self, win):
        result = -1
        try:
            self.__lock__.acquire()
            self.__windows__.append(win)
            result = len(self.__windows__)
        finally:
            self.__lock__.release()
        logging.debug("append windows=%s" % (result))
        return result
    
    def remove(self, win):
        result = -1
        try:
            self.__lock__.acquire()
            self.__windows__.remove(win)
            result = len(self.__windows__)
        finally:
            self.__lock__.release()
        logging.debug("remove windows=%s" % (result))
        return result




class ItemHBox(gtk.HBox):
    def __init__(self,title,item,fill=False, padding=0):
        gtk.HBox.__init__(self, False, 0)
        self.label = gtk.Label(title)
        self.item = item
        self.pack_start(self.label, expand=False, padding=padding)
        self.pack_start(gtk.HSeparator(),False,False,2)
        self.pack_end(item, expand=fill, padding=padding)




class ItemPath(gtk.HBox):
    def __init__(self):
        gtk.HBox.__init__(self)
        self.path = gtk.Entry()
        self.btn = gtk.Button()
        #self.btn.set_label(BUTTON_DDD)
        self.btn.set_relief(gtk.RELIEF_NONE)
        self.btn.set_image(gtk.image_new_from_stock(gtk.STOCK_OPEN, gtk.ICON_SIZE_BUTTON))
        self.pack_start(self.path)
        self.pack_end(self.btn, False)
        self.btn.connect('clicked', self._on_click)
        
    def _on_click(self, btn):
        pwin = retrieve_window(self)
        try:
            if pwin is not None:
                pwin.set_sensitive(False)
            dlg = gtk.FileChooserDialog(TITLE_DIRCHOOSER_DLG, pwin, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
            d = self.path.get_text()
            if d is not None:
                d = os.path.expanduser(d)
                if os.path.isdir(d):
                    dlg.set_current_folder(d)
            rc = dlg.run()
            if rc == gtk.RESPONSE_OK:
                s = dlg.get_filename()
                logging.debug("DIR=%s" % (s))
                self.path.set_text(s)
            dlg.destroy()
        finally:
            if pwin is not None:
                pwin.set_sensitive(True)
        
    def set_path(self, path):
        s = os.path.expanduser(path)
        self.path.set_text(s)

    def get_path(self):
        return self.path.get_text()




class ItemFile(gtk.HBox):
    def __init__(self):
        gtk.HBox.__init__(self)
        self.path = gtk.Entry()
        self.btn = gtk.Button()
        #self.btn.set_label(BUTTON_DDD)
        self.btn.set_relief(gtk.RELIEF_NONE)
        self.btn.set_image(gtk.image_new_from_stock(gtk.STOCK_FILE, gtk.ICON_SIZE_BUTTON))
        self.pack_start(self.path)
        self.pack_end(self.btn, False)
        self.btn.connect('clicked', self._on_click)
        
    def _proc_get_text(self, text):
        return text
        
    def _proc_set_text(self, text):
        return text
        
    def _on_click(self, btn):
        pwin = retrieve_window(self)
        try:
            if pwin is not None:
                pwin.set_sensitive(False)
            dlg = gtk.FileChooserDialog(TITLE_FILECHOOSER_DLG, pwin, gtk.FILE_CHOOSER_ACTION_OPEN, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
            p = self._proc_get_text(self.path.get_text())
            if p is not None:
                p = os.path.expanduser(p)
                if os.path.isdir(p):
                    dlg.set_current_folder(p)
                else:
                    d = os.path.dirname(p)
                    if os.path.isdir(d):
                        dlg.set_current_folder(d)
                    if os.path.isfile(p):
                        f = os.path.basename(p)
                        dlg.select_filename(p)
            rc = dlg.run()
            if rc == gtk.RESPONSE_OK:
                s = dlg.get_filename()
                logging.debug("FILE=%s" % (s))
                self.path.set_text(self._proc_set_text(s))
            dlg.destroy()
        finally:
            if pwin != None:
                pwin.set_sensitive(True)
        
    def set_path(self, path):
        if path is not None:
            s = os.path.expanduser(path)
            self.path.set_text(s)

    def get_path(self):
        return self.path.get_text()




class VBoxFrame(gtk.Frame):
    def __init__(self, label=None, border=6):
        gtk.Frame.__init__(self)
        self.set_label(label)
        self.vbox = gtk.VBox(False, 0)
        self.vbox.set_border_width(border)
        self.add(self.vbox)
    
    def pack_start(self, child, expand=True, fill=True, padding=0):
        self.vbox.pack_start(child, expand, fill, padding)
    
    def pack_end(self, child, expand=True, fill=True, padding=0):
        self.vbox.pack_end(child, expand, fill, padding)




class MultiLineMsgView(gtk.ScrolledWindow):
    def __init__(self, lines=32, font_size_pt=10):
        gtk.ScrolledWindow.__init__(self)
        self.set_border_width(1)
        self.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        
        store = gtk.ListStore(str)
        view = gtk.TreeView(store)
        view.props.name = 'msgview-treeview'
        view.props.rules_hint = False
        view.set_size_request(100, 10)
        view.get_selection().set_mode(gtk.SELECTION_NONE)
        view.set_property('headers-visible', False)
        view.set_property('enable-grid-lines', False)
        view.set_property('enable-tree-lines', False)
        view.set_property('hover-selection', False)
        view.set_property('can-focus', False)
        
        column1 = gtk.TreeViewColumn("message")
        view.append_column(column1)
        cell1 = gtk.CellRendererText()
        cell1.set_property('size-points', font_size_pt)
        column1.pack_start(cell1, True)
        column1.set_attributes(cell1, markup=0)
        column1.set_resizable(True)
        
        #self.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.add(view)
        
        self.__view__ = view
        self.__store__ = store
        self.__lines__ = lines
        self.__count__ = 0
        self.__last_iter__ = None
        self.__lock__ = threading.Lock()
    
    def __append_message__(self, msg):
        if self.__count__ >= self.__lines__:
            i = self.__store__.get_iter_root()
            self.__store__.remove(i)
        else:
            self.__count__ += 1
        self.__last_iter__ = self.__store__.append([msg])
        path = self.__store__.get_path(self.__last_iter__)
        self.__view__.scroll_to_cell(path)
    
    def __update_message__(self, msg):
        if self.__last_iter__ is None:
            self.__append_message__(msg)
        else:
            self.__store__.set_value(self.__last_iter__, 0, msg)
    
    def append(self, msg):
        try:
            self.__lock__.acquire()
            self.__append_message__(msg)
        finally:
            self.__lock__.release()
    
    def update(self, msg):
        try:
            self.__lock__.acquire()
            self.__update_message__(msg)
        finally:
            self.__lock__.release()




def initNotificationUI(app_name):
    if __has_pynotify__:
        pynotify.init(app_name)

class NotificationUI:
    def __init__(self, icon_image_path):
        self._notification_title = "-"
        self._icon_image_path = icon_image_path
        self._icon_image = gtk.gdk.pixbuf_new_from_file(icon_image_path)
        self._icon_image_wrn = gtk.Label().render_icon(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
        self._icon_image_err = gtk.Label().render_icon(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_DIALOG)
        self.notification =  None
        if __has_pynotify__:
            try:
                self.notification = pynotify.Notification(self._notification_title, "-")
            except Exception, e:
                logging.error(str(e))
        else:
            logging.warn("pynotify not exists.")
    
    def get_vendor(self):
        if __has_pynotify__:
            try:
                return pynotify.get_server_info()['vendor']
            except Exception, e:
                logging.error(str(e))
        return ""

    
    def set_msg_title(self, title):
        self._notification_title = title
        
    
    
    def __show_msg(self, msg, status=0, icon=None, prio=None, timeout=None):
        """
        msg - message
        status - libstatus
        prio - pynotify urgency
        timeout - timeout in msec
        """
        if prio is None:
            prio = pynotify.URGENCY_NORMAL
        if timeout is None:
            timeout = pynotify.EXPIRES_DEFAULT
        self.notification.set_timeout(timeout)
        self.notification.set_urgency(prio)
        self.notification.set_icon_from_pixbuf(icon)
        self.notification.update(self._notification_title, concatstatus(msg, status), None)
        self.notification.show()
    
    def show_msg(self, msg, timeout=None, mt=False):
        if __has_pynotify__ and __use_pynotify__:
            if self.notification is not None:
                try:
                    self.__show_msg(msg, icon=self._icon_image, timeout=timeout)
                    return
                except Exception, e:
                    logging.warn(str(e))
        toval = 4 #default 4 sec
        if timeout is not None:
            toval = timeout / 1000
        notify_dialog(None, self._notification_title, msg, None, self._icon_image_path, True, toval, mt)
    
    def show_warn_msg(self, status=0, msg=MSG_ERROR, timeout=None, mt=False):
        if __has_pynotify__ and __use_pynotify__:
            if self.notification is not None:
                try:
                    self.__show_msg(msg, status, self._icon_image_wrn, pynotify.URGENCY_CRITICAL, timeout)
                    return
                except Exception, e:
                    logging.warn(str(e))
        s = None
        if status is not 0:
            s = pyscand.strstatus(status)
        toval = 0
        if timeout is not None:
            toval = timeout / 1000
        warning_dialog(None, self._notification_title, msg, s, self._icon_image_path, True, toval, mt)
    
    def show_err_msg(self, status=0, msg=MSG_ERROR, timeout=None, mt=False):
        if __has_pynotify__ and __use_pynotify__:
            if self.notification is not None:
                try:
                    self.__show_msg(msg, status, self._icon_image_err, pynotify.URGENCY_CRITICAL, timeout)
                    return
                except Exception, e:
                    logging.warn(str(e))
        s = None
        if status is not 0:
            s = pyscand.strstatus(status)
        toval = 0
        if timeout is not None:
            toval = timeout / 1000
        error_dialog(None, self._notification_title, msg, s, self._icon_image_path, True, toval, mt)


class ErrorMessageUI:
    """Abstract Error Message Notification UI"""
    def __init__(self):
        pass
    
    def error(self, msg=None, detail=None, status=0):
        s, m, sm = msg_comp(msg, detail, status)
        sys.stderr.write(s + "\n")


class NotifyErrMgsUI(ErrorMessageUI):
    def __init__(self, notification, title):
        ErrorMessageUI.__init__(self)
        self._notification = notification
        self._notification_title = title
        self._icon_image_err = gtk.Label().render_icon(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_DIALOG)
    
    def error(self, msg=None, detail=None, status=0):
        s, m, sm = msg_comp(msg, detail, status)
        if self._notification is not None:
            self._notification.set_urgency(pynotify.URGENCY_CRITICAL)
            self._notification.set_icon_from_pixbuf(self._icon_image_err)
            self._notification.update(self._notification_title, s)
            self._notification.show()
        else:
            logging.error("%s: %s" % (self._notification_title, s))

class DlgErrMgsUI(ErrorMessageUI):
    def __init__(self, parent, title, multithreading=True):
        ErrorMessageUI.__init__(self)
        self._parent_win = parent
        self._dialog_title = title
        self._multithreading = multithreading
    
    def error(self, msg=None, detail=None, status=0):
        s, m, sm = msg_comp(msg, detail, status)
        error_dialog(self._parent_win, self._dialog_title, m, sm, mt=self._multithreading)




import inspect
class CommonPalletWindow(gtk.Window):
    def onShow(self):
        if not self.__filter:
            def __filter(event, data):
                print event.type, event.window
                print inspect.getmembers(event)
                return gtk.gdk.FILTER_CONTINUE
            self.get_window().add_filter(__filter, self)
            self.__filter = True
    
    def onHide(self):
        pass
    
    def onOk(self):
        #When return 'False',  this dialog does not close.
        return True
    
    def onCancel(self):
        #When return 'False',  this dialog does not close.
        return True
    
    def onClose(self):
        #When return 'False',  this dialog does not close.
        return True
    
    def __init__(self, title=None, parent=None, flags=0, buttons=None, exit_handler=None, button_area=True, menubar=False, padding=8):
        gtk.Window.__init__(self)
        if title is not None:
            self.set_title(title)
        self.set_modal(False)
        self._parent = parent
        self.set_transient_for(parent)
        if parent is not None:
            self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        #else:
        #    print "parent == None"
        #    self.set_position(gtk.WIN_POS_CENTER)
        self.__filter = True#False
        self.__has_modal_child = False
        self._flags = flags
        self._buttons = buttons
        self._exit_handler = []
        self.__finished = False
        self.__response_code = 0
        self.__user_response_handler = {} 
        self.__button_area = button_area
        self.__shown = False
        self.__helpuri = None
        
        self.add_exit_handler(exit_handler)
        
        self.__base = gtk.VBox()
        
        
        if menubar:
            self.menubar = gtk.MenuBar()
            self.__base.pack_start(self.menubar, False, True)
        
        
        self.vbox = gtk.VBox()
        self.vbox.set_property('border-width', padding)#このVBoxのまわりにスペースをとるために設定
        self.__base.pack_start(self.vbox)
        
        if self.__button_area:
            self._action_btns = gtk.HButtonBox()
            self._action_btns.set_layout(gtk.BUTTONBOX_END)
            self._action_btns.set_spacing(6)
            bnum = 0
            if self._buttons is not None:
                bnum = len(self._buttons)
            if bnum >= 2:
                for i in xrange(0, bnum, 2):
                    bl = self._buttons[i]
                    id = self._buttons[i+1]
                    btn = gtk.Button(bl, )
                    btn.connect('clicked', self.__response_proc, id)
                    #self._action_btns.set_child_packing(btn , False, False, 4, gtk.PACK_END)
                    self._action_btns.pack_end(btn)
            h = gtk.HBox()
            h.pack_end(self._action_btns, False, False, 6)
            self.__base.pack_end(h, False, False, 6)
            
            if not (self._flags & gtk.DIALOG_NO_SEPARATOR) == gtk.DIALOG_NO_SEPARATOR:
                self.__base.pack_end(gtk.HSeparator(), False, False, 0)
            
        self.add(self.__base)
        
        #self.connect('event', self.__event_proc)
        self.connect('key-press-event', self.__kp_proc)
        self.connect('delete-event', self.__delete_proc)
        #self.connect('destroy-event', self.__destroy_proc)
        #self.connect('window-state-event', self.__win_state_proc)
        #self.connect('show-help', self.__show_help_proc)
        self.connect('destroy', self.__destroy)
    
    def setHelpUri(self, uri):
        self.__helpuri = uri
    
    def __event_proc(self, widget, event, *args):
        logging.debug("__event_proc, %s, %s, %s" % (widget, event, str(args)))
        return False
    
    def __show_help_proc(self, help_type, *args):
        logging.debug("__show_help_proc, %s, %s, %s" % (widget, help_type, str(args)))
        return False
    
    def __win_state_proc(self, widget, event, *args):
        logging.debug("__win_state_proc, %s, %s, %s" % (widget, event, str(args)))
        return True
    
    def __kp_proc(self, widget, event, *args):
        logging.debug("__kp_proc, %s, %s, %s" % (widget, event, str(args)))
        #print self.__helpuri
        if event.keyval == gtk.keysyms.F1 and self.__helpuri != None:
            # show help
            try:
                open_help(self.__helpuri)
            except Exception,e:
                error_dialog(self, self.get_title(), str(e))
            return True
        return False

    def __destroy_proc(self, widget, event, *args):
        logging.debug("__destroy_proc, %s, %s, %s" % (widget, event, str(args)))
        return False

    def __delete_proc(self, widget, event, *args):
        logging.debug("__delete_proc, %s, %s, %s" % (widget, event, str(args)))
        if not self.__has_modal_child:
            self.__response_proc(self, gtk.RESPONSE_DELETE_EVENT)
        return True
    
    def __response_proc(self, dialog, response_id, *args):
        logging.debug("__response_proc, %s, %s, %s" % (dialog, response_id, str(args)))
        logging.debug("RESPONSE: %s" % (response_id))
        self.__response_code = response_id
        if self.__user_response_handler.has_key(response_id):
            #print "has user handler"
            self.__finished = self.__user_response_handler[response_id]()
        else:
            if response_id == gtk.RESPONSE_OK:
                self.__finished = self.onOk()
            elif response_id == gtk.RESPONSE_CANCEL:
                self.__finished = self.onCancel()
            elif response_id == gtk.RESPONSE_CLOSE:
                self.__finished = self.onClose()
            elif response_id == gtk.RESPONSE_DELETE_EVENT:
                self.__finished = True
                
        if self.__finished:
            logging.debug("response finished.")
            self.close_window()
    
    def __do_exit_handler(self):
       if self._exit_handler is not None:
           if len(self._exit_handler) > 0:
               self._exit_handler.reverse()
               for h in self._exit_handler:
                   h()
    
    
    def __destroy(self, *args):
        logging.debug("destroy '%s' window. args=%s" % (self.get_title(), str(args)))
        self.close_window()
        gtk.Window.destroy(self)
   
    def close_window(self):
        self.__finished = True
        self.hide_all()
        self.__do_exit_handler()
    
    def add_exit_handler(self, func):
        if func is not None:
            self._exit_handler.append(func)

    def set_response(self, id, func):
        logging.debug("set_response: id=%d, func=%s" % (id, str(func)))
        self.__user_response_handler[id] = func
    
    def add_action_widget(self, widget, id):
        if self.__button_area:
            self._action_btns.pack_end(widget)
            widget.connect('clicked', self.__response_proc, id)
        
    def add_button(self, label, id):
        if self.__button_area:
            btn = gtk.Button(label)
            self._action_btns.pack_end(btn)
            btn.connect('clicked', self.__response_proc, id)
        
        
    def do_modal(self):
        rc = 0
        #evmask = 0
        try:
            if self._parent is not None:
                self._parent.set_sensitive(False)
                #w = self._parent.get_window()
                #evmask = w.get_events()
                #w.set_events(gtk.gdk.EXPOSURE_MASK)
                if isinstance(self._parent, CommonPalletWindow):
                    self._parent.__has_modal_child = True
                else:
                    self.set_modal(True)
            rc = self.__loop()
        finally:
            if self._parent is not None:
                self._parent.set_sensitive(True)
                #w.set_events(evmask)
                if isinstance(self._parent, CommonPalletWindow):
                    self._parent.__has_modal_child = False
        return rc
        
    def do_modeless(self):
        self.set_modal(False)
        rc = self.__loop()
        return rc
    
    def __loop(self):
        self.show_all()
        interrupt = None
        try:
            while not self.__finished:
                gtk.main_iteration_do(False)
        except Exception, e:
            logging.warn("Exception - %s" % (e))
        except KeyboardInterrupt, e:
            logging.info("Abort")
            interrupt = e
        self.hide_all()
        self.__do_exit_handler()
        if interrupt is not None:
            os.kill(os.getpid(), signal.SIGTERM)
        return self.__response_code

    def show(self):
        logging.debug("show")
        gtk.Window.show(self)
        self.__shown = True
        self.onShow()
    
    def hide(self):
        logging.debug("hide")
        self.__shown = False
        gtk.Window.hide(self)
        self.onHide()
    
    def show_all(self):
        logging.debug("show_all")
        gtk.Window.show_all(self)
        self.__shown = True
        self.onShow()

    def hide_all(self):
        logging.debug("hide_all")
        self.__shown = False
        gtk.Window.hide_all(self)
        self.onHide()

    def is_shown(self):
        return self.__shown

    def gtkmain(self):
        self.add_exit_handler(gtk.main_quit)
        self.show_all()
        try:
            gtk.main()
        except KeyboardInterrupt,e:
            logging.info("Abort")
        
    def quit(self):
        """destroy window"""
        logging.debug("QUIT")
        self.destroy()




class NotificationWindow(CommonPalletWindow):
    WIDTH = 200
    HEIGHT = 80
    
    def __init__(self, parent=None, title=None, timeout_sec=10):
        CommonPalletWindow.__init__(self, parent=parent, title=title)
        
        self.label = gtk.Label()
        self.label.set_line_wrap_mode(True)
        self.vbox.pack_start(self.label, True, True, 4)
        
        self.set_size_request(NotificationWindow.WIDTH, NotificationWindow.HEIGHT)
        #self.set_gravity(gtk.gdk.GRAVITY_SOUTH_EAST)
        self.set_gravity(gtk.gdk.GRAVITY_STATIC)
        #self.set_decorated(False)
        self.add_button(BUTTON_CLOSE, gtk.RESPONSE_CLOSE)
        gobject.timeout_add_seconds(timeout_sec, self.__timeout__)
    
    def __timeout__(self):
        self.close_window()
    
    def set_text(self, msg):
        self.label.set_text(msg)




class AbstractConfigPanel(gtk.VBox):
    def __init__(self, label, padding=8, helpuri=None):
        gtk.VBox.__init__(self)
        self.set_border_width(padding)
        self.label = gtk.Label(label)
        self.__changed__ = False
        self.__helpuri__ = helpuri
        
        self._applybtn = gtk.Button(BUTTON_APPLY)
        self._applybtn.connect('clicked', self.__apply_cb)
        self._page_btns = gtk.HButtonBox()
        self._page_btns.set_layout(gtk.BUTTONBOX_END)
        self._page_btns.set_spacing(0)
        self._page_btns.pack_end(self._applybtn)
        self.pack_end(self._page_btns, False, False, 0)
        self.pack_end(gtk.VSeparator(), False, False, 4)
        
        self.set_changed(False)
    
    def __apply_cb(self, button, *args):
        self.save()
        self.set_changed(False)
        logging.debug("apply [%s]" % (self.label.get_text()))
    
    def apply(self):
        self.__apply_cb(self._applybtn)
    
    def get_label(self):
        return self.label
    
    def set_changed(self, st):
        self.__changed__ = st
        self._applybtn.set_sensitive(st)
    
    def get_changed(self):
        return self.__changed__
    
    def save(self):
        pass
    
    def update(self):
        pass


class TabConfigDialog(CommonPalletWindow):
    def __init__(self, title, panels=[]):
        CommonPalletWindow.__init__(self, title)
        self.__current_page_num__ = None
        
        self.notebook = gtk.Notebook()
        self.notebook.set_tab_pos(gtk.POS_TOP)
        self.notebook.popup_enable()
        self.notebook.set_scrollable(True)
        self.vbox.pack_start(self.notebook, expand=True, fill=True, padding=0)
        
        for p in panels:
            self.notebook.append_page(p, p.get_label())
        
        
        self.add_button(BUTTON_CLOSE, gtk.RESPONSE_CLOSE)
        self.set_response(gtk.RESPONSE_DELETE_EVENT, self.onClose)
        
        self.notebook.connect('switch-page', self.__switch_page_cb)
    
    
    def __switch_page_cb(self, notebook, page, page_num, *args):
        if not self.is_shown():
            self.__current_page_num__ = page_num
            return
        p = self.notebook.get_nth_page(page_num)
        p_now = self.notebook.get_nth_page(self.__current_page_num__)
        logging.debug("switch-page [%s]->[%s]" %
                      (self.notebook.get_tab_label(p_now).get_text(),
                       self.notebook.get_tab_label(p).get_text()))
        if isinstance(p_now, AbstractConfigPanel):
            if p_now.get_changed():
                logging.debug("changed.")
                rc = confirm_dialog(self, self.get_title(), MSG_CONFIRM_SAVE_CHG)
                if rc == gtk.RESPONSE_OK:
                    p_now.save()
            p_now.set_changed(False)
        
        if isinstance(p, AbstractConfigPanel):
            p.update()
            p.set_changed(False)
            if p.__helpuri__ is not None:
                self.setHelpUri(p.__helpuri__)
        
        self.__current_page_num__ = page_num
    
    def saveall(self):
        for i in xrange(self.notebook.get_n_pages()):
            p = self.notebook.get_nth_page(i)
            if isinstance(p, AbstractConfigPanel):
                logging.debug("saving... [%s]" % (self.notebook.get_tab_label(p).get_text()))
                p.save()
    
    def onClose(self):
        p_now = self.notebook.get_nth_page(self.__current_page_num__)
        logging.debug("page [%s]" % (self.notebook.get_tab_label(p_now).get_text()))
        if isinstance(p_now, AbstractConfigPanel):
            if p_now.get_changed():
                logging.debug("changed.")
                rc = confirm_dialog(self, self.get_title(), MSG_CONFIRM_SAVE_CHG)
                if rc == gtk.RESPONSE_OK:
                    p_now.save()
            p_now.set_changed(False)
        return True



