#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
Image file format converter.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"

# Directories
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(UTL_BASEDIR)
sys.path.append(".")

import os
import os.path
import re
import tempfile
import exceptions
import logging

from defs import *
import papersize




def shesc(s):
    s = s.replace("\"", "\\\"")
    if not s.startswith('"'):
        s = '"' + s + '"'
    return s




def can_convert_to(ext, reqpkg=None):
    if ext == EXT_PNM or ext == EXT_PPM or ext == EXT_PGM or ext == EXT_PBM:
        #no conv
        return True
    elif ext == EXT_PNG:
        if not check_command("pnmtopng") == None:
            return True
        if type(reqpkg) == list:
            reqpkg.append("netpbm")
        return False
    elif ext == EXT_JPG:
        if not check_command("pnmtojpeg") == None:
            return True
        if type(reqpkg) == list:
            reqpkg.append("netpbm")
        return False
    elif ext == EXT_TIF:
        if not check_command("pnmtotiff") == None:
            return True
        if type(reqpkg) == list:
            reqpkg.append("netpbm")
        return False
    elif ext == EXT_PDF:
        cmd1 = check_command("pnmtops")
        cmd2 = check_command("gs")
        if (not cmd1 == None) and (not cmd2 == None):
            return True
        if type(reqpkg) == list:
            if cmd1 == None:
                reqpkg.append("netpbm")
            if cmd2 == None:
                reqpkg.append("ghostscript")
        return False
    else:
        # unknown 
        return False




def execcmd(cmd, redirect=None, succeedcodes=[0]):
    """
    cmdで指定されたコマンド文字列を実行する。
    redirectには、コマンド実行した際の標準出力の書き出し先のパスを指定する。
    succeedcodesには、正常終了とみなす終了ステータス値を指定する（リスト）。
    """
    logging.info("cmd='%s', redirect=%s, succeescodes=%s" % (cmd, redirect, str(succeedcodes)))
    if redirect is None:
        logging.debug("Popen('%s')" % cmd)
        p = Popen(cmd, shell=True, stderr=PIPE, close_fds=True)
    else:
        # リダイレクトするとリダイレクト先のパスが文字化けする場合があるため、以下の処理としている。
        dir = os.path.dirname(redirect)
        file = os.path.basename(redirect)
        fd, tmppath = tempfile.mkstemp("tmp", "cnv", dir)
        os.close(fd)
        tmpname = os.path.basename(tmppath)
        cmd = "cd %s; %s > %s" % (shesc(dir), cmd, tmpname)
        logging.debug("Popen('%s')" % cmd)
        p = Popen(cmd, shell=True, stderr=PIPE, close_fds=True)
    sts = p.wait()
    err = p.stderr.read()
    logging.debug("exist status: %d, return code: %d" % (sts, p.returncode))
    logging.debug("succeedcodes: " + str(succeedcodes))
    logging.info(err)
    for code in succeedcodes:
        if sts == code:
            logging.debug("success")
            if redirect is not None:
                rencmd = "mv %s %s" % (tmppath, redirect)
                logging.debug("rename: "+rencmd)
                check_call(rencmd, shell=True)
            return
    # not success
    if redirect is not None:
        try:
            os.remove(redirect)
        except Exception,e:
            logging.warn(str(e))
    exmsg = ""
    try:
        if len(err) > 0:
             exmsg = err
    finally:
        pass
    if len(exmsg) == 0:
        exmsg = FORMAT_EXCEPTMSG_CMDEXEC_NG % (cmd)
    raise Exception(exmsg)




def listtostr(list):
    result = ""
    for f in list:
        if not len(result) == 0:
            result = result + " "
        result = result + shesc(f)
    return result




def deletefiles(filelist):
    for f in filelist:
        try:
            os.remove(f)
        except Exception, e:
            logging.error(str(e))




#Abstract File Converter
class FileConverter:
    def __init__(self):
        pass
    
    def convert(self, filelist):
        pass




#Command execution converter
class CommandConverter(FileConverter):
    def __init__(self, command, ext):
        FileConverter.__init__(self)
        self._command = command
        self._ext = ext
    
    def convert(self, filelist):
        result = []
        for f in filelist:
            logging.info("Convert %s" % (f))
            ss = os.path.splitext(f)
            out = unicode(ss[0] + "." + self._ext)
            logging.info("  output: "+out)
            cmd = "%s %s" % (self._command, shesc(f))
            execcmd(cmd, out)
            result.append(out)
        return result




#PNM to PNG
class PnmToPngConverter(CommandConverter):
    def __init__(self):
        CommandConverter.__init__(self, "pnmtopng", EXT_PNG)




#PNM to JPEG
class PnmToJpegConverter(CommandConverter):
    def __init__(self):
        CommandConverter.__init__(self, "pnmtojpeg", EXT_JPG)




#PNM to Tiff
class PnmToTiffConverter(CommandConverter):
    def __init__(self):
        CommandConverter.__init__(self, "pnmtotiff", EXT_TIF)




#PDF converter
class PnmToPdfConverter(FileConverter):
    def __init__(self, resolution=75, paper_size=papersize.A4):
        FileConverter.__init__(self)
        self._command = "pnmtops"
        self._gscommand = "gs"
        self._psext = "ps"
        self._pdfext = EXT_PDF
        self._resolution_dpi = resolution
        self._papersize = paper_size
    
    def convert(self, filelist):
        w = self._papersize.width_inch()
        h = self._papersize.height_inch()
        dpi = self._resolution_dpi
        psflst = []
        for f in filelist:
            logging.info("Convert(PS) %s" % (f))
            ss = os.path.splitext(f)
            out = unicode(ss[0] + "." + self._psext)
            cmd = "%s -setpage -dpi %d -width=%f -height=%f -equalpixels %s" % (self._command, dpi, w, h, shesc(f))
            execcmd(cmd, out, [0, 1])
            psflst.append(out)
        pss = listtostr(psflst)
        ss = os.path.splitext(filelist[0])
        m = re.match(r"^(.*)\[[0-9]+\]$", ss[0])
        if not m == None:
            bname = m.groups()[0]
        else:
            bname = ss[0]
        pdf = bname + "." + self._pdfext
        tmp = bname + ".tmp"
        # see http://www.ghostscript.com/doc/current/Use.htm#Paper_size
        sizeopt = ""
        if self._papersize.is_gs_pagetypename():
            sizeopt = "-sPAPERSIZE=%s" % (self._papersize.pagetypename())
        else:
            sizeopt = "-dDEVICEWIDTHPOINTS=%f -dDEVICEHEIGHTPOINTS=%f" % (self._papersize.width_pt(), self._papersize.height_pt())
        cmd = "%s -dBATCH -dNOPAUSE -sDEVICE=pdfwrite %s -dPDFSETTINGS=/ebook -sOutputFile=%s %s" % (self._gscommand, sizeopt, shesc(tmp), pss)
        execcmd(cmd)
        try:
            cmd = "sed -e 's/^\\/Title(\\(.*\\))\\(.*\\)$/\\2/g' %s | sed -e 's/^\\/Creator(\\(.*\\))\\(.*\\)$/\\/Creator(scantool)\\2/g'" % (shesc(tmp))
            execcmd(cmd, pdf)
            deletefiles([tmp])
        except Exception, e:
            logging.error(str(e))
            os.rename(tmp, pdf)
        deletefiles(psflst)
        return [pdf]


