#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
System tray program.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"
__icon__ = "/usr/libexec/okimfputl/images/exeicon48x48.png"

# Directories
DRV_BASEDIR="/usr/libexec/okimfpdrv/"
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(DRV_BASEDIR)
sys.path.append(UTL_BASEDIR)
sys.path.append("../driver/.libs/")#DEBUG
sys.path.append(".")

import locale
locale.setlocale(locale.LC_ALL, '')

import os
import pygtk
pygtk.require('2.0')
import gtk
gtk.gdk.threads_init()
import threading
import logging
import gc


from defs import *
from guicom import *


# determine whether to use appindicator or not
__has_appindicator__ = False
__use_appindicator__ = False
__desktop_session__ = get_desktop_environ()
try:
    import appindicator
    __has_appindicator__ = True
    if __desktop_session__ is not None and __desktop_session__ == S_DESKTOP_UNITY:
        __use_appindicator__ = True
except:
    pass


import scantool
import pushass
import pushconfig
import genconfig
import pyscand


CONFIG_WIN_WIDTH = 380
CONFIG_WIN_HEIGHT = 160


class ScanningMenu(gtk.Menu):
    def __activete(self, menuitem, *args):
        s =menuitem.get_label()
        logging.debug("Scanning Menu: activete: label=%s" % (s))
        pyscand.cancel_scan(s)
    
    def __init__(self):
        gtk.Menu.__init__(self)
        scdevs = scantool.scan_manager.list_devices()
        if len(scdevs) > 0:
            for d in scdevs:
                logging.debug("ScanningMenu: item=%s" % (str(d)))
                item = gtk.MenuItem(d)
                item.connect('activate', self.__activete)
                self.append(item)
        else:
            item = gtk.MenuItem(MENUITEM_NO_EXEC)
            item.set_sensitive(False)
            self.append(item)
        self.show_all()



class TrayMenu(gtk.Menu):
    def _menuact_start_watch(self, *args):
        if self._owner.pushassinst.is_registered():
            logging.debug("Cancel register")
            if self._owner.pushassinst.cancel_register():
                self.menuitem_watchps.set_label(MENUITEM_START_ACCEPT)
        else:
            logging.debug("Register")
            if self._owner.pushassinst.register():
                self.menuitem_watchps.set_label(MENUITEM_STOP_ACCEPT)
    
    
    def _menuact_scan_tool(self, *args):
        self._owner.run_scan_tool()
    
    def _menuact_scan(self, *args):
        self._owner.run_scan()
    
    def _menuact_config(self, *args):
        self._owner.run_config()
    
    def _menuact_help(self, *args):
        open_help("file://"+UTLDOC_DIR+"/applet.html")
    
    def _menuact_drvdoc(self, *args):
        open_help("file://"+DRVDOC_DIR+"/index.html")
    
    
    def _menuact_gc(self, *args):
        n = gc.collect()
        logging.info("gc->%s" % n)
        logging.info(str(gc.garbage))
    
    
    def _exit(self, *args):
        self._owner.quit()
    
    def _cb_thrmgt(self, *args):
        global __use_appindicator__
        logging.debug("_cb_thrmgt " + str(args))
        #self.menuitem_cclscan.remove_submenu()
        #del self.scan_ccl_submenu
        self.scan_ccl_submenu = ScanningMenu()
        if __use_appindicator__:
            # Appindicatorの場合はここでサブメニューを変更する。
            # StatusIconの場合はデスクトップが固まる場合があるのでpopup時に更新する。
            sm = self.menuitem_cclscan.set_submenu(self.scan_ccl_submenu)
       
    
    def __init__(self, owner):
        gtk.Menu.__init__(self)
        self._owner = owner
        
        scantool.scan_manager.set_callback(self._cb_thrmgt)
        
        # MenuItem
        mn = ""
        if not self._owner.pushassinst.is_registered():
            mn = MENUITEM_START_ACCEPT
        else:
            mn = MENUITEM_STOP_ACCEPT
        self.menuitem_watchps = gtk.MenuItem(mn)
        self.menuitem_watchps.connect('activate', self._menuact_start_watch)
        self.append(self.menuitem_watchps)
        
         # MenuItem
        self.menuitem_scantool = gtk.MenuItem(MENUITEM_RUN_SCANTOOL)
        self.menuitem_scantool.connect('activate', self._menuact_scan_tool)
        self.append(self.menuitem_scantool)
        
         # MenuItem
        self.menuitem_scan = gtk.MenuItem(MENUITEM_SCAN_START)
        self.menuitem_scan.connect('activate', self._menuact_scan)
        self.append(self.menuitem_scan)
        
        # MenuItem
        self.menuitem_cclscan = gtk.MenuItem(MENUITEM_CANCEL_SCAN)
        self.append(self.menuitem_cclscan)
        self.scan_ccl_submenu = ScanningMenu()
        self.menuitem_cclscan.set_submenu(self.scan_ccl_submenu)
        
        
        # MenuItem (Separator)
        self.append(gtk.SeparatorMenuItem())
        
        # MenuItem
        self.menuitem_utlcfg = gtk.MenuItem(MENUITEM_RUN_UTLCONF)
        self.menuitem_utlcfg.connect('activate', self._menuact_config)
        self.append(self.menuitem_utlcfg)
        
        # MenuItem (Separator)
        self.append(gtk.SeparatorMenuItem())
        
        # MenuItem
        self.menuitem_drvdoc = gtk.MenuItem(MENUITEM_DRVDOC)
        self.menuitem_drvdoc.connect('activate', self._menuact_drvdoc)
        self.append(self.menuitem_drvdoc)
        
        # MenuItem
        self.menuitem_help = gtk.MenuItem(MENUITEM_HELP)
        self.menuitem_help.connect('activate', self._menuact_help)
        self.append(self.menuitem_help)
        
        
        if logging.getLogger().isEnabledFor(logging.DEBUG):
            # MenuItem (Separator)
            self.append(gtk.SeparatorMenuItem())
            
            # MenuItem
            self.menuitem_gc = gtk.MenuItem("Froce GC")
            self.menuitem_gc.connect('activate', self._menuact_gc)
            self.append(self.menuitem_gc)
        
        
        # MenuItem (Separator)
        self.append(gtk.SeparatorMenuItem())
        
        # MenuItem
        self.menuitem_exit = gtk.MenuItem(MENUITEM_QUIT_TRAYICON)
        self.menuitem_exit.connect('activate', self._exit)
        self.append(self.menuitem_exit)
       
        self.show_all()
    
    def popup(self, parent_menu_shell, parent_menu_item, func, button, activate_time, data=None):
        sm = self.menuitem_cclscan.set_submenu(self.scan_ccl_submenu)
        gtk.Menu.popup(self, parent_menu_shell, parent_menu_item, func, button, activate_time, data)




class MfpaTray:
    
    def main(self):
        try:
            gtk.main()
        except KeyboardInterrupt,e:
            logging.info("Abort")
    
    def present_all_window(self):
        n = 0
        if self._scan_tool_dlg is not None:
            logging.debug(str(self._scan_tool_dlg))
            self._scan_tool_dlg.present()
            n += 1
        if self._config_dlg is not None:
            logging.debug(str(self._config_dlg))
            self._config_dlg.present()
            n += 1
        if scantool.ScanDialog.dlgs.count() is not 0:
            scantool.ScanDialog.dlgs.debug()
            n += scantool.ScanDialog.dlgs.present_all()
        if pushass.PushScanDialog.dlgs.count() is not 0:
            pushass.PushScanDialog.dlgs.debug()
            n += pushass.PushScanDialog.dlgs.present_all()
        return n
    
    def quit(self):
        logging.info("Quit tray icon.")
        n = self.present_all_window()
        if n == 0:
            self.pushassinst.stop()
            gtk.main_quit()
            logging.info("finished.")
        else:
            error_dialog(None, TITLE_TRAYICON_APPLET, MSG_QUIT_WRN)
    
    def __exit_scan_tool(self):
        self._scan_tool_dlg = None
    
    def run_scan_tool(self, *args):
        if self._scan_tool_dlg == None:
            self._scan_tool_dlg = scantool.ScanPresetDialog()
            self._scan_tool_dlg.add_exit_handler(self.__exit_scan_tool)
            self._scan_tool_dlg.set_modal(False)
            self._scan_tool_dlg.show_all()
        else:
            self._scan_tool_dlg.present()
    
    def run_scan(self, *args):
        try:
            dlg = scantool.ScanDialog(None, None, None)
            dlg.show_all()
        except Exception, e:
            error_dialog(None, TITLE_SCAN, unicode(e))
    
    def __exit_config(self):
        self._config_dlg = None
    
    def run_config(self, *args):
        if self._config_dlg == None:
            self._config_dlg = buildConfigDialog()
            self._config_dlg.add_exit_handler(self.__exit_config)
            self._config_dlg.set_modal(False)
            self._config_dlg.show_all()
        else:
            self._config_dlg.present()
    
    def _pushass_callback(self, registered):
        global __use_appindicator__
        if not __use_appindicator__:
            if registered:
                self.statusicon.set_tooltip(TIPS_ACCEPT_PUSHSCAN)
            else:
                self.statusicon.set_tooltip(TIPS_NOT_ACCEPT_PUSHSCAN)
        
        if registered:
            self.menu.menuitem_watchps.set_label(MENUITEM_STOP_ACCEPT)
        else:
            self.menu.menuitem_watchps.set_label(MENUITEM_START_ACCEPT)
    
    def __init__(self):
        global __use_appindicator__
        
        self._scan_tool_dlg = None
        self._config_dlg = None
        
        # PushScan Assistant instantiate
        self.pushassinst = pushass.PushScanAssistant(self._pushass_callback)
        self.pushassinst.setDaemon(True)
        self.pushassinst.set_auto_register(True)
        self.pushassinst.register()
        
        if not __use_appindicator__:
            # Status icon
            self.menu = TrayMenu(self)
            self.statusicon = gtk.StatusIcon()
            self.statusicon.set_from_file(ICON_STATUSICON_IMG_PATH)
            self.statusicon.connect('popup-menu', self.popup_menu)
            self.statusicon.connect('activate', self.activate_icon)

        else:
            # Appindicator
            self.menu = TrayMenu(self)
            self.ai = appindicator.Indicator(TITLE_TRAYICON_APPLET, ICON_STATUSICON_IMG_PATH,appindicator.CATEGORY_SYSTEM_SERVICES)
            self.ai.set_status(appindicator.STATUS_ACTIVE)
            self.ai.set_menu(self.menu)
        
        # PushScan Assistant Start
        self.pushassinst.start()
    
    def popup_menu(self, status_icon, button, time, *args):
        self.menu.popup(None, None, None, button, time)
    
    def activate_icon(self, status_icon, *args):
        self.present_all_window()




def buildConfigDialog():
    panels = []
    panels.append(genconfig.GeneralConfigPanel())
    panels.append(pushconfig.PushScanConfigPanel())
    dlg = TabConfigDialog(TITLE_UTL_CONFIG, panels)
    # Set the help document URI for this window
    dlg.setHelpUri("file://"+UTLDOC_DIR+"/genconfig.html")
    dlg.set_icon_from_file(__icon__)
    dlg.resize(CONFIG_WIN_WIDTH, CONFIG_WIN_HEIGHT)
    return dlg



if __name__ == "__main__":
    import getopt
    confguionly = False
    logging.basicConfig(format=LOGFORMAT)
    opts, args = getopt.getopt(sys.argv[1:], 'cl:', ['config', 'loglevel=', 'no-pynotify'])
    #print "args=", args
    #print "opts=", opts
    for k, v in opts:
        if k in ('-l', '--loglevel'):
            rl = logging.getLogger()
            rl.setLevel(LOGLEVELS[v])
            if LOGLEVELS[v] is logging.DEBUG:
                #gc.set_debug(gc.DEBUG_COLLECTABLE | gc.DEBUG_UNCOLLECTABLE | gc.DEBUG_INSTANCES | gc.DEBUG_OBJECTS | gc.DEBUG_SAVEALL)
                gc.set_debug(gc.DEBUG_LEAK)
        elif k in ('c', '--config'):
            confguionly = True
        elif k in ('--no-pynotify'):
            disable_pynotify()
    
    if not confguionly:
        if __has_appindicator__:
            if __use_appindicator__:
                logging.debug("use appindicator [%s]" % (__desktop_session__))
            else:
                logging.debug("not use appindicator [%s]" % (__desktop_session__))
        else:
            logging.debug("has no appindicator [%s]" % (__desktop_session__))
        
        w = gtk.Window()
        w.set_icon_from_file(__icon__)
        genconfig.configure()
        initNotificationUI(TITLE_TRAYICON_APPLET)
        trayicon = MfpaTray()
        trayicon.main()
    else:
        w = buildConfigDialog()
        w.gtkmain()
