#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
manipulate configuration files.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"

# Directories
DRV_BASEDIR="/usr/libexec/okimfpdrv/"
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(DRV_BASEDIR)
sys.path.append(UTL_BASEDIR)
sys.path.append("../driver/.libs/")#DEBUG
sys.path.append(".")

import os
import os.path
import signal
import time
import ConfigParser
import locale
import exceptions
import logging

from defs import *
from guicom import *


# User config file directory
USER_BASEDIR=os.path.expanduser("~/.okimfp/")

# Config Files
SCAN_PRESET_FILENAME="scanconfig.conf"
PUSH_CONFIG_FILENAME="pushconfig.conf"
APP_CONFIG_FILENAME="appconfig.conf"
GEN_CONFIG_FILENAME="config.conf"

# Utility config section name
SEC_GENERAL="general"
SEC_SCANNER="scanner"

# PushScan config section name
PUSHCONF_TOFOLDER="to_folder"
PUSHCONF_TOAPPLICATION="to_application"

# Config item name
## Display order
OPT_ORDER="order"
## Scan configuration
OPT_SCANSRC="scan_source"
OPT_SCANMODE="scan_mode"
OPT_SCANRES="scan_resolution"
OPT_DUPSCAN="duplex_scan"
OPT_PAPERSIZE="paper_size"
OPT_OUTFMT="output_format"
OPT_OUTDIR="output_dir"
OPT_OUTFILE="output_file"
OPT_EXEAPP="exec_application"
OPT_ICON="tool_icon"
OPT_DEVICE="device"
## Push configuration
OPT_PRESET="preset"
OPT_SHOW_SCAN_DIALOG="show_scan_dialog"
## Application configuration
OPT_COMMAND="command"
## Utility configuration, SEC_SCANNER options
OPT_NOTIFICATION="notification"
OPT_HELP_VIEWER="help_view_browser"

# Values
## Common
V_TRUE="True"
V_FALSE="False"
## OPT_NOTIFICATION values
V_USE_PYNOTIFY="pynotify"
V_USE_DIALOG="dialog"




def get_value(dict, name):
    try:
        value = dict[name]
        return value
    except KeyError, e:
        return None
    except:
        return None




def _enval(value):
    if value is None:
        return ""
    s = str(value)
    return s.replace('%', '%%')

def _deval(value):
    return value.replace('%%', '%')


class AbstractConfig:
    def __init__(self, filename):
        self._select_file(filename)
        self._parser = ConfigParser.SafeConfigParser()
    
    def _select_file(self, filename):
        self._filename = filename
        if os.path.isfile(USER_BASEDIR + self._filename):
            # has usr config
            self._path = USER_BASEDIR + self._filename
        else:
            # has no usr config file
            (d, l) = get_lang_dir(UTL_BASEDIR)
            if l == None:
                logging.fatal("Installation error.")
            deskenv = get_desktop_environ()
            if deskenv in (None, S_DESKTOP_UNITY):
                deskenv = S_DESKTOP_GNOME
            self._path = d + os.path.sep + deskenv + os.path.sep + self._filename
            if not os.path.exists(self._path):
                logging.fatal("Installation error.")
        logging.info("path=%s" % (self._path))
    
    def _mkuserdir(self):
        if os.path.isdir(USER_BASEDIR) == False:
            os.mkdir(USER_BASEDIR)
    
    def load(self):
        self._parser.read([self._path])
        
        #check ordered
        secs = self._parser.sections()
        ordered = True
        for sec in secs:
            opts = self._parser.options(sec)
            if not self._parser.has_option(sec, OPT_ORDER):
                ordered = False
                break #has no order option
        
        if not ordered:
            n = 0
            for sec in secs:
                self._parser.set(sec, OPT_ORDER, str(n))
                n = n + 1
    
    def save(self):
        self._mkuserdir()
        fd = os.open(USER_BASEDIR + self._filename, os.O_CREAT|os.O_TRUNC|os.O_RDWR, 0644)
        fp = os.fdopen(fd, "w")
        self._parser.write(fp)
        fp.close()


    def delete(self, section):
        if self._parser.has_option(section, OPT_ORDER):
            delorder = self._parser.getint(section, OPT_ORDER)
            for sec in self._parser.sections():
                if self._parser.has_option(sec, OPT_ORDER):
                    order = self._parser.getint(sec, OPT_ORDER)
                    if order > delorder:
                        n_order = order - 1
                        self._parser.set(sec, OPT_ORDER, str(n_order))
        self._parser.remove_section(section)
        
    def rename(self, oldsection, newsection):
        if self._parser.has_section(newsection):
            self._parser.remove_section(newsection) #force delete
        if self._parser.has_section(oldsection) == False:
            secs = self._parser.sections()
            n_order = len(secs)
            self._parser.add_section(section)
            self._parser.set(section, OPT_ORDER, str(n_order))
        else:
            d = self.gets(oldsection)
            self._parser.remove_section(oldsection)
            self._parser.add_section(newsection)
            for k in d.keys():
                self._parser.set(newsection, k, str(d[k]))
    
    def __chksec__(self, section):
        has_sec = self._parser.has_section(section)
        if has_sec == False:
            secs = self._parser.sections()
            n_order = len(secs)
            self._parser.add_section(section)
            self._parser.set(section, OPT_ORDER, str(n_order))
    
    def add(self, section, itemsdict):
        self.__chksec__(section)
        for k in itemsdict.keys():
            if k == OPT_ORDER:
                continue
            v = _enval(itemsdict[k])
            logging.debug("set [%s] %s -> %s" % (section, k, v))
            self._parser.remove_option(section, k)
            self._parser.set(section, k, v)

    def set(self, section, itemname, value):
        self.__chksec__(section)
        self._parser.remove_option(section, itemname)
        if self._parser.has_section(section) == False:
            self._parser.add_section(section)
        self._parser.set(section, itemname, _enval(value))

    def gets(self, section):
        try:
            res = {}
            c = self._parser.items(section)
            for i in c:
                v = _deval(i[1])
                res[i[0]] = self._value_parse(i[0], v)
            return res
        except:
            return None

    def get(self, section, item):
        try:
            v = self._parser.get(section, item)
            v = _deval(v)
            return self._value_parse(item, v)
        except:
            return None

    def find(self, itemname, value):
        sv = str(value)
        for sec in self._parser.sections():
            c = self._parser.items(sec)
            for (name, value) in c:
                if name == itemname and value == sv:
                    return sec
        return None


    def _value_parse(self, name, value):
        return value


    def sections(self):
        return self._parser.sections()
    
    def sections_ordered(self):
        secs = self._parser.sections()
        for sec in secs:
            opts = self._parser.options(sec)
            if not self._parser.has_option(sec, OPT_ORDER):
                return secs #has no order option
        result = []
        for n in xrange(len(secs)):
            for sec in secs:
                order = self._parser.getint(sec, OPT_ORDER)
                if order == n:
                    result.append(sec)
        return result

    def has_section(self, section):
        return self._parser.has_section(section)

    def show(self):
        for sec in self._parser.sections():
            print "Section[%s]" % (sec)
            for i in self._parser.items(sec):
                print "  " + str(i)




class ScanConfig(AbstractConfig):
    def __init__(self):
        AbstractConfig.__init__(self, SCAN_PRESET_FILENAME)
    
    def _value_parse(self, name, value):
        if name == OPT_SCANSRC:
            return int(value)
        elif name == OPT_SCANMODE:
            return int(value)
        elif name == OPT_SCANRES:
            return float(value)
        elif name == OPT_DUPSCAN:
            return int(value)
        elif name == OPT_PAPERSIZE:
            pass
        elif name == OPT_OUTFMT:
            pass
        elif name == OPT_OUTDIR:
            m = re.match(r"^\$\((.*)\)$", value)
            if m == None:
                m = re.match(r"^\${(.*)}$", value)
                if m == None:
                    m = re.match(r"^\$(.*)$", value)
                    if m == None:
                        return os.path.expanduser(value)
            return get_user_special_dir(m.groups()[0])
        elif name == OPT_OUTFILE:
            pass
        elif name == OPT_EXEAPP:
            pass
        elif name == OPT_ICON:
            pass
        elif name == OPT_ORDER:
            return int(value)
        return value




class PushScanConfig(AbstractConfig):
    def __init__(self):
        AbstractConfig.__init__(self, PUSH_CONFIG_FILENAME)
    
    def _value_parse(self, name, value):
        logging.debug("%s: %s" % (name, value))
        if name == OPT_PRESET:
            pass
        elif name == OPT_SHOW_SCAN_DIALOG:
            if value == V_TRUE:
                return True
            return False
        elif name == OPT_ORDER:
            return int(value)
        return value
    
    def in_used(self, presetname):
        for section in self._parser.sections():
            if self._parser.has_option(section, OPT_PRESET):
                v = self._parser.get(section, OPT_PRESET)
                if v == presetname:
                    return True
        return False




class ApplicationConfig(AbstractConfig):
    def __init__(self):
        AbstractConfig.__init__(self, APP_CONFIG_FILENAME)
    
    def _value_parse(self, name, value):
        if name == OPT_COMMAND:
            pass
        elif name == OPT_ORDER:
            return int(value)
        return value




class GeneralConfig(AbstractConfig):
    def __init__(self):
        AbstractConfig.__init__(self, GEN_CONFIG_FILENAME)
    
    def _value_parse(self, name, value):
        if name == OPT_NOTIFICATION:
            pass
        elif name == OPT_HELP_VIEWER:
            if value is not None and len(value) is 0:
                return None
        elif name == OPT_ORDER:
            return int(value)
        return value



