#!/usr/bin/python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
papersize.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"

# Directories
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(UTL_BASEDIR)
sys.path.append(".")

import locale
locale.setlocale(locale.LC_ALL, '')

import logging

from defs import *


def mmtoinch(mm):
    return mm / MM_INCH


class SizeValue:
    def __init__(self, mm=0.0, inch=0.0, pt=0.0):
        self.value = 0.0
        self.unit = ''
        if mm != 0:
            self.unit = UNIT_MM
            self.value = mm
        elif inch != 0:
            self.unit = UNIT_INCH
            self.value = inch
        elif pt != 0:
            self.unit = UNIT_POINT
            self.value = pt
        else:
            raise RuntimeError("value not specified.")
    
    def __str__(self):
        return self.toStr()
    
    def toStr(self, show_unit=True, use_unit=None, fmt="%.1f"):
        if use_unit == None:
            use_unit = self.unit
        
        v = 0
        if use_unit == UNIT_MM:
            v = self.mmValue()
        elif use_unit == UNIT_INCH:
            v = self.inchValue()
        elif use_unit == UNIT_POINT:
            v = self.ptValue()
        else:
            raise RuntimeError("Illegal unit. use_unit=%d" % (use_unit))
        
        sv = locale.format(fmt, v)
        if show_unit:
            return "%s [%s]" % (sv, UNIT_LOCAL[use_unit])
        else:
            return sv
    
    def mmValue(self):
        if self.unit == UNIT_MM:
            return self.value
        elif self.unit == UNIT_INCH:
            return self.value * MM_INCH
        elif self.unit == UNIT_POINT:
            return (self.value / POINT_INCH) * MM_INCH
        else:
            raise RuntimeError("Illegal unit. unit=%d" % (self.unit))
    
    def inchValue(self):
        if self.unit == UNIT_MM:
            return self.value / MM_INCH
        elif self.unit == UNIT_INCH:
            return self.value
        elif self.unit == UNIT_POINT:
            return self.value / POINT_INCH
        else:
            raise RuntimeError("Illegal unit. unit=%d" % (self.unit))
    
    def ptValue(self):
        if self.unit == UNIT_MM:
            return (self.value / MM_INCH) * POINT_INCH
        elif self.unit == UNIT_INCH:
            return self.value * POINT_INCH
        elif self.unit == UNIT_POINT:
            return self.value
        else:
            raise RuntimeError("Illegal unit. unit=%d" % (self.unit))


class PaperSize:
    def __init__(self, displayname, width=0.0, height=0.0, width_inch=0.0, height_inch=0.0, width_pt=0.0, height_pt=0.0, pagetypename=None, gstype=True):
        self._name = displayname
        self._width = SizeValue(mm=width, inch=width_inch, pt=width_pt)
        self._height = SizeValue(mm=height, inch=height_inch, pt=height_pt)
        self._type = pagetypename
        if pagetypename != None:
            self._gstype = gstype
        else:
            self._gstype = False

    def __str__(self):
        return self.toStr()
        
    def toStr(self, use_unit=None):
        return "%s (%s)" % (self.displayname(), self.sizestr(use_unit))
    
    def sizestr(self, use_unit=None):
        if self._width.unit == self._height.unit:
            return "%s x %s" % (self._width.toStr(show_unit=False, use_unit=use_unit), self._height.toStr(use_unit=use_unit))
        else:
            return "%s x %s" % (self._width.toStr(use_unit=use_unit), self._height.toStr(use_unit=use_unit))
    
    def displayname(self):
        return self._name
    
    def pagetypename(self):
        """Get Gostscript pagetypename."""
        return self._type
    
    def is_gs_pagetypename(self):
        return self._gstype
    
    def width(self):
        return self._width.mmValue()
    
    def height(self):
        return self._height.mmValue()
    
    def papersize(self):
        return (self._width.mmValue(), self._height.mmValue())
    
    def width_inch(self):
        return self._width.inchValue()
    
    def height_inch(self):
        return self._height.inchValue()
    
    def papersize_inch(self):
        return (self._width.inchValue(), self._height.inchValue())
    
    def width_pt(self):
        return self._width.ptValue()
    
    def height_pt(self):
        return self._height.ptValue()
    
    def papersize_pt(self):
        return (self._width.ptValue(), self._height.ptValue())




#Ａサイズ
A0=PaperSize(PAPERSIZENAME_A0, width=841.0, height=1189.0, pagetypename="a0")
A1=PaperSize(PAPERSIZENAME_A1, width=594.0, height=841.0, pagetypename="a1")
A2=PaperSize(PAPERSIZENAME_A2, width=420.0, height=594.0, pagetypename="a2")
A3=PaperSize(PAPERSIZENAME_A3, width=297.0, height=420.0, pagetypename="a3")
A4=PaperSize(PAPERSIZENAME_A4, width=210.0, height=297.0, pagetypename="a4")
A5=PaperSize(PAPERSIZENAME_A5, width=148.0, height=210.0, pagetypename="a5")
A6=PaperSize(PAPERSIZENAME_A6, width=105.0, height=148.0, pagetypename="a6")
A7=PaperSize(PAPERSIZENAME_A7, width=74.0, height=105.0, pagetypename="a7")
A8=PaperSize(PAPERSIZENAME_A8, width=52.0, height=74.0, pagetypename="a8")
A9=PaperSize(PAPERSIZENAME_A9, width=37.0, height=52.0, pagetypename="a9")
A10=PaperSize(PAPERSIZENAME_A10, width=26.0, height=37.0, pagetypename="a10")
A11=PaperSize(PAPERSIZENAME_A11, width=18.0, height=26.0)
A12=PaperSize(PAPERSIZENAME_A12, width=13.0, height=18.0)

#Ｂサイズ
B0=PaperSize(PAPERSIZENAME_B0, width=1030.0, height=1456.0, pagetypename="b0")
B1=PaperSize(PAPERSIZENAME_B1, width=728.0, height=1030.0, pagetypename="b1")
B2=PaperSize(PAPERSIZENAME_B2, width=515.0, height=728.0, pagetypename="b2")
B3=PaperSize(PAPERSIZENAME_B3, width=364.0, height=515.0, pagetypename="b3")
B4=PaperSize(PAPERSIZENAME_B4, width=257.0, height=364.0, pagetypename="b4")
B5=PaperSize(PAPERSIZENAME_B5, width=182.0, height=257.0, pagetypename="b5")
B6=PaperSize(PAPERSIZENAME_B6, width=128.0, height=182.0, pagetypename="b6")
B7=PaperSize(PAPERSIZENAME_B7, width=91.0, height=128.0)
B8=PaperSize(PAPERSIZENAME_B8, width=64.0, height=91.0)
B9=PaperSize(PAPERSIZENAME_B9, width=45.0, height=64.0)
B10=PaperSize(PAPERSIZENAME_B10, width=32.0, height=54.0)
B11=PaperSize(PAPERSIZENAME_B11, width=22.0, height=32.0)
B12=PaperSize(PAPERSIZENAME_B12, width=16.0, height=22.0)


#レター（国際判便箋）
#8.5 インチ × 11 インチ
LETTER=PaperSize(PAPERSIZENAME_LETTER, width_inch=8.5, height_inch=11.0, pagetypename="letter")

#リーガル（米政府公文書）
#8.5 インチ × 14 インチ
LEAGAL=PaperSize(PAPERSIZENAME_LEGAL, width_inch=8.5, height_inch=14.0, pagetypename="legal")

#リーガル
#8.5 インチ × 13.5 インチ
LEAGAL135=PaperSize(PAPERSIZENAME_LEGAL135, width_inch=8.5, height_inch=13.5, pagetypename="legal 13.5", gstype=False)

#リーガル
#8.5 インチ × 13 インチ
FLSA=PaperSize(PAPERSIZENAME_FLSA, width_inch=8.5, height_inch=13.0, pagetypename="flsa") #U.S. foolscap 
FLSE=PaperSize(PAPERSIZENAME_FLSE, width_inch=8.5, height_inch=13.0, pagetypename="flse") #European foolscap 

#Executive
#7.25 インチ × 10.5 インチ
EXECUTIVE=PaperSize(PAPERSIZENAME_EXECUTIVE, width_inch=7.25, height_inch=10.5, pagetypename="executive", gstype=False)


#All paper sizes
PAPERSIZES = [
    A0, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11, A12,
    B0, B1, B2, B3, B4, B5, B6, B7, B8, B9, B10, B11, B12,
    LETTER, LEAGAL, LEAGAL135, FLSA, FLSE, EXECUTIVE]


def get_papersize_by_displayname(displayname):
    for p in PAPERSIZES:
        if p.displayname() == displayname:
            return p
    logging.warn("'%s' not found." % (displayname))
    return None

def get_papersize_by_pagetypename(pagetypename):
    for p in PAPERSIZES:
        if p.pagetypename() == pagetypename:
            return p
    logging.warn("'%s' not found." % (pagetypename))
    return None


def get_displaynamelist(papersizelist):
    l = []
    for psz in papersizelist:
        l.append(psz.displayname())
    l.sort()
    return l



if __name__ == "__main__":
    logging.basicConfig(format=LOGFORMAT)
    if len(sys.argv) > 1:
        rl = logging.getLogger()
        rl.setLevel(LOGLEVELS[sys.argv[1]])
    
    for p in PAPERSIZES:
        print p
        print p.toStr(UNIT_MM)
        print p.toStr(UNIT_INCH)
        print p.toStr(UNIT_POINT)
    p = PaperSize("Test1", width_pt=100, height_pt=200, pagetypename="custom", gstype=False)
    print p
    print p.toStr(UNIT_MM)
    print p.toStr(UNIT_INCH)
    print p.toStr(UNIT_POINT)
    p = PaperSize("Test2", width_inch=1.5, height_pt=200, pagetypename="custom", gstype=False)
    print p
    print p.toStr(UNIT_MM)
    print p.toStr(UNIT_INCH)
    print p.toStr(UNIT_POINT)

