#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
PushScan assistant.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"
__icon__ = "/usr/libexec/okimfputl/images/exeicon48x48.png"

# Directories
DRV_BASEDIR="/usr/libexec/okimfpdrv/"
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(DRV_BASEDIR)
sys.path.append(UTL_BASEDIR)
sys.path.append("../driver/.libs/")#DEBUG
sys.path.append(".")

import locale
locale.setlocale(locale.LC_ALL, '')

import os
import signal
import time

import pygtk
pygtk.require('2.0')
import gtk
#import pango
import threading
import exceptions
import logging

from defs import *
from mfpcfgfile import *
from guicom import *
import pyscand
import scantool




class PushScanDialog(scantool._ScanDialog):
    # PushScanDialog List
    dlgs = WindowList()
    
    def __init__(self, parent, idstr, devname, presetname, dict):
        #parent, title, presetname, dict, show_start_btn=True, devlist=None, actdev=None, width=320, height=260
        scantool._ScanDialog.__init__(self, parent, TITLE_SCAN+" - "+TITLE_PUSH_ASSISTANT, presetname, dict, False, [devname], devname)
        self.set_position(gtk.WIN_POS_CENTER)
        self.set_selectable_devicecb(False)
        # Set the help document URI for this window
        self.setHelpUri("file://"+UTLDOC_DIR+"/scantool-scan.html")
        
        msg = MSG_PUSHSCAN_ACCEPT % {MSG_PUSHSCAN_ACCEPT_ID:idstr, MSG_PUSHSCAN_ACCEPT_DEVICE:devname}
        self.msgview.append(msg)
        
        PushScanDialog.dlgs.append(self)
    
    def onClose(self):
        scantool._ScanDialog.onClose(self)
        PushScanDialog.dlgs.remove(self)
        return True




class ScanCallback(NotificationUI):
    def __init__(self, icon_image=None):
        NotificationUI.__init__(self, icon_image)
        self.__show_prog = True
        try:
            if self.get_vendor() == 'KDE':
                self.__show_prog = False
        except:
            pass
    
    def callback(self, type, args):
        path = ""
        s = ""
        if type == pyscand.PROGRESS:
            status = args[pyscand.PRM_STATUS]
            path = args[pyscand.PRM_PATH]
            prog = args[pyscand.PRM_PROGRESS]
            source = args[pyscand.PRM_SOURCE]
            page = args[pyscand.PRM_PAGE]
            #print "status=%s" % pyscand.strstatus(status)
            if status == pyscand.E_BUSY:
                #Wait for scan to begin / Scanning
                if prog == 0:
                    s = FORMAT_PROGRESS_BEGIN % {FORMAT_PROGRESS_PAGENUMBER:page}
                    if source == pyscand.ADF:
                        pass
                else:
                    strprog = locale.format("%.1f", prog)
                    s = FORMAT_PROGRESS % {FORMAT_PROGRESS_PAGENUMBER:page, FORMAT_PROGRESS_PERCENTAGE:strprog}
                    if source == pyscand.ADF:
                        pass
                if self.__show_prog:
                    self.show_msg(s)
                else:
                    logging.info(s)
            elif status == pyscand.E_END_OF_PAGE or status == pyscand.E_SUCCESS:
                #Scan Complete
                s = FORMAT_PROGRESS_FINISHED % {FORMAT_PROGRESS_PAGENUMBER:page}
                self.show_msg(s)
            elif status == pyscand.E_CANCELED:
                s = FORMAT_PROGRESS_CANCELED % {FORMAT_PROGRESS_PAGENUMBER:page}
                self.show_msg(s)
            else:
                #Error
                s = FORMAT_PROGRESS_FAIL % {FORMAT_PROGRESS_DETAIL:pyscand.strstatus(status)}
                self.show_err_msg(msg=s)



class PushScanAssistant(threading.Thread,NotificationUI):
    def __init__(self, callback=None):
        """
        def callback(registered)
        registered: True/False
        """
        threading.Thread.__init__(self)
        NotificationUI.__init__(self, ICON_EXEICON48x48_IMG_PATH)
        self.set_msg_title(TITLE_PUSH_ASSISTANT)
        self.__run = True
        self.__end = False
        self.__canceled = False
        self._autoreg = True
        self._autoreg_setval = True
        self._callback = callback
    
    def _push_scan(self, param):
        logging.debug("StartPushScan:  param=" + str(param))
        toxxx = ""
        idstr=""
        if param[pyscand.PRM_ACTION] == pyscand.TO_FOLDER:
            toxxx = PUSHCONF_TOFOLDER
            idstr = S_SCAN_TO_FOLDER
        elif param[pyscand.PRM_ACTION] == pyscand.TO_APPLICATION:
            toxxx = PUSHCONF_TOAPPLICATION
            idstr = S_SCAN_TO_APP
        else:
            logging.warn("unsupported pushid. (id=%d)" % (param[pyscand.PRM_ACTION]))
            return
        
        pushscanid = param[pyscand.PRM_PUSHSCAN_ID]
        devname = param[pyscand.PRM_DEVICE]
        
        if use_notification():
            self.show_msg(MSG_PUSHSCAN_ACCEPT % {MSG_PUSHSCAN_ACCEPT_ID:idstr, MSG_PUSHSCAN_ACCEPT_DEVICE:devname}, mt=True)
        
        pushconf = PushScanConfig()
        pushconf.load()
        show_dialog = pushconf.get(SEC_GENERAL, OPT_SHOW_SCAN_DIALOG)
        pscfg = pushconf.gets(toxxx)
        if pscfg == None:
            self.show_warn_msg(0, MSG_PUSHSCAN_INVCFG % idstr, mt=True)
            return
        preset = pscfg[OPT_PRESET]
        if preset == None or len(preset) == 0:
            self.show_warn_msg(0, MSG_PUSHSCAN_NOTCFG % idstr, mt=True)
            return
        
        
        scanconf = ScanConfig()
        scanconf.load()
        sccfg = scanconf.gets(preset)
        if sccfg == None:
            self.show_warn_msg(0, MSG_SCCNF_NOT_EXIST % (preset), mt=True)
            return
        
        
        if use_notification() and not (show_dialog):
            cb = ScanCallback(scantool.get_icon_path(sccfg))
            cb.set_msg_title(devname)
            errui = NotifyErrMgsUI(cb.notification, devname)
            status = scantool.scan_manager.new_thread(devname, preset, sccfg, cb.callback, errui, pushscanid)
            if not status == pyscand.E_SUCCESS:
                self.show_err_msg(status, MSG_SCAN_START_NG, mt=True)
        else:
            gtk.gdk.threads_enter()
            cb = PushScanDialog(None, idstr, devname, preset, sccfg)
            gtk.gdk.threads_leave()
            errui = DlgErrMgsUI(cb, TITLE_PUSH_ASSISTANT)
            status = scantool.scan_manager.new_thread(devname, preset, sccfg, cb.callback, errui, pushscanid)
            if not status == pyscand.E_SUCCESS:
                self.show_err_msg(status, MSG_SCAN_START_NG, mt=True)
    
    def _cancel_push_scan(self, param):
        logging.debug("CancelPushScan: param=" + str(param))
        if use_notification():
            self.show_msg(MSG_PUSHSCAN_CANCELD % (param[pyscand.PRM_DEVICE]))
        #scantool.scan_manager.delete_thread(param[pyscand.PRM_DEVICE])
    
    def register(self, notifyError=True, mt=False):
        logging.debug("do register")
        self._autoreg = self._autoreg_setval
        (status, user) = pyscand.register_client()
        if status == 0:
            #success
            logging.debug("register success")
            self.show_msg(MSG_PUSH_ACPT_STA, mt=mt)
            return True
        else:
            #error
            if notifyError:
                if status == pyscand.E_BUSY:
                    self.show_warn_msg(msg=MSG_ALREADY_REGUSR % (user), mt=mt)
                else:
                    self.show_err_msg(status, MSG_DAEMON_ACCESS_NG, mt=mt)
            logging.warn("register fail (status=%s [%d])" % (pyscand.strstatus(status), status))
            return False
        
    def cancel_register(self):
        if not self.__canceled:
            logging.debug("do cancel register")
            pyscand.cancel_register_client()
            self._autoreg = False
            self.__canceled = True
            logging.debug("success")
            return True
        return False
    
    def is_registered(self):
        return pyscand.is_registered()
    
    def set_auto_register(self, b):
        self._autoreg_setval = b
        self._autoreg = b
    
    def run(self):
        logging.info("PushAssistant main....")
        self.__end = False
        while self.__run:
            if pyscand.is_registered() == True:
                if not self._callback == None:
                    self._callback(True)
                while pyscand.is_registered() == True:
                    self.__canceled = False
                    logging.debug("recv event...")
                    (status, eventid, param) = pyscand.recv_client_event()
                    logging.debug("status=%s, eventid=%s, param=%s" % (status, eventid, param))
                    if status == pyscand.E_SUCCESS:
                        if eventid == pyscand.EVENT_PUSHSCAN:
                            self._push_scan(param)
                        elif eventid == pyscand.EVENT_PUSHSCANCCL:
                            self._cancel_push_scan(param)
                    elif status == pyscand.E_NODATA:
                        time.sleep(1)
                    else:
                        if self.__run and not self.__canceled:
                            self.show_err_msg(status, mt=True)
            
            if not self._callback == None:
                self._callback(False)
                
            t = time.time()
            while self.__run and not pyscand.is_registered():
                if self._autoreg:
                    # Auto register
                    nt = time.time()
                    if t + RETRY_INTERVAL_TIME < nt:
                        if self.register(False, True):
                            break
                        t = nt
                time.sleep(1)
        self.__end = True
        logging.info("PushAssistant main exit.")
        
    def stop(self):
        pyscand.exit_client()
        self.__run = False
        for i in xrange(10):
            time.sleep(1)
            if self.__end:
                return True
        return False


if __name__ == "__main__":
    import getopt
    logging.basicConfig(format=LOGFORMAT)
    opts, args = getopt.getopt(sys.argv[1:], 'l:', ['loglevel=', 'no-pynotify'])
    #print "args=", args
    #print "opts=", opts
    for k, v in opts:
        if k in ('-l', '--loglevel'):
            rl = logging.getLogger()
            rl.setLevel(LOGLEVELS[v])
            if LOGLEVELS[v] is logging.DEBUG:
                #gc.set_debug(gc.DEBUG_COLLECTABLE | gc.DEBUG_UNCOLLECTABLE | gc.DEBUG_INSTANCES | gc.DEBUG_OBJECTS | gc.DEBUG_SAVEALL)
                gc.set_debug(gc.DEBUG_LEAK)
        elif k in ('--no-pynotify'):
            disable_pynotify()
    
    # PushScanAssistant Instance
    inst = None
    
    initNotificationUI(TITLE_PUSH_ASSISTANT)
    
    def signal_handler(signum, frame):
        logging.info("Signal handler called with signal " + signum)
        if signum == signal.SIGTERM or signum == signal.SIGINT:
            logging.info("Abort")
            inst.stop()
            time.sleep(1)
            sys.exit(0)
    
    signal.signal(signal.SIGUSR1, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)
    #signal.signal(signal.SIGALRM, signal_handler)
    #signal.alarm(1)
    
    inst = PushScanAssistant()
    inst.setDaemon(True)
    if inst.register():
        inst.start()
        # wait for signal
        while True:
            try:
                gtk.main()
            except KeyboardInterrupt,e:
                logging.info("Abort")
                sys.exit(0)
            time.sleep(1)
        
