#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
manipulate PushScan configuration files.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"
__icon__ = "/usr/libexec/okimfputl/images/exeicon48x48.png"

# Directories
DRV_BASEDIR="/usr/libexec/okimfpdrv/"
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(DRV_BASEDIR)
sys.path.append(UTL_BASEDIR)
sys.path.append("../driver/.libs/")#DEBUG
sys.path.append(".")

import locale
locale.setlocale(locale.LC_ALL, '')

import pygtk
pygtk.require('2.0')
import gtk
import gtk.gdk
import gobject
import logging

from guicom import *
from defs import *
from mfpcfgfile import *
import genconfig
PUSHCONFIG_WIN_HEIGHT = 90


class PushScanConfigPanel(AbstractConfigPanel):
    def __init__(self, padding=8):
        AbstractConfigPanel.__init__(self, LABEL_PUSHSCAN, padding, "file://"+UTLDOC_DIR+"/pushconfig.html")
        
        self._scanconf = ScanConfig()
        self._scanconf.load()
        self._pushconf = PushScanConfig()
        self._pushconf.load()
        
        self._sellist = [VALUE_NONE] + self._scanconf.sections_ordered()
        
        
        f = VBoxFrame(label=LABEL_PUSH_CONFIG)
        self.pack_start(f, False, padding=0)
        
        act = self._pushconf.get(PUSHCONF_TOFOLDER, OPT_PRESET)
        cb = build_ComboBox(self._sellist)
        if not set_active_ComboBox(cb, act):
            self._pushconf.set(PUSHCONF_TOFOLDER, OPT_PRESET, None)
        cb.connect('changed', self._chgcb_tofolder)
        self.tofolder_preset = ItemHBox(LABEL_SCAN_TO_FOLDER, cb, padding=0)
        f.pack_start(self.tofolder_preset, False, padding=0)
        
        
        act = self._pushconf.get(PUSHCONF_TOAPPLICATION, OPT_PRESET)
        cb = build_ComboBox(self._sellist)
        if not set_active_ComboBox(cb, act):
            self._pushconf.set(PUSHCONF_TOAPPLICATION, OPT_PRESET, None)
        cb.connect('changed', self._chgcb_toapplication)
        self.toapp_preset = ItemHBox(LABEL_SCAN_TO_APP, cb, padding=0)
        f.pack_start(self.toapp_preset, False, padding=0)
        
        
        f = VBoxFrame(label=LABEL_PUSH_SCAN_STATUS)
        self.pack_start(f, False, padding=0)
        
        self.show_scandlg = gtk.CheckButton(LABEL_SHOW_DIALOG)
        self.show_scandlg.connect('toggled', self._change_show_scandlg)
        f.pack_start(self.show_scandlg, False, padding=0)
        
        self.update()
    
    def _chgcb_tofolder(self, cb, *args):
        name = cb.get_active_text()
        logging.debug("to folder: %s" % (name))
        if name == VALUE_NONE:
            name = None
        self.set_changed(True)
    
    def _chgcb_toapplication(self, cb, *args):
        name = cb.get_active_text()
        logging.debug("to application: %s" % (name))
        if name == VALUE_NONE:
            name = None
        self.set_changed(True)
    
    def _change_show_scandlg(self, *args):
        self.set_changed(True)
    
    def save(self):
        name = self.tofolder_preset.item.get_active_text()
        logging.debug("to folder: %s" % (name))
        if name == VALUE_NONE:
            name = None
        self._pushconf.set(PUSHCONF_TOFOLDER, OPT_PRESET, name)
        
        name = self.toapp_preset.item.get_active_text()
        logging.debug("to application: %s" % (name))
        if name == VALUE_NONE:
            name = None
        self._pushconf.set(PUSHCONF_TOAPPLICATION, OPT_PRESET, name)
        
        if has_notification_function():
            notify_method = self._genconf.get(SEC_GENERAL, OPT_NOTIFICATION)
            if notify_method is None or notify_method == V_USE_PYNOTIFY:
                if self.show_scandlg.get_active():
                    self._pushconf.set(SEC_GENERAL, OPT_SHOW_SCAN_DIALOG, True)
                else:
                    self._pushconf.set(SEC_GENERAL, OPT_SHOW_SCAN_DIALOG, False)
        
        self._pushconf.save()
        logging.info("saved.")
    
    def update(self):
        self._genconf = GeneralConfig()
        self._genconf.load()
        logging.debug(str(self._genconf.gets(SEC_GENERAL)))
        
        act = self._pushconf.get(PUSHCONF_TOFOLDER, OPT_PRESET)
        set_active_ComboBox(self.tofolder_preset.item, act)
        
        act = self._pushconf.get(PUSHCONF_TOAPPLICATION, OPT_PRESET)
        set_active_ComboBox(self.toapp_preset.item, act)
        
        if has_notification_function():
            notify_method = self._genconf.get(SEC_GENERAL, OPT_NOTIFICATION)
            if notify_method is None or notify_method == V_USE_PYNOTIFY:
                show_dialog = self._pushconf.get(SEC_GENERAL, OPT_SHOW_SCAN_DIALOG)
                if show_dialog is None:
                    show_dialog = False
                self.show_scandlg.set_active(show_dialog)
                self.show_scandlg.set_sensitive(True)
            else:
                self.show_scandlg.set_active(True)
                self.show_scandlg.set_sensitive(False)
        else:
            self.show_scandlg.set_active(True)
            self.show_scandlg.set_sensitive(False)




class PushConfigDialog(CommonPalletWindow):
    
    def onOk(self):
        notify_method = self._genconf.get(SEC_GENERAL, OPT_NOTIFICATION)
        if notify_method is None or notify_method == V_USE_PYNOTIFY:
            if self.show_scandlg.get_active():
                self._pushconf.set(SEC_GENERAL, OPT_SHOW_SCAN_DIALOG, True)
            else:
                self._pushconf.set(SEC_GENERAL, OPT_SHOW_SCAN_DIALOG, False)
        
        self._pushconf.save()
        return True
    
    def __init__(self):
        CommonPalletWindow.__init__(self, TITLE_PUSH_CONFIG)
        # Set the help document URI for this window
        self.setHelpUri("file://"+UTLDOC_DIR+"/pushconfig.html")
        
        self.set_icon_from_file(__icon__)
        
        
        self.vbox.pack_start(PushScanConfigPanel(padding=0))
        
        
        self.add_button(BUTTON_OK, gtk.RESPONSE_OK)
        self.add_button(BUTTON_CANCEL, gtk.RESPONSE_CANCEL)
        
        
        self.resize(PUSHCONFIG_WIN_WIDTH, PUSHCONFIG_WIN_HEIGHT)






if __name__ == "__main__":
    import getopt
    logging.basicConfig(format=LOGFORMAT)
    opts, args = getopt.getopt(sys.argv[1:], 'l:', ['loglevel=', 'no-pynotify'])
    #print "args=", args
    #print "opts=", opts
    for k, v in opts:
        if k in ('-l', '--loglevel'):
            rl = logging.getLogger()
            rl.setLevel(LOGLEVELS[v])
            if LOGLEVELS[v] is logging.DEBUG:
                #gc.set_debug(gc.DEBUG_COLLECTABLE | gc.DEBUG_UNCOLLECTABLE | gc.DEBUG_INSTANCES | gc.DEBUG_OBJECTS | gc.DEBUG_SAVEALL)
                gc.set_debug(gc.DEBUG_LEAK)
        elif k in ('--no-pynotify'):
            disable_pynotify()
    
    genconfig.configure()
    dlg = PushConfigDialog()
    dlg.gtkmain()



