#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright (c) 2012 Oki Data Corporation. All rights reserved.

## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, write to the Free Software Foundation, Inc., 59 Temple
## Place, Suite 330, Boston, MA  02111-1307  USA

"""
Scan Tool.
"""
__author__ = " <@>"
__copyright__ = "Copyright (c) 2012 Oki Data Corporation. All rights reserved."
__credits__ = []
__license__ = ""
__version__ = "1.7-0"
__date__ = "2012"
__maintainer__ = ""
__email__ = ""
__status__ = "Production"
__icon__ = "/usr/libexec/okimfputl/images/exeicon48x48.png"

# Directories
DRV_BASEDIR="/usr/libexec/okimfpdrv/"
UTL_BASEDIR="/usr/libexec/okimfputl/"

import sys
sys.path.append(DRV_BASEDIR)
sys.path.append(UTL_BASEDIR)
sys.path.append("../driver/")#DEBUG
sys.path.append("../driver/.libs/")#DEBUG
sys.path.append(".")

import locale
locale.setlocale(locale.LC_ALL, '')

import os
import os.path
import pygtk
pygtk.require('2.0')
import gtk
import gtk.gdk
import gobject
import pango
gtk.gdk.threads_init()

import time
import threading
import exceptions
import logging
import gc

from defs import *
from mfpcfgfile import *
from guicom import *
import genconfig
import imgfilecv
import pyscand
import pyscandsupp

import appconfig
import papersize

__papersizes__ = [
    papersize.A3,
    papersize.A4,
    papersize.A5,
    papersize.A6,
    papersize.B4,
    papersize.B5,
    papersize.LEAGAL,
    papersize.LEAGAL135,
    papersize.FLSA,
    papersize.EXECUTIVE,
    papersize.LETTER
]


#ICON_START = gtk.gdk.pixbuf_new_from_file(ICON_START_IMG_PATH)
#ICON_STOP = gtk.gdk.pixbuf_new_from_file(ICON_STOP_IMG_PATH)
MAXICONSZ = 64
COMBOBOX_PRESET_NAME_WIDTH = 186
SCAN_WIN_WIDTH = 360
SCAN_WIN_HEIGHT = 280
SCANTOOL_WIN_WIDTH = 274
SCANTOOL_WIN_HEIGHT = 360


def get_icon_path(cfgdict):
    """get iconfile-path from scan-configuraton dict."""
    try:
        iconpath = cfgdict[OPT_ICON]
        if iconpath == BUILTIN_ICON_TOFOLDER:
            iconpath = ICON_TOFOLDER_IMG_PATH
        elif iconpath == BUILTIN_ICON_TOAPP:
            iconpath = ICON_TOAPP_IMG_PATH
        else:
            pass
    except KeyError, e:
        iconpath = ICON_TOFOLDER_IMG_PATH
    return iconpath


def get_icon(cfgdict):
    """get icon from scan-configuraton dict."""
    try:
        path = get_icon_path(cfgdict)
        if path == None or len(path) == 0:
            return None
        pb = gtk.gdk.pixbuf_new_from_file(path)
        w = pb.get_width()
        h = pb.get_height()
        #print w, h
        if w > h:
            if w > MAXICONSZ:
                pb = pb.scale_simple(MAXICONSZ, int(h*(MAXICONSZ/float(w))), gtk.gdk.INTERP_BILINEAR)
        elif w < h:
            if h > MAXICONSZ:
                pb = pb.scale_simple(int(w*(MAXICONSZ/float(h))), MAXICONSZ, gtk.gdk.INTERP_BILINEAR)
        else:
            #same
            if w > MAXICONSZ:
                pb = pb.scale_simple(MAXICONSZ, MAXICONSZ, gtk.gdk.INTERP_BILINEAR)
        return pb
    except Exception, e:
        logging.error(str(e))
        return None




class PresetItem(gtk.DrawingArea):
    ICONSZ = MAXICONSZ
    SEP = 4
    LABELW = 192
    H = ICONSZ
    W = ICONSZ + SEP + LABELW
    BORDER = 1
    PAD = 1
    OH = H + (BORDER + PAD)*2
    OW = W + (BORDER + PAD)*2
    FGCOLOR = gtk.gdk.Color('#000000')
    BGCOLOR = gtk.gdk.Color('#FFFFFF')
    
    def __init__(self, presetname, dict):
        gtk.DrawingArea.__init__(self)
        self.set_size_request(PresetItem.OW, PresetItem.OH)
        self.connect('expose-event', self.expose_cb)
        self.pbicon = get_icon(dict)
        w = self.pbicon.get_width()
        h = self.pbicon.get_height()
        self.adj_icon_x = 0
        self.adj_icon_y = 0
        if w < MAXICONSZ:
            self.adj_icon_x = (MAXICONSZ - w) / 2
        if h < MAXICONSZ:
            self.adj_icon_y = (MAXICONSZ - h) / 2
        #print "size=", w, h
        #print "adj=", self.adj_icon_x, self.adj_icon_y
        self.presetname = presetname
        self.presetdict = dict
        self.__buildcontent__()
        self.show()
    
    def __buildcontent__(self):
        pm = gtk.gdk.Pixmap(None, PresetItem.OW, PresetItem.OH, 24)
        gc = pm.new_gc()
        gc.set_function(gtk.gdk.COPY)
        gc.set_fill(gtk.gdk.SOLID)
        gc.set_line_attributes(1, gtk.gdk.LINE_SOLID, 0, 0)
        gc.set_rgb_fg_color(PresetItem.BGCOLOR)
        gc.set_rgb_bg_color(PresetItem.BGCOLOR)
        pm.draw_rectangle(gc, True, 0, 0, PresetItem.OW-1, PresetItem.OH-1)
        gc.set_rgb_fg_color(PresetItem.FGCOLOR)
        gc.set_rgb_bg_color(PresetItem.BGCOLOR)
        pm.draw_rectangle(gc, False, 0, 0, PresetItem.OW-1, PresetItem.OH-1)
        
        pm.draw_pixbuf(gc, self.pbicon, 0, 0, PresetItem.BORDER+PresetItem.PAD+self.adj_icon_x, PresetItem.BORDER+PresetItem.PAD+self.adj_icon_y)
        
        lbl = gtk.Label()
        lbl.set_size_request(PresetItem.LABELW, PresetItem.H)
        lbl.set_line_wrap(True)
        lbl.set_selectable(False)
        lbl.set_padding(4, 0)
        lbl.set_text(self.presetname)
        layout = lbl.get_layout()
        (w, h) = layout.get_size()
        x = PresetItem.BORDER + PresetItem.PAD + PresetItem.ICONSZ + PresetItem.SEP
        y = PresetItem.BORDER + PresetItem.PAD + (PresetItem.H - (h / pango.SCALE)) / 2
        pm.draw_layout(gc, x, y, layout, PresetItem.FGCOLOR)
        
        self.__content__ = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, PresetItem.OW, PresetItem.OH)
        self.__content__.get_from_drawable(pm, pm.get_colormap(), 0, 0, 0, 0, PresetItem.OW, PresetItem.OH)
    
    def expose_cb(self, area, *args):
        if self.window is None:
            return True
        gc = self.window.new_gc()
        self.window.draw_pixbuf(gc, self.__content__, 0, 0, 0, 0)
        return True




class ScanThread(threading.Thread):
    
    """Scan Thread"""
    
    def __init__(self, device, presetname, cfg, callback=None, errui=None, pushscanid=0):
        threading.Thread.__init__(self)
        self.setDaemon(True)
        if errui == None:
            self._errui = ErrorMessageUI()
        else:
            self._errui = errui
        self._device = device
        self._presetname = presetname
        self._cfg = cfg
        if callback == None:
            self._user_callback = self._callback
        else:
            self._user_callback = callback
        self._pushscanid = pushscanid
        self.outputs = []
    
    
    def _scan_callback(self, type, args):
        try:
            if type == pyscand.PROGRESS:
                status = args["status"]
                path = args["path"]
                #prog = args["progress"]
                #source = args["source"]
                #page = args["page"]
                #print "status=%s" % pyscand.strstatus(status)
                if status == pyscand.E_BUSY:
                    #Wait for scan to begin / Scanning
                    pass
                elif status == pyscand.E_END_OF_PAGE or status == pyscand.E_SUCCESS:
                    #Scan Complete
                    self.outputs.append(path)
                else:
                    pass
            if not self._user_callback == None:
                self._user_callback(type, args)
        except Exception,e:
            logging.error("_scan_callback error. exception="+str(e))
    
    
    def run(self):
        try:
            logging.debug("start ScanThread device=%s, preset=%s, pushscanid=%d" %
                          (self._device, self._presetname, self._pushscanid))
            
            apconf = ApplicationConfig()
            apconf.load()
            
            outputdir = SPV_DEF_OUTDIR
            val = get_value(self._cfg, OPT_OUTDIR)
            if not val == None:
                outputdir = os.path.expanduser(val)
            outputdir = os.path.realpath(outputdir)
            
            if os.path.isdir(outputdir) == False:
                self._errui.error(MSG_OUTDIR_NOTEXISTS, outputdir)
                self._scan_callback(CB_ERRORMSG_IND, {CB_ARG_MSG:MSG_OUTDIR_NOTEXISTS})
                return
            
            filename = SPV_DEF_OUTFILE
            val = get_value(self._cfg, OPT_OUTFILE)
            if not val == None:
                filename = val
            
            ext = ""
            val = get_value(self._cfg, OPT_OUTFMT)
            if not val == None:
                ext = val
            
            execcmd = get_value(self._cfg, OPT_EXEAPP)
    
            # Output file path
            tsstr = time.strftime("%Y%m%d%H%M%S", time.localtime())
            outpath = "%s/%s-%s-%s" % (outputdir, filename, self._device, tsstr)
            
            # Paper size -> Geometory setting
            val = get_value(self._cfg, OPT_PAPERSIZE)
            if not val == None:
                psize = papersize.get_papersize_by_pagetypename(val)
                logging.debug("Paper size=%s" % (psize))
                self._cfg[pyscand.O_WIDTH] = psize.width()
                self._cfg[pyscand.O_HEIGHT] = psize.height()
                self._cfg[pyscand.O_PAPER_WIDTH] = psize.width()
                self._cfg[pyscand.O_PAPER_HEIGHT] = psize.height()
            
            # Resolution
            resolution = 75 #dpi
            val = get_value(self._cfg, OPT_SCANRES)
            if not val == None:
                resolution = val
            
            
            status = pyscand.scan(self._device, self._cfg, outpath, self._scan_callback, self._pushscanid)
            if status == pyscand.E_SUCCESS:
                logging.info("Scan successful.")
                
                # Convert File Format
                converter = None
                if ext == EXT_PNM:
                    pass
                elif ext == EXT_PNG:
                    converter = imgfilecv.PnmToPngConverter()
                elif ext == EXT_JPG:
                    converter = imgfilecv.PnmToJpegConverter()
                elif ext == EXT_TIF:
                    converter = imgfilecv.PnmToTiffConverter()
                elif ext == EXT_PDF:
                    converter = imgfilecv.PnmToPdfConverter(resolution=resolution, paper_size=psize)
                if not converter == None:
                    logging.info("Convert...")
                    gtk.gdk.threads_enter()
                    self._scan_callback(CB_INFO_IND, {CB_ARG_MSG:FORMAT_PROGRESS_CNVFMT % (ext)})
                    gtk.gdk.threads_leave()
                    try:
                        cvlist = converter.convert(self.outputs)
                        imgfilecv.deletefiles(self.outputs)
                        self.outputs = cvlist
                    except Exception, e:
                        logging.error("Convert Error: " + str(e))
                        gtk.gdk.threads_enter()
                        self._scan_callback(CB_ERRORMSG_IND, {CB_ARG_MSG:MSG_FILECONV_NG+" - "+str(e)})
                        gtk.gdk.threads_leave()
                        self._errui.error(MSG_FILECONV_NG, str(e))
                        return
                
                gtk.gdk.threads_enter()
                self._scan_callback(CB_OUTPUT_FILE, {CB_ARG_FILE:self.outputs})
                gtk.gdk.threads_leave()
                
                # Execute Application
                if execcmd is not None and not len(execcmd) == 0:
                    logging.info("Exec application...'%s'" % (execcmd))
                    gtk.gdk.threads_enter()
                    self._scan_callback(CB_INFO_IND, {CB_ARG_MSG:FORMAT_PROGRESS_RUNAPP % (execcmd)})
                    gtk.gdk.threads_leave()
                    apcfg = apconf.gets(execcmd)
                    if apcfg is not None:
                        cmd = apcfg[OPT_COMMAND]
                        cmd = cmd.replace(EXEC_OPT_DIR, outputdir)
                        cmd = cmd.replace(EXEC_OPT_FILE, imgfilecv.listtostr(self.outputs))
                        cmd = cmd.strip()
                        if len(cmd) > 0:
                            logging.debug("command string='%s'" % (cmd))
                            try:
                                #os.system(cmd + " &")
                                os.stat(cmd.split()[0]) #check command exists
                                check_call(cmd + "&", shell=True)
                            except Exception, e:
                                s, m ,sm = msg_comp(MSG_APP_EXEC_NG, str(e))
                                gtk.gdk.threads_enter()
                                self._scan_callback(CB_ERRORMSG_IND, {CB_ARG_MSG:s})
                                gtk.gdk.threads_leave()
                                self._errui.error(MSG_APP_EXEC_NG, str(e))
                                return
                        else:
                            logging.warn("Command string is empty. app-config name='%s'" % (execcmd))
                            gtk.gdk.threads_enter()
                            self._scan_callback(CB_ERRORMSG_IND, {CB_ARG_MSG:MSG_APP_EXEC_NG})
                            gtk.gdk.threads_leave()
                    else:
                        logging.warn("not found. app-config name='%s'" % (execcmd))
                        gtk.gdk.threads_enter()
                        self._scan_callback(CB_ERRORMSG_IND, {CB_ARG_MSG:MSG_APP_EXEC_NG})
                        gtk.gdk.threads_leave()
            elif status == pyscand.E_CANCELED:
                logging.info("Scan cenceled.")
                #self._errui.error(None, None, status)
                self._scan_callback(CB_CANCELED_IND, {pyscand.PRM_STATUS:status})
            else:
                logging.info("Scan error. (status=%d)" %(status))
                gtk.gdk.threads_enter()
                self._scan_callback(CB_ERROR_IND, {pyscand.PRM_STATUS:status})
                gtk.gdk.threads_leave()
                self._errui.error(MSG_SCAN_ERROR, None, status)
                
        finally:
            scan_manager.delete_thread(self._device)
            logging.debug("finish ScanThread.")




class ScanThreadManageCtx:
    def __init__(self, devicename, presetname, cfg):
        self.devicename = devicename
        self.presetname = presetname
        self.presetcfg = cfg
        self.scanthread = None




CBPRM_START_THREAD="start-thread"
CBPRM_STOP_THREAD="stop-thread"

class ScanThreadManage:
    
    """Manage ScanThreads"""
    
    def __init__(self):
        self.__thrs__ = {}
        self.__lock__ = threading.Lock()
        self.__callback__ = None
        
        
    def list_devices(self):
        return self.__thrs__.keys()
    
    
    def set_callback(self, callback):
        self.__callback__ = callback
    
    
    def new_thread(self, device, presetname, cfg, callback=None, errui=None, pushscanid=0):
        global CBPRM_START_THREAD
        try:
            self.__lock__.acquire()
            try:
                if self.__thrs__.has_key(device):
                    logging.info("Now Scanning. device=%s" % device)
                    return pyscand.E_BUSY
                
                logging.info("new scan thread for device %s." % device)
                logging.debug("callback ---> " + str(callback))
                self.__thrs__[device] = ScanThreadManageCtx(device, presetname, cfg)
                self.__thrs__[device].scanthread = ScanThread(device, presetname, cfg, callback, errui, pushscanid)
                self.__thrs__[device].scanthread.start()
                if not self.__callback__ == None:
                    self.__callback__(CBPRM_START_THREAD, device)
                return pyscand.E_SUCCESS
            except Exception,e:
                logging.error(str(e))
                return pyscand.E_ERROR
        finally:
            self.__lock__.release()
    
    def delete_thread(self, device):
        global CBPRM_STOP_THREAD
        try:
            self.__lock__.acquire()
            if self.__thrs__.has_key(device):
                ctx = self.__thrs__.pop(device)
                pyscand.cancel_scan(device)
        finally:
            self.__lock__.release()
            if not self.__callback__ == None:
                self.__callback__(CBPRM_STOP_THREAD, device)
                
    def is_scanning(self, device):
        result = False
        try:
            self.__lock__.acquire()
            if self.__thrs__.has_key(device):
                result = True
        finally:
            self.__lock__.release()
        return result
        


# ScanThreadManage instance...
scan_manager = ScanThreadManage()


class _ScanDialog(CommonPalletWindow, ErrorMessageUI):
    def __init__(self, parent, title, presetname=None, dict=None, show_start_btn=True, devlist=None, actdev=None, width=SCAN_WIN_WIDTH, height=SCAN_WIN_HEIGHT):
        CommonPalletWindow.__init__(self, parent=parent, title=title)#, flags=gtk.DIALOG_NO_SEPARATOR)
        self.set_position(gtk.WIN_POS_CENTER)
        
        global __icon__
        self.set_icon_from_file(__icon__)
        
        self._presetname = presetname
        self._presetdict = dict
        self.__scanning = False
        self.__last_prog = None
        
        
        self.presets = ScanConfig()
        self.presets.load()
        
        
        self.liststore = gtk.ListStore(gtk.gdk.Pixbuf, str)
        self.presetcb = gtk.ComboBox(self.liststore)
        cellpb = gtk.CellRendererPixbuf()
        cellpb.set_fixed_size(64, 64)
        #cellpb.set_property('cell-background', 'white')
        cellstr = gtk.CellRendererText()
        #cellstr.set_property('cell-background', 'white')
        #cellstr.set_property('foreground', 'black')
        cellstr.set_property('wrap-mode', True)
        cellstr.set_property('wrap-width', COMBOBOX_PRESET_NAME_WIDTH)
        cellstr.set_fixed_size(COMBOBOX_PRESET_NAME_WIDTH, -1)
        self.presetcb.pack_start(cellpb, False)
        self.presetcb.pack_start(cellstr, False)
        self.presetcb.set_attributes(cellpb, pixbuf=0)
        self.presetcb.set_attributes(cellstr, text=1)
        f = gtk.HBox()
        f.pack_start(self.presetcb, False, padding=0)
        #f.pack_start(gtk.Label(None),True)
        self.vbox.pack_start(f, False, True, 0)
        
        pl = self.presets.sections_ordered()
        if len(pl) is not 0:
            #print "display order: %s" % (pl)
            iter1st = None
            for sec in pl:
                d = self.presets.gets(sec)
                icon = get_icon(d)
                iter = self.liststore.append([icon, sec])
                if presetname is not None and presetname == sec:
                    self.presetcb.set_active_iter(iter)
                    if iter1st is None:
                        iter1st = iter
                else:
                    if iter1st is None:
                        self.presetcb.set_active_iter(iter)
                        iter1st = iter
            
        else:
            self.presetcb.set_sensitive(False)
        
        if dict is not None:
            # dictが指定された場合は、変更不可
            self.presetcb.set_button_sensitivity(gtk.SENSITIVITY_OFF)
        
        
#        p = PresetItem(presetname, dict)
#        f = gtk.HBox()
#        f.pack_start(p, padding=4)
#        self.vbox.pack_start(f, False, True, 4)
        
        
        self.dev_liststore = gtk.ListStore(str)
        self.devicecb = gtk.ComboBox(self.dev_liststore)
        cellstr = gtk.CellRendererText()
        #cellstr.set_property('foreground', 'black')
        self.devicecb.pack_start(cellstr, False)
        self.devicecb.set_attributes(cellstr, text=0)
        if devlist is not None:
            self.set_devicecb(devlist, actdev)
        self.h_seldeb = gtk.HBox()
        self.h_seldeb.pack_start(gtk.Label(None),False,False,2)
        self.h_seldeb.pack_start(gtk.Label(LABEL_DEVICE+"  "),False, False)
        self.h_seldeb.pack_start(self.devicecb, False, False)
        self.h_seldeb.pack_start(gtk.Label(None),True)
        self.vbox.pack_start(self.h_seldeb, False, False, 2)
        
        
        self.msgview = MultiLineMsgView(font_size_pt=8)
        f = gtk.HBox()
        f.pack_start(self.msgview, padding=0)
        self.vbox.pack_start(f, True, True, 0)
        
        
        self.btn_start = gtk.Button(BUTTON_START_SCAN)
        self.btn_cancel = gtk.Button(BUTTON_CANCEL_SCAN)
        
        if show_start_btn:
            self.add_action_widget(self.btn_start, 1001)
            self.btn_cancel.set_sensitive(False)
        else:
            self.btn_cancel.set_sensitive(True)
        self.add_action_widget(self.btn_cancel, 1002)
        self.add_button(BUTTON_CLOSE, gtk.RESPONSE_CLOSE)
        
        self.set_response(1001, self.onStart)
        self.set_response(1002, self.onCancel)
        self.set_response(gtk.RESPONSE_DELETE_EVENT, self.onClose)
        
        self.devicecb.connect('changed', self._change_dev)
        self.presetcb.connect('changed', self._change_dev)
        
        self.resize(width, height)
        self.set_default_size(width, height)
        
        self.show_all()
        self.get_root_window().raise_()
    
    def _get_active_preset(self):
        presetname = None
        iter = self.presetcb.get_active_iter()
        if iter is not None:
            presetname = self.liststore.get_value(iter, 1)
        return presetname
    
    def _get_active_device(self):
        dev = None
        iter = self.devicecb.get_active_iter()
        if iter is not None:
            dev = self.dev_liststore.get_value(iter, 0)
        return dev
    
    def _finished(self):
        """call at the time of finished scanning."""
        self.btn_start.set_sensitive(True)
        self.btn_cancel.set_sensitive(False)
        self.h_seldeb.set_sensitive(True)
        self.presetcb.set_sensitive(True)
        self.__scanning = False
        
    def _change_dev(self, *args):
        #act = self.devicecb.get_active_text()
        act = self._get_active_device()
        if act == VALUE_NONE:
            self.btn_start.set_sensitive(False)
        else:
            if self._get_active_preset() is not None:
                self.btn_start.set_sensitive(True)
    
    def __error(self, msg=None, detail=None, status=0, show_msgview=False, mt=True):
        s, m, sm = msg_comp(msg, detail, status)
        if s is not None:
            if show_msgview:
                gtk_threads_call(mt, self.msgview.append, (markup_set_color(s, '#FF0000'),))
            error_dialog(retrieve_window(self), self.get_title(), m, sm, mt=mt)
    
    def error(self, msg=None, detail=None, status=0):
        """ErrorMessageUI method"""
        self.__error(msg, detail, status)
    
    def set_devicecb(self, devlist, actdev):
        #set_ComboBox(self.devicecb, devlist, actdev)
        iter1st = None
        for dev in devlist:
            iter = self.dev_liststore.append([dev])
            if actdev is not None and actdev == dev:
                self.devicecb.set_active_iter(iter)
            if iter1st is None:
                iter1st = iter
        if actdev is None:
            self.devicecb.set_active_iter(iter1st)
    
    def enable_devicecb(self, enable):
        self.h_seldeb.set_sensitive(enable)
    
    def set_selectable_devicecb(self, selectable):
        if selectable:
            self.devicecb.set_button_sensitivity(gtk.SENSITIVITY_ON)
        else:
            self.devicecb.set_button_sensitivity(gtk.SENSITIVITY_OFF)
    
    def onStart(self):
        logging.debug("Start")
        presetname = self._get_active_preset()
        dict = self.presets.gets(presetname)
        ##devname = self.devicecb.get_active_text()
        devname = self._get_active_device()
        self.msgview.append("%s - %s" % (devname, presetname))
        rc = scan_manager.new_thread(devname, presetname, dict, self.callback, self)
        if rc == pyscand.E_SUCCESS:
            #success
            self.btn_start.set_sensitive(False)
            self.btn_cancel.set_sensitive(True)
            self.h_seldeb.set_sensitive(False)
            self.presetcb.set_sensitive(False)
            
            self.__scanning = True
        else:
            self.__error(MSG_SCAN_START_NG, None, rc, True, False)
        return False
    
    def onCancel(self):
        logging.debug("Cancel")
        devname = self.devicecb.get_active_text()
        if self.__scanning and devname is not None:
            scan_manager.delete_thread(devname)
            self.__scanning = False
        return False
        
    def onClose(self):
        logging.debug("Close")
        self.onCancel()
        self.destroy()
        return True
    
    def callback(self, type, args):
        #logging.debug("CALLBACK")
        path = ""
        s = ""
        if type == pyscand.BEGIN:
            logging.debug("*** BEGIN")
            self.__scanning = True
            #status = args[pyscand.PRM_STATUS]
            #path = args[pyscand.PRM_PATH]
            #prog = args[pyscand.PRM_PROGRESS]
            #source = args[pyscand.PRM_SOURCE]
            page = args[pyscand.PRM_PAGE]
            #print "status=%s, prog=%f" % (pyscand.strstatus(status), prog)
#            gtk.gdk.threads_enter()
            s = FORMAT_PROGRESS_BEGIN % {FORMAT_PROGRESS_PAGENUMBER:page}
            if page == 1:
                self.msgview.append(s)
            else:
                self.msgview.update(s)
            self.__last_prog = None
#            gtk.gdk.threads_leave()
        elif type == pyscand.PROGRESS:
            status = args[pyscand.PRM_STATUS]
            #path = args[pyscand.PRM_PATH]
            prog = args[pyscand.PRM_PROGRESS]
            #source = args[pyscand.PRM_SOURCE]
            page = args[pyscand.PRM_PAGE]
            logging.debug("*** PROGRESS status=%s, prog=%f" % (pyscand.strstatus(status), prog))
            if status == pyscand.E_BUSY:
                #Wait for scan to begin / Scanning
                s = None
                if self.__last_prog is None or prog != self.__last_prog:
                    strprog = locale.format("%.1f", prog)
                    s = FORMAT_PROGRESS % {FORMAT_PROGRESS_PAGENUMBER:page, FORMAT_PROGRESS_PERCENTAGE:strprog}
#                    gtk.gdk.threads_enter()
#                    if self.__last_prog is None:
#                        self.msgview.append(s)
#                    else:
#                        self.msgview.update(s)
                    self.msgview.update(s)
                    self.__last_prog = prog
                    #self.progress.set_value(prog)
                    #self.progress.set_fraction(prog/100)
#                    gtk.gdk.threads_leave()
            elif status == pyscand.E_END_OF_PAGE or status == pyscand.E_SUCCESS:
                #Scan Complete
                pass
            elif status == pyscand.E_CANCELED:
#                gtk.gdk.threads_enter()
                s = FORMAT_PROGRESS_CANCELED % {FORMAT_PROGRESS_PAGENUMBER:page}
                self.msgview.update(s)
#                gtk.gdk.threads_leave()
            else:
                #Error
                pass
        elif type == pyscand.FINISHED:
            logging.debug("*** FINISHED")
            status = args[pyscand.PRM_STATUS]
            #path = args[pyscand.PRM_PATH]
            #prog = args[pyscand.PRM_PROGRESS]
            #source = args[pyscand.PRM_SOURCE]
            page = args[pyscand.PRM_PAGE]
#            gtk.gdk.threads_enter()
            s = None
            if status == pyscand.E_END_OF_PAGE or status == pyscand.E_SUCCESS:
                #Scan Complete
                s = FORMAT_PROGRESS_FINISHED % {FORMAT_PROGRESS_PAGENUMBER:page}
            elif status == pyscand.E_CANCELED:
                s = FORMAT_PROGRESS_CANCELED % {FORMAT_PROGRESS_PAGENUMBER:page}
            if s is not None:
                self.msgview.update(s)
            self._finished()
#            gtk.gdk.threads_leave()
        elif type == CB_INFO_IND:
            logging.debug("*** CB_INFO_IND")
            msg = args[CB_ARG_MSG]
#            gtk.gdk.threads_enter()
            self.msgview.append(msg)
            self._finished()
#            gtk.gdk.threads_leave()
        elif type == CB_ERROR_IND:
            logging.debug("*** CB_ERROR_IND")
            status = args[pyscand.PRM_STATUS]
            detail = pyscand.strstatus(status)
#            gtk.gdk.threads_enter()
            s = FORMAT_PROGRESS_FAIL % {FORMAT_PROGRESS_DETAIL:detail}
            self.msgview.append(markup_set_color(s, '#FF0000'))
            self._finished()
#            gtk.gdk.threads_leave()
        elif type == CB_ERRORMSG_IND:
            logging.debug("*** CB_ERRORMSG_IND")
            detail = args[CB_ARG_MSG]
#            gtk.gdk.threads_enter()
            s = FORMAT_PROGRESS_FAIL % {FORMAT_PROGRESS_DETAIL:detail}
            self.msgview.append(markup_set_color(s, '#FF0000'))
            self._finished()
#            gtk.gdk.threads_leave()
        elif type == CB_CANCELED_IND:
            logging.debug("*** CB_CANCELED_IND")
            status = args[pyscand.PRM_STATUS]
            detail = pyscand.strstatus(status)
#            gtk.gdk.threads_enter()
            self.msgview.append(detail)
            self._finished()
#            gtk.gdk.threads_leave()
        elif type == CB_OUTPUT_FILE:
            logging.debug("*** CB_OUTPUT_FILE")
            fl = args[CB_ARG_FILE]
            n = len(fl)
            if n > 2:
                s = fl[0] + ", ..., " + fl[n-1]
            elif n > 1:
                s = fl[0] + ", " + fl[n-1]
            else:
                s = fl[0]
#            gtk.gdk.threads_enter()
            self.msgview.append(s)
#            gtk.gdk.threads_leave()
        else:
            logging.debug("type=%s" % type)
        #print "*** finish callback"


class ScanDialog(_ScanDialog):
    # ScanDialog List
    dlgs = WindowList()
    
    def __init__(self, parent, presetname, dict):
        #parent, title, presetname, dict, show_start_btn=True, devlist=None, actdev=None, width=320, height=260
        _ScanDialog.__init__(self, parent, TITLE_SCAN, presetname, dict)
        # Set the help document URI for this window
        self.setHelpUri("file://"+UTLDOC_DIR+"/scantool-scan.html")
        
        # get device list
        devicelist = []
        rc = pyscand.get_device_list(devicelist)
        if rc == pyscand.E_SUCCESS:
            if len(devicelist) == 0:
                self.destroy()
                raise Exception(MSG_NO_DEVICES)
        elif rc == pyscand.E_CONNECTION_REFUSED:
            self.destroy()
            raise Exception(MSG_NOT_WORK_DAEMON)
        elif rc == pyscand.E_IOERROR:
            self.destroy()
            raise Exception(MSG_DAEMON_ACCESS_NG)
        else:
            self.destroy()
            raise Exception("%s (%s)" % (MSG_ERROR, pyscand.strstatus(rc)))
        devlist = [VALUE_NONE] + pyscandsupp.to_device_name_list(devicelist)
        self.set_devicecb(devlist, VALUE_NONE)
        
        self.msgview.append(MSG_START_SCAN)
        
        ScanDialog.dlgs.append(self)
    
    def onClose(self):
        _ScanDialog.onClose(self)
        ScanDialog.dlgs.remove(self)
        return True




class ConfigList(gtk.VBox):
    """Scan configuration list"""
    
    def _tv_row_activated(self, *args):
        # when double clicked
        self.start_active_preset()
    
    def _tv_btn_rel(self, widget, signal_id, *args):
        if not self._ctxmenu == None:
            if signal_id.button == 3:   #right button clicked
                self._ctxmenu.popup(None, None, None, signal_id.button, 0)
        
    def _ls_row_changed(self, treemodel, path, iter, *args):
        # 移動先のインデックスが渡される
        # このコールバックが呼ばれているときは、移動元と移動先の両方が存在している。
        if self._suppress:
            return
        #print treemodel
        #print path
        #print iter
        #print args
        presetname = self.liststore.get(iter, 1)[0]
        #print "row_changed: %s" % (presetname)
        i = self.liststore.get_iter_root()
        n = 0
        while i != None:
            t = self.liststore.get(i, 1)[0]
            if t == presetname:
                p = self.liststore.get_path(i)
                if p != path:
                        #移動先のパスと異なるので除外
                    i = self.liststore.iter_next(i)
                    continue
            #print "PRESET#%02d: %s" % (n, t)
            self.presets.set(t, OPT_ORDER, n) #update display order
            n = n + 1
            i = self.liststore.iter_next(i)
        self.save()
         
    
    def _cb_changed(self, widget, path, text, model):
        model[path][4] = text
        preset = model[path][1]
        self.presets.set(preset, OPT_DEVICE, text)
        self.presets.save()
    
    def __init__(self, menu=None):
        gtk.VBox.__init__(self)
        self._ctxmenu = menu
        self._suppress = False
        
        self.scrollwin = gtk.ScrolledWindow()
        
        self.liststore = gtk.ListStore(gtk.gdk.Pixbuf, str)
        
        self.tv = gtk.TreeView(self.liststore)
        self.tv.set_property('headers-visible', False)
        self.tv.set_property('enable-grid-lines', False)
        self.tv.set_property('enable-tree-lines', False)
        
        
        self.tvcolumn1 = gtk.TreeViewColumn("Col1")
        self.tv.append_column(self.tvcolumn1)
        
        
        self.cellpb = gtk.CellRendererPixbuf()
        self.cellstr = gtk.CellRendererText()
        self.cellpb.set_fixed_size(64, 64)
        #self.cellstr.set_property('editable', True)
        #self.cellstr.set_property('cell-background', 'cyan')
        #self.cellstr.set_property('foreground', 'black')
        self.cellstr.set_property('wrap-mode', True)
        self.cellstr.set_property('wrap-width', COMBOBOX_PRESET_NAME_WIDTH)
        self.tvcolumn1.pack_start(self.cellpb, False)
        self.tvcolumn1.pack_start(self.cellstr, True)
        self.tvcolumn1.set_attributes(self.cellpb, pixbuf=0)
        self.tvcolumn1.set_attributes(self.cellstr, text=1)
        self.tvcolumn1.set_resizable(True)
        self.tvcolumn1.set_property('clickable', True)#If True, the header can be clicked
        #self.tvcolumn1.connect('clicked', self._tvcol_clicked)
        
        
        self.tv.set_reorderable(True)
        #self.tv.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_BOTH)
        #self.tv.set_property('fixed-height-mode', True)
        
        
        self.presets = ScanConfig()
        self.presets.load()
        
        pl = self.presets.sections_ordered()
        #print "display order: %s" % (pl)
        for sec in pl:
            cfgdict = self.presets.gets(sec)
            self.__append_preset(sec, cfgdict)
        self.tv.set_cursor((0,))

        self.scrollwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scrollwin.add(self.tv)
        
        self.pack_start(self.scrollwin)
        
        self.liststore.connect('row-changed', self._ls_row_changed)#use
        #self.liststore.connect('rows-reordered', self._ls_rows_reordered)
        #self.tv.connect('columns-changed', self._tv_columns_changed)
        #self.tv.connect('cursor-changed', self._tv_cursor_changed)#use
        #self.tv.connect('select-cursor-row', self._tv_select_cursor_row)
        #self.tv.connect('toggle-cursor-row', self._tv_toggle_cursor_row)
        self.tv.connect('row-activated', self._tv_row_activated)
        self.tv.connect('button-release-event', self._tv_btn_rel)
        
        
    def __append_preset(self, presetname, cfgdict):
        icon = get_icon(cfgdict)
        iter = self.liststore.append([icon, presetname])
        path = self.liststore.get_path(iter)
        self.tv.set_cursor(path)

    def __delete_preset(self, presetname):
        ite = self.liststore.get_iter_root()
        while ite != None:
            pn = self.liststore.get(ite, 1)[0]
            logging.debug(pn)
            if pn == presetname:
                self.liststore.remove(ite)
                break
            ite = self.liststore.iter_next(ite)

    def __update_preset(self, old_presetname, new_presetname, cfgdict):
        for row in self.liststore:
            if row[1] == old_presetname:
                row[0] = get_icon(cfgdict)
                row[1] = new_presetname

    def start_active_preset(self):
        (path, column) = self.tv.get_cursor()
        logging.debug("path=%s, column=%s" % (path, column))
        if not path == None:
            row = self.liststore[path[0]]
            pname = row[1]
            pcfg = self.presets.gets(pname)
            try:
                #scandlg = ScanDialog(retrieve_window(self), pname, pcfg)
                #scandlg = ScanDialog(retrieve_window(self), pname, None)
                scandlg = ScanDialog(None, pname, None)
                scandlg.show_all()
            except Exception, e:
                error_dialog(retrieve_window(self), TITLE_SCAN, unicode(e))
            return pname
        return None
        
    
    def number_of_preset(self):
        return len(self.presets.sections())

    def get_active_preset(self):
        (path, column) = self.tv.get_cursor()
        if path != None:
            presetname = self.liststore[path][1]
            cfgdict = self.presets.gets(presetname)
            return (presetname, cfgdict)
        return None
        
    def delete_active_preset(self):
        (path, column) = self.tv.get_cursor()
        if path != None:
            presetname = self.liststore[path][1]
            self.presets.delete(presetname)
            self.presets.save()
            self.__delete_preset(presetname)
        
    def add_preset(self, presetname, cfgdict):
        self.presets.add(presetname, cfgdict)
        self.presets.save()
        self._suppress = True
        self.__append_preset(presetname, cfgdict)
        self._suppress = False
        
        
    def modify_preset(self, old_presetname, new_presetname, cfgdict):
        if old_presetname != new_presetname:
            self.presets.rename(old_presetname, new_presetname)
        self.presets.add(new_presetname, cfgdict)#update
        self.presets.save()
        self._suppress = True
        self.__update_preset(old_presetname, new_presetname, cfgdict)
        self._suppress = False
        
    
    def save(self):
        self.presets.save()









class PaperSizeConfigPanel(ItemHBox):
    def __init__(self):
        #self.cb = build_ComboBox(papersize.get_displaynamelist(__papersizes__))
        self.cb = gtk.ComboBox()
        self.liststore = gtk.ListStore(str, str)
        cell = gtk.CellRendererText()
        self.cb.pack_start(cell)
        self.cb.add_attribute(cell, 'text', 0)
        cell = gtk.CellRendererText()
        self.cb.pack_start(cell)
        self.cb.add_attribute(cell, 'text', 1)
        for p in __papersizes__:
            name = p.displayname()
            descr = "(" + p.sizestr() + ")"
            self.liststore.append([name, descr])
        self.cb.set_model(self.liststore)
        #self.cb.set_row_span_column(1)
        #self.cb.set_column_span_column(2)
        self.cb.set_active(0)
        ItemHBox.__init__(self, SPN_PAPERSIZE, self.cb, False)
        
    def get_papersize(self):
        """return pagetypenam."""
        act = self.cb.get_active_text()
        p = papersize.get_papersize_by_displayname(act)
        return p.pagetypename()
        
    def set_papersize(self, pagetypename):
        logging.debug("set_papersize: pagetypename='%s'" % (pagetypename))
        p = papersize.get_papersize_by_pagetypename(pagetypename)
        if p == None:
            logging.debug("invalid pagetypename. pagetypename='%s'" % (pagetypename))
            return #invalid
        set_active_ComboBox(self.cb, p.displayname())




class AppConfigPanel(gtk.VBox):
    def _sens_app_cfg(self, f):
        self.app.set_sensitive(f)
        
    def _change_use_app(self, *args):
        if self.use_app.get_active():
            self._sens_app_cfg(True)
        else:
            self._sens_app_cfg(False)

    def __init__(self):
        gtk.VBox.__init__(self, False, 0)
        self.use_app = gtk.CheckButton(SPV_CK_EXECAPP)
        self.use_app.connect('toggled', self._change_use_app)
        self.pack_start(self.use_app, False)
        self.iapp = appconfig.ItemApplication()
        self.app = ItemHBox(SPN_APPLICATION, self.iapp, True)
        self.pack_start(self.app, False)
        self._sens_app_cfg(False)
        
    def get(self):
        if self.use_app.get_active():
            return self.iapp.get_active()
        else:
            return None
        
    def set(self, appconfname):
        if appconfname == None or len(appconfname) == 0:
            self.use_app.set_active(False)
            self._sens_app_cfg(False)
        else:
            b = self.iapp.set_active(appconfname)
            if b:
                if not self.use_app.get_active():
                    self.use_app.set_active(True)
                self._sens_app_cfg(True)
            else:
                self.use_app.set_active(False)
                self._sens_app_cfg(False)
    
    def check_values(self):
        if self.use_app.get_active():
            self.iapp.check_values()


class ScanConfigPanel(gtk.VBox):
    def __init__(self, presetname, cfgdict):
        gtk.VBox.__init__(self, False, 0)
        self._suppress = True
        
        self._presetname = presetname
        self._cfgdict = cfgdict

        # Configuration Name(Title)
        self.text_cn = gtk.Entry()
        self.config_name = ItemHBox(SPN_PRESETNAME, self.text_cn, True)
        self.pack_start(self.config_name,False)
        
        
        # Scan Settings
        f = VBoxFrame(label=LABEL_SCANSETTING)
        self.pack_start(f, False, padding=2)
        
        self.cb_scansrc = build_ComboBox((SPV_AUTO,SPV_FLATBED,SPV_ADF))
        self.cb_scansrc.connect('changed', self._change_scan_source)
        self.h1 = ItemHBox(SPN_SCANSRC, self.cb_scansrc)
        f.pack_start(self.h1, False)
        
        self.cb_dupscan = build_ComboBox((SPV_DUPSCAN_NONE,SPV_DUPSCAN_RLB,SPV_DUPSCAN_TB))
        self.h1o1 = ItemHBox(SPN_DUPSCAN, self.cb_dupscan)
        f.pack_start(self.h1o1, False)
        self._change_scan_source(self.cb_dupscan)
        
        self.cb_scanmode = build_ComboBox((SPV_SCANMODE_COLOR,SPV_SCANMODE_GRAY,SPV_SCANMODE_BW))
        self.h2 = ItemHBox(SPN_SCANMODE, self.cb_scanmode)
        f.pack_start(self.h2, False)
        
        self.item_papersize = PaperSizeConfigPanel()
        f.pack_start(self.item_papersize, False)
        
        adjustment = gtk.Adjustment(value=DEF_SCAN_RES, lower=MIN_SCAN_RES, upper=MAX_SCAN_RES, step_incr=STP_SCAN_RES)
        self.sb_scanres = gtk.SpinButton(adjustment)
        self.h4 = ItemHBox(SPN_SCANRES, self.sb_scanres)
        f.pack_start(self.h4, False)
        
        
        # File Format & Output Directory Settings
        f = VBoxFrame(label=LABEL_OUTPUTSETTING)
        self.pack_start(f, False, padding=2)
        
        extlst = [EXT_PNM,EXT_PNG,EXT_JPG,EXT_TIF,EXT_PDF]
        #for ext in [EXT_PNM,EXT_PNG,EXT_JPG,EXT_TIF,EXT_PDF]:
        #    if imgfilecv.can_convert_to(ext):
        #        extlst.append(ext)
        self.cb_filetype = build_ComboBox(extlst)
        self.cb_filetype.connect('changed', self._chg_cb_filetype)
        self.filetype = ItemHBox(SPN_OUTFMT, self.cb_filetype)
        f.pack_start(self.filetype, False)
        
        self.item_outpath = ItemPath()
        self.outdir = ItemHBox(SPN_OUTDIR, self.item_outpath, True)
        f.pack_start(self.outdir, False)
        
        self.text_outfile = gtk.Entry()
        self.outfile = ItemHBox(SPN_OUTFILE, self.text_outfile, True)
        f.pack_start(self.outfile,False)
        
        
        # Application Settings
        f = VBoxFrame(label=LABEL_APPSETTING)
        self.pack_start(f, False, padding=2)
        
        self.item_appcfg = AppConfigPanel()
        f.pack_start(self.item_appcfg, False)
        
        
        # Icon Settings
        f = VBoxFrame(label=LABEL_ICONSETTING)
        self.pack_start(f, False, padding=2)
        
        self.cb_chooseicon = build_ComboBox((SPV_ICONIMG_TOFOLDER,SPV_ICONIMG_TOAPP,SPV_ICONIMG_CUSTOM))
        self.selicon = ItemHBox(SPN_ICONIMG, self.cb_chooseicon)
        f.pack_start(self.selicon, False)
        self.cb_chooseicon.connect('changed', self._chg_cb_chooseicon)
        
        self.iconpath = ItemFile()
        self.toolicon = ItemHBox(SPN_CUSTOM_ICONIMG, self.iconpath, True)
        self.toolicon.set_sensitive(False)
        f.pack_start(self.toolicon, False)
        
        
        self.set_values(presetname, cfgdict)
        
        self.show_all()
    
    
    def _chg_cb_chooseicon(self, *args):
        text = self.cb_chooseicon.get_active_text()
        if text == SPV_ICONIMG_CUSTOM:
            self.toolicon.set_sensitive(True)
        else:
            self.toolicon.set_sensitive(False)

    def _chg_cb_filetype(self, *args):
        w = retrieve_window(self)
        if w == None:
            return
        if w.is_shown():
            self._check_filetype()

    def _change_scan_source(self, combobox):
        idx = self.h1.item.get_active()
        #print idx
        if idx != 1:
            self.h1o1.set_sensitive(True)
        else:
            self.h1o1.set_sensitive(False)
            

    def _check_filetype(self):
        fmtext = self.cb_filetype.get_active_text()
        reqpkg = []
        if not imgfilecv.can_convert_to(fmtext, reqpkg):
            msg = MSG_WRN_OFILEFMT % {MSG_WRN_OFILEFMT_EXT:fmtext}
            pkgs = ""
            for pkg in reqpkg:
                if len(pkgs) > 0:
                    pkgs = pkgs + "\n"
                pkgs = pkgs + pkg
            warning_dialog(retrieve_window(self), TITLE_SCAN_CONFIG, msg, pkgs)

    def check_values(self):
        self._check_filetype()
        self.item_appcfg.check_values()
    

    def set_values(self, presetname, cfgdict):
        #print presetname
        #print cfgdict
        if presetname != None:
            self.text_cn.set_text(presetname)

        if cfgdict != None:
            k = OPT_SCANSRC
            if cfgdict.has_key(k):
                self.cb_scansrc.set_active(cfgdict[k])
            
            k = OPT_DUPSCAN
            if cfgdict.has_key(k):
                self.cb_dupscan.set_active(cfgdict[k])
            
            k = OPT_SCANMODE
            if cfgdict.has_key(k):
                self.cb_scanmode.set_active(cfgdict[k])
            
            k = OPT_PAPERSIZE
            if cfgdict.has_key(k):
                self.item_papersize.set_papersize(cfgdict[k])
            
            k = OPT_SCANRES
            if cfgdict.has_key(k):
                self.sb_scanres.set_value(cfgdict[k])
            
            k = OPT_OUTFMT
            if cfgdict.has_key(k):
                set_active_ComboBox(self.cb_filetype, cfgdict[k])
            
            k = OPT_OUTDIR
            if cfgdict.has_key(k):
                self.item_outpath.set_path(cfgdict[k])
            else:
                self.item_outpath.set_path(os.path.expanduser(SPV_DEF_OUTDIR))
            
            k = OPT_OUTFILE
            if cfgdict.has_key(k):
                self.text_outfile.set_text(cfgdict[k])
            else:
                self.text_outfile.set_text(SPV_DEF_OUTFILE)
            
            k = OPT_EXEAPP
            if cfgdict.has_key(k):
                self.item_appcfg.set(cfgdict[k])
            
            k = OPT_ICON
            if cfgdict.has_key(k):
                p = cfgdict[k]
                if p == BUILTIN_ICON_TOFOLDER:
                    self.cb_chooseicon.set_active(0)
                    self.iconpath.set_path("")
                    self.toolicon.set_sensitive(False)
                elif p == BUILTIN_ICON_TOAPP:
                    self.cb_chooseicon.set_active(1)
                    self.iconpath.set_path("")
                    self.toolicon.set_sensitive(False)
                else:
                    #custom
                    self.cb_chooseicon.set_active(2)
                    self.iconpath.set_path(p)
                    self.toolicon.set_sensitive(True)
            else:
                self.cb_chooseicon.set_active(0)
                self.iconpath.set_path("")
                self.toolicon.set_sensitive(False)
        else:
            self.item_outpath.set_path(os.path.expanduser(SPV_DEF_OUTDIR))
            self.text_outfile.set_text(SPV_DEF_OUTFILE)
            self.cb_chooseicon.set_active(0)
            self.iconpath.set_path("")
            self.toolicon.set_sensitive(False)


    def get_values(self):
        """return (presetname, cfgdict)."""
        presetname = self.text_cn.get_text()
        
        cfgdict = {}
        cfgdict[OPT_SCANSRC] = self.cb_scansrc.get_active()
        cfgdict[OPT_DUPSCAN] = self.cb_dupscan.get_active()
        cfgdict[OPT_SCANMODE] = self.cb_scanmode.get_active()
        cfgdict[OPT_PAPERSIZE] = self.item_papersize.get_papersize()
        cfgdict[OPT_SCANRES] = self.sb_scanres.get_value()
        cfgdict[OPT_OUTFMT] = self.cb_filetype.get_active_text()
        cfgdict[OPT_OUTDIR] = self.item_outpath.get_path()
        cfgdict[OPT_OUTFILE] = self.text_outfile.get_text()
        cfgdict[OPT_EXEAPP] = self.item_appcfg.get()
        i = self.cb_chooseicon.get_active()
        if i == 0:
            cfgdict[OPT_ICON] = BUILTIN_ICON_TOFOLDER
        elif i == 1:
            cfgdict[OPT_ICON] = BUILTIN_ICON_TOAPP
        else:
            cfgdict[OPT_ICON] = self.iconpath.get_path()
        
        return (presetname, cfgdict)







class ConfigDialog(CommonPalletWindow):
    
    """ConfigDialog"""
    
    def onShow(self):
        self.scancfg_panel.check_values()
    
    def onOk(self):
        logging.debug("onOk modify_mode=%s" % (self._modify_mode))
        self._result = self.scancfg_panel.get_values()
        if self._result == None:
            return False
        
        (presetname, cfgdict) = self._result
        if self._modify_mode:
            #modify
            if self._in_presetname != presetname:
                #change presetname
                if self._presets.has_section(presetname):
                    # already exists preset
                    error_dialog(retrieve_window(self), TITLE_SCAN_CONFIG, MSG_SCANPRESETNAME_EXISTED, presetname)
                    return False
                
                pushconf = PushScanConfig()
                pushconf.load()
                if pushconf.in_used(self._in_presetname):
                    # this Preset is used by PushScan
                    error_dialog(retrieve_window(self), TITLE_SCAN_CONFIG, MSG_SCANCFG_MOD_WRN, self._in_presetname)
                    return False
        else:
            #not modify
            if self._presets.has_section(presetname):
                # already exists preset
                error_dialog(retrieve_window(self), TITLE_SCAN_CONFIG, MSG_SCANPRESETNAME_EXISTED, presetname)
                return False
        
        return True

    
    def __init__(self, parentwin, presets, presetname, cfgdict):
        CommonPalletWindow.__init__(self, parent=parentwin, flags=gtk.DIALOG_NO_SEPARATOR, buttons=(BUTTON_OK, gtk.RESPONSE_OK, BUTTON_CANCEL, gtk.RESPONSE_CANCEL))
        self.set_title(TITLE_SCAN_CONFIG)
        # Set the help document URI for this window
        self.setHelpUri("file://"+UTLDOC_DIR+"/scantool-config.html")
        
        #self.connect("delete-event", gtk.main_quit)
        self._modify_mode = False
        if cfgdict != None:
            self._modify_mode = True
        self._presets = presets
        self._in_presetname = presetname
        self._in_cfgdict = cfgdict
        self._result = None
        
        
        self.scancfg_panel = ScanConfigPanel(presetname, cfgdict)
        self.vbox.pack_start(self.scancfg_panel)
         
        
        self.resize(320, 400)
        #self.show_all()

        
    def result(self):
        #print self._result
        return self._result




class ScanPresetDialog(CommonPalletWindow):
    """Scan Tool Window."""
    
    def _quit(self, *args):
        #print args
        if ScanDialog.dlgs.count(self) > 0:
            error_dialog(None, TITLE_SCAN_TOOL, MSG_SCAN_TOOL_QUIT_WRN)
            return False
        
        self.hide_all()
        self.destroy()
        return True
    
    
    def _start_scan(self, *args):
        presetname = self.cfg_sel.start_active_preset()
        if presetname == None:
            error_dialog(retrieve_window(self), TITLE_SCAN_TOOL, MSG_PRESET_NOT_SELECT)

    
    
    def _add(self, menuitem, *args):
        n = self.cfg_sel.number_of_preset()
        if n >= MAX_SCAN_CONFIG_PRESET:
            error_dialog(retrieve_window(self), TITLE_SCAN_TOOL, MSG_SCANCFG_REGMAX)
            return
        try:
            dlg = ConfigDialog(self, self.cfg_sel.presets, SPV_DEF_PRESETNAME, None)
            rc = dlg.do_modal()
            if rc == gtk.RESPONSE_OK:
                (presetname, cfgdict) = dlg.result()
                self.cfg_sel.add_preset(presetname, cfgdict)
                self.submenu_edit_modify.set_sensitive(True)
                self.submenu_edit_delete.set_sensitive(True)
                self.submenu_scan_start.set_sensitive(True)
                self.mi_modify.set_sensitive(True)
                self.mi_del.set_sensitive(True)
                self.mi_scan_start.set_sensitive(True)
            dlg.destroy()
        finally:
            pass
    
    def _delete(self, menuitem, *args):
        p = self.cfg_sel.get_active_preset()
        if p == None:
            error_dialog(retrieve_window(self), TITLE_SCAN_TOOL, MSG_PRESET_NOT_SELECT)
            return
        (presetname, cfgdict) = p
        
        pushconf = PushScanConfig()
        pushconf.load()
        if pushconf.in_used(presetname):
            # this Preset is used by PushScan
            error_dialog(retrieve_window(self), TITLE_SCAN_CONFIG, MSG_SCANCFG_DEL_WRN, presetname)
            return
        
        rc = confirm_dialog(retrieve_window(self), TITLE_SCAN_CONFIG, MSG_CONFIRM_DEL_SCANPRESET, presetname)
        if rc == gtk.RESPONSE_OK:
            self.cfg_sel.delete_active_preset()
            if self.cfg_sel.number_of_preset() == 0:
                self.submenu_edit_modify.set_sensitive(False)
                self.submenu_edit_delete.set_sensitive(False)
                self.submenu_scan_start.set_sensitive(False)
                self.mi_modify.set_sensitive(False)
                self.mi_del.set_sensitive(False)
                self.mi_scan_start.set_sensitive(False)
    
    def _modify(self, menuitem, *args):
        p = self.cfg_sel.get_active_preset()
        if p == None:
            error_dialog(retrieve_window(self), TITLE_SCAN_TOOL, MSG_PRESET_NOT_SELECT)
            return
        (presetname, cfgdict) = p
        try:
            dlg = ConfigDialog(self, self.cfg_sel.presets, presetname, cfgdict)
            rc = dlg.do_modal()
            if rc == gtk.RESPONSE_OK:
                #print "Modify OK"
                (new_presetname, cfgdict) = dlg.result()
                self.cfg_sel.modify_preset(presetname, new_presetname, cfgdict)
            dlg.destroy()
        finally:
            pass
    
    def __init__(self):
        CommonPalletWindow.__init__(self, button_area=False, menubar=True, padding=0)
        self.set_title(TITLE_SCAN_TOOL)
        # Set the help document URI for this window
        self.setHelpUri("file://"+UTLDOC_DIR+"/scantool.html")
        
        self.set_icon_from_file(__icon__)
        #self.connect("delete-event", gtk.main_quit)
        #self.connect('response', self._response)
        self.__count_clddlg = 0
        
        
        # Context Menu
        self.ctxmenu = gtk.Menu()
        self.mi_scan_start = gtk.MenuItem(MENUITEM_SCAN_START)
        self.mi_scan_start.connect('activate', self._start_scan)
        self.ctxmenu.append(self.mi_scan_start)
        self.ctxmenu.append(gtk.SeparatorMenuItem())
        self.mi_add = gtk.MenuItem(MENUITEM_EDIT_ADD)
        self.mi_add.connect('activate', self._add)
        self.ctxmenu.append(self.mi_add)
        self.mi_modify = gtk.MenuItem(MENUITEM_EDIT_MODIFY)
        self.mi_modify.connect('activate', self._modify)
        self.ctxmenu.append(self.mi_modify)
        self.mi_del = gtk.MenuItem(MENUITEM_EDIT_DELETE)
        self.mi_del.connect('activate', self._delete)
        self.ctxmenu.append(self.mi_del)
        self.ctxmenu.show_all()
        
        
        # Menu Bar
        self.menu_file = gtk.MenuItem(MENUITEM_FILE)
        self.menu_edit = gtk.MenuItem(MENUITEM_EDIT)
        self.menu_scan = gtk.MenuItem(MENUITEM_SCAN)
        self.menubar.append(self.menu_file)
        self.menubar.append(self.menu_edit)
        self.menubar.append(self.menu_scan)
        
        self.submenu_file = gtk.Menu()
        self.submenu_file_quit = gtk.MenuItem(MENUITEM_FILE_QUIT)
        self.submenu_file.append(self.submenu_file_quit)
        self.menu_file.set_submenu(self.submenu_file)
        self.submenu_file_quit.connect('activate', self._quit)
        
        self.submenu_scan = gtk.Menu()
        self.submenu_scan_start = gtk.MenuItem(MENUITEM_SCAN_START)
        self.submenu_scan.append(self.submenu_scan_start)
        self.menu_scan.set_submenu(self.submenu_scan)
        self.submenu_scan_start.connect('activate', self._start_scan)
        
        self.submenu_edit = gtk.Menu()
        self.submenu_edit_add = gtk.MenuItem(MENUITEM_EDIT_ADD)
        self.submenu_edit_modify = gtk.MenuItem(MENUITEM_EDIT_MODIFY)
        self.submenu_edit_delete = gtk.MenuItem(MENUITEM_EDIT_DELETE)
        self.submenu_edit.append(self.submenu_edit_add)
        self.submenu_edit.append(self.submenu_edit_modify)
        self.submenu_edit.append(self.submenu_edit_delete)
        self.menu_edit.set_submenu(self.submenu_edit)
        self.submenu_edit_add.connect('activate', self._add)
        self.submenu_edit_modify.connect('activate', self._modify)
        self.submenu_edit_delete.connect('activate', self._delete)
        
        
        self.cfg_sel = ConfigList(menu=self.ctxmenu)
        self.vbox.pack_start(self.cfg_sel)
        
        
        if self.cfg_sel.number_of_preset() == 0:
            self.submenu_edit_modify.set_sensitive(False)
            self.submenu_edit_delete.set_sensitive(False)
            self.submenu_scan_start.set_sensitive(False)
            self.mi_modify.set_sensitive(False)
            self.mi_del.set_sensitive(False)
            self.mi_scan_start.set_sensitive(False)
        
        self.set_response(gtk.RESPONSE_DELETE_EVENT, self._quit)
        
        self.set_default_size(SCANTOOL_WIN_WIDTH, SCANTOOL_WIN_HEIGHT)
        self.resize(SCANTOOL_WIN_WIDTH, SCANTOOL_WIN_HEIGHT)




if __name__ == "__main__":
    import getopt
    logging.basicConfig(format=LOGFORMAT)
    opts, args = getopt.getopt(sys.argv[1:], 'l:', ['loglevel=', 'no-pynotify'])
    #print "args=", args
    #print "opts=", opts
    for k, v in opts:
        if k in ('-l', '--loglevel'):
            rl = logging.getLogger()
            rl.setLevel(LOGLEVELS[v])
            if LOGLEVELS[v] is logging.DEBUG:
                #gc.set_debug(gc.DEBUG_COLLECTABLE | gc.DEBUG_UNCOLLECTABLE | gc.DEBUG_INSTANCES | gc.DEBUG_OBJECTS | gc.DEBUG_SAVEALL)
                gc.set_debug(gc.DEBUG_LEAK)
        elif k in ('--no-pynotify'):
            disable_pynotify()
    
    genconfig.configure()
    dlg = ScanPresetDialog()
    dlg.gtkmain()


